package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.TreeSet;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import listener.CleverMouseListener;
import listener.MouseInteractionObject;

import fuzzy.FuzzyRule;
import guiCore.FuzzyRuleCollection;
import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.DropTarget;
import guiUtil.Interaction;
import guiUtil.InteractionField;
import guiUtil.NumberFormater;

public class AndrewsView extends RuleVisualizer implements ChangeListener, DropTarget{
	
	private int rule_Width = 20;
	private int rule_Height = 20;	
	private int padding_H = 10;
	private int padding_V = 5;
	
	private int block_width = 30;
	private int block_height = 12;
	
	private int dragedIndex = -1;
	
	private double start = 0;
	private double end = Math.PI*4; 
	private int resolution = 500;	
	
	private double y_scale = 2;
	private double x_scale = 50;
	
	private int x_trans = 70;
	private int y_trans = 250;
	private int startX = 0;
	private int startY = 0;
	
	private double maxValue = 0;
	
	private JSlider y_Slider;
	private JPanel y_SliderPanel;
	
	private JSlider x_Slider;
	private JPanel x_SliderPanel;
	
	private JPanel sliderPanel;
	
	private DrawPanel rulePanel;
	private CleverMouseListener rulePanelMouseListener = new CleverMouseListener();
	
	private ArrayList<Integer> attr_Permutation = new ArrayList<Integer>();
	
	private AffineTransform ruleTransform = new AffineTransform();
	
	private ArrayList<ParallelAxisViewRule> pavr_List = new ArrayList<ParallelAxisViewRule>();
	private ArrayList<RuleAndColor> currFr_List = new ArrayList<RuleAndColor>();
	
	//###############################################################################################
	// Constructor
	//###############################################################################################

	public AndrewsView(Session session_P) 
	{
		super(session_P, new DrawPanel());
		
		drawPanel.setBackground(Color.white);
		
		cm_Listener.addToPressedList(this);
		cm_Listener.addToDraggedList(this);
		
		y_Slider = new JSlider(JSlider.VERTICAL
							,0
							,1000
							,(int)(y_scale*100));
		y_Slider.setName("yChange");
		y_Slider.addChangeListener(this);
		y_Slider.setPreferredSize(new Dimension(20, 400));
		
		y_SliderPanel = new JPanel();
		y_SliderPanel.setLayout(new  BorderLayout());
		y_SliderPanel.add(y_Slider, BorderLayout.CENTER);
		
		x_Slider = new JSlider(JSlider.VERTICAL
				,0
				,2000
				,(int)(x_scale*10));
		x_Slider.setName("xChange");
		x_Slider.addChangeListener(this);
		x_Slider.setPreferredSize(new Dimension(20, 400));
		
		x_SliderPanel = new JPanel();
		x_SliderPanel.setLayout(new  BorderLayout());
		x_SliderPanel.add(x_Slider, BorderLayout.CENTER);
		
		sliderPanel = new JPanel();
		sliderPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		sliderPanel.add(y_SliderPanel);
		sliderPanel.add(x_SliderPanel);
		
		rulePanel = new DrawPanel();
		rulePanel.setPreferredSize(new Dimension(200, 80));
		rulePanel.addMouseMotionListener(rulePanelMouseListener);
		rulePanel.addMouseListener(rulePanelMouseListener);
		
		mainPanel.add(sliderPanel, BorderLayout.WEST);
		mainPanel.add(rulePanel, BorderLayout.SOUTH);
		
		for(int i=0; i<session.getFr_premiseSize(); i++)
		{
			attr_Permutation.add(i);
		}
		
		this.ruleTransform.translate(0, block_height + padding_V);
		
		DrawAttrBlock dab = new DrawAttrBlock();
		rulePanel.addDrawObject(dab);
		rulePanelMouseListener.addToPressedList(dab);
		rulePanelMouseListener.addToReleasedList(dab);
	}
	
	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################


	@Override
	public boolean action(MouseEvent me, int eventType) 
	{
		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
												
					startX = me.getX();
					startY = me.getY();

				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED: break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	
				x_trans = x_trans-(startX-me.getX());
				y_trans = y_trans-(startY-me.getY());
				
				startX = me.getX();
				startY = me.getY();
				break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;
		}
		drawPanel.repaint();
		
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) 
	{
		return true;
	}
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	protected void updateVisualisation()
	{
		currFr_List.clear();
		for(ParallelAxisViewRule pavr: pavr_List)
		{			
			int birth = session.getRuleInfoAt(pavr.getId()).getBirth();
			int death = session.getRuleInfoAt(pavr.getId()).getDeath() - 1;
			int gen = this.frCol.getGeneration();
			
			if(gen > death && death > -1)
			{
				gen = death;
			}
			else if(gen < birth)
			{
				gen = birth;
			}
			
			FuzzyRule fr = session.getFuzzyRule(pavr.getId(), gen);
					
			currFr_List.add(new RuleAndColor(fr,
									pavr.getColor()));
			
			for(int i=0; i<pavr.getNumStatics(); i++)
			{
				currFr_List.add(new RuleAndColor(session.getFuzzyRule(pavr.getId(), pavr.getStaticGenAt(i)),
									pavr.getSubColor(i)));
			}
			
			for(int i=0; i<pavr.getNumDelay(); i++)
			{
				gen = gen - pavr.getDelayAt(i);
				
				if(gen < session.getRuleInfoAt(pavr.getId()).getBirth()) gen = session.getRuleInfoAt(pavr.getId()).getBirth();
								
				currFr_List.add(new RuleAndColor(session.getFuzzyRule(pavr.getId(), gen),
									pavr.getSubColor(pavr.getNumStatics()+i)));
			}
		}
		
		rulePanel.repaint();
		
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2d) 
	{
		g2d.translate(x_trans, y_trans);
		drawCurves(g2d);
//		drawData(g2d);
		drawAxis(g2d);
	}
	
	//###############################################################################################
	// Override Methods DropTarget
	//###############################################################################################
	
	@Override
	public boolean hit(MouseEvent me) 
	{
		Rectangle visible_rect = drawPanel.getVisibleRect();
		
		if(drawPanel.isShowing()
				&& me.getXOnScreen() < drawPanel.getLocationOnScreen().getX()
											+ visible_rect.x
											+ visible_rect.getWidth()
				&& me.getXOnScreen() > drawPanel.getLocationOnScreen().getX()
											+ visible_rect.x
				&& me.getYOnScreen() < drawPanel.getLocationOnScreen().getY()
											+ visible_rect.y
											+ visible_rect.getHeight()
				&& me.getYOnScreen() > drawPanel.getLocationOnScreen().getY()
											+ visible_rect.y)
		{
			return true;
		}		
		return false;
	}

	@Override
	public void drop(Object[] data) 
	{
		if(data != null)
		{
			for(int i=0; i<data.length; i++)
			{
				if(data[i] instanceof Integer)
				{
					this.addRule((Integer)data[i]);
				}
			}
		}
	}
	
	//###############################################################################################
	// draw Methods
	//###############################################################################################
	
	private void drawAxis(Graphics2D g2d)
	{
		g2d.setColor(Color.black);
		
		//x-Axis
		g2d.drawLine(0
				, 0
				, (int)((end - start) * x_scale)
				, 0);
		
		
		double labelRes = (end - start)/8;
		double grad		= 720/8d;
		String grad_str = null;
		
		for(int i=1; i<9; i++)
		{
			g2d.drawLine((int)(labelRes * i * x_scale)
					, -10
					, (int)(labelRes * i * x_scale)
					, 10);
			grad_str = NumberFormater.doubleToString((grad*i), 2);
			TextLayout textLayout = new TextLayout(grad_str, g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString(grad_str
					, (int)(labelRes * i * x_scale - bounds.getCenterX())
					, 25);
		}
		
		//y-Axis
		g2d.drawLine(0
				,(int)(- maxValue * y_scale)
				,0
				,(int)(maxValue * y_scale));
		
		double res = maxValue/8;
		double value = maxValue/8d;
		grad_str = null;
		
		for(int i=1; i<9; i++)
		{
			g2d.drawLine(-5
					, (int)(-res * i * y_scale)
					, 5
					, (int)(-res * i * y_scale));
			grad_str = NumberFormater.doubleToString((value*i), 2);
			TextLayout textLayout = new TextLayout(grad_str, g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString(grad_str
					, - 10 - (int)bounds.getWidth()
					, (int)( - res * i * y_scale - bounds.getCenterY()));
			
			g2d.drawLine(-5
					, (int)(res * i * y_scale)
					, 5
					, (int)(res * i * y_scale));
			grad_str = NumberFormater.doubleToString((-value*i), 2);
			textLayout = new TextLayout(grad_str, g2d.getFont(), g2d.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2d.drawString(grad_str
					, - 10 - (int)bounds.getWidth()
					, (int)(res * i * y_scale - bounds.getCenterY()));
		}
	}
	
	private void drawCurves(Graphics2D g2d)
	{
		double[] x;
		
		for(RuleAndColor rac: currFr_List)
		{
			int death = session.getRuleInfoAt(rac.fr.getId()).getDeath();
			
			if(session.getRuleInfoAt(rac.fr.getId()).getBirth() <= session.getCurrPopulationIndex()
					&& (death == 0
							|| death > session.getCurrPopulationIndex()))
			{			
//				x = this.mix_ms(rac.fr.getCenter(), rac.fr.getWidth());
//				
//				g2d.setColor(session.getRuleInfoAt(rac.fr.getId()).getColor());
//				drawCurve(g2d, x);
				
//				g2d.setColor(Color.black);
//				drawCurve(g2d, rac.fr.getCenter());
				g2d.setColor(session.getRuleInfoAt(rac.fr.getId()).getColor());
//				drawCurve(g2d, sumCenterWidth(rac.fr.getCenter(), rac.fr.getWidth()));
//				drawCurve(g2d, subCenterWidth(rac.fr.getCenter(), rac.fr.getWidth()));
				
				drawCurve(g2d, sumCenterWidth(rac.fr.getCenter(), rac.fr.getWidth())
						,subCenterWidth(rac.fr.getCenter(), rac.fr.getWidth()));
			}
		}
	}
	
	private void drawCurve(Graphics2D g2d, double[] x1, double[] x2)
	{
		double step = (end - start)/resolution;
		
		double value = 0;
		for(int i=1; i<=resolution; i++)
		{			
			
			value = andrewCurve(x2, step*(i-1));
			
			if(maxValue < value) maxValue = value;
			
//			g2d.drawLine((int)(step * i * x_scale)
//					, (int)(csCurve(x, step*i)*y_scale)
//					, (int)(step * (i-1) * x_scale)
//					, (int)(value*y_scale));
			
			g2d.drawLine((int)(step * i * x_scale)
					, (int)(-andrewCurve(x1, step*i)*y_scale)
					, (int)(step * (i-1) * x_scale)
					, (int)(-value*y_scale));
			
			double[] value_A = andrewCurveV(x1,x2,step*(i-1));
			double[] value_B = andrewCurveV(x1,x2,step*i);
			
			g2d.drawLine((int)(step * i * x_scale)
					, (int)(-value_B[0]*y_scale)
					, (int)(step * (i-1) * x_scale)
					, (int)(-value_A[0]*y_scale));
			
//			Color color = g2d.getColor();
//			g2d.setColor(Color.black);
			
			g2d.drawLine((int)(step * i * x_scale)
					, (int)(-value_B[1]*y_scale)
					, (int)(step * (i-1) * x_scale)
					, (int)(-value_A[1]*y_scale));
			
//			g2d.setColor(color);
		}
	}
	
	private void drawData(Graphics2D g2d)
	{
		double[][] data = session.getCurrData();
		int winID = 0;
		
		for(int i = 0; i< data.length; i++)
		{			
			for(int j=0; j<data[i].length - 1; j++)
			{			
				winID = FuzzyRuleCollection.compWinningRule(data[i], frCol);
				
				if(isInCurrList(winID))
				{
					g2d.setColor(session.getRuleInfoAt(winID).getColor());
					drawCurve(g2d, data[i]);
				}
			}
		}
	}
	
	private boolean isInCurrList(int id)
	{
		for(RuleAndColor rac: currFr_List)
		{
			if(rac.fr.getId() == id) return true;
		}
		return false;
	}
	
	
	private void drawCurve(Graphics2D g2d, double[] x)
	{
		double step = (end - start)/resolution;
		
		double value = 0;
		for(int i=1; i<=resolution; i++)
		{			
			value = andrewCurve(x, step*(i-1));
			
			if(maxValue < value) maxValue = value;
			
//			g2d.drawLine((int)(step * i * x_scale)
//					, (int)(csCurve(x, step*i)*y_scale)
//					, (int)(step * (i-1) * x_scale)
//					, (int)(value*y_scale));
			
			g2d.drawLine((int)(step * i * x_scale)
					, (int)(andrewCurve(x, step*i)*y_scale)
					, (int)(step * (i-1) * x_scale)
					, (int)(value*y_scale));
			
		}
		
	}
	private double andrewCurve(double[] x, double t)
	{
		double retValue = 0;
		int tFactor = 1;
		
		retValue = x[0]/Math.sqrt(2);
		
		for(int i=1; i<x.length; i++)
		{
			tFactor = ((i+1)/2);
			
			if(i%2 != 0)
			{				
				retValue = retValue + x[attr_Permutation.get(i)]*Math.sin(tFactor*t);
			}
			else
			{
				retValue = retValue + x[attr_Permutation.get(i)]*Math.cos(tFactor*t);
			}	
		}
		
		return retValue;
	}
	
	private double[] andrewCurveV(double[] x1, double[] x2, double t)
	{
		double[] retValue = new double[2];
		int tFactor = 1;

		retValue[0] = x1[0]/Math.sqrt(2);
		retValue[1] = x2[0]/Math.sqrt(2);
		
		for(int i=1; i<x1.length; i++)
		{
			tFactor = ((i+1)/2);
			
			if(i%2 != 0)
			{				
				if(((int)((t)/(Math.PI/tFactor)))%2 == 0)
				{
					retValue[0] = retValue[0] + x1[attr_Permutation.get(i)]*Math.sin(tFactor*t);
					retValue[1] = retValue[1] + x2[attr_Permutation.get(i)]*Math.sin(tFactor*t);
				}
				else
				{
					retValue[0] = retValue[0] + x2[attr_Permutation.get(i)]*Math.sin(tFactor*t);
					retValue[1] = retValue[1] + x1[attr_Permutation.get(i)]*Math.sin(tFactor*t);
				}
			}
			else
			{
				if(((int)((t-Math.PI/2)/(Math.PI/tFactor)))%2 == 0)
				{
					retValue[0] = retValue[0] + x1[attr_Permutation.get(i)]*Math.cos(tFactor*t);
					retValue[1] = retValue[1] + x2[attr_Permutation.get(i)]*Math.cos(tFactor*t);
				}
				else
				{
					retValue[0] = retValue[0] + x2[attr_Permutation.get(i)]*Math.cos(tFactor*t);
					retValue[1] = retValue[1] + x1[attr_Permutation.get(i)]*Math.cos(tFactor*t);
				}
			}	
		}		
		return retValue;
	}
	
//	private double andrewCurve(double[] x, double t)
//	{
//		double retValue = 0;
//		int tFactor = 1;
//		
//		retValue = x[0]/Math.sqrt(2);
//		
//		for(int i=1; i<x.length; i++)
//		{
//			tFactor = ((i+1)/2);
//			
//			if(i%2 != 0)
//			{				
//				retValue = retValue + x[i]*Math.sin(tFactor*t);
//			}
//			else
//			{
//				retValue = retValue + x[i]*Math.cos(tFactor*t);
//			}	
//		}
//		
//		return retValue;
//	}
	
	private double[] sumCenterWidth(double[] c, double[] w)
	{
		double[] retValue = new double[c.length];
		
		for(int i=0; i<c.length; i++)
		{
			retValue[i] = c[i] + w[i];
		}
		
		return retValue;
	}
	
	private double[] subCenterWidth(double[] c, double[] w)
	{
		double[] retValue = new double[c.length];
		
		for(int i=0; i<c.length; i++)
		{
			retValue[i] = c[i] - w[i];
		}
		
		return retValue;
	}
	
	private double csCurve(double[] x, double t)
	{
		double retValue = 0;
		int tFactor = 1;
		
//		retValue = x[0]/Math.sqrt(Math.PI);
		
		for(int i=0; i<x.length; i++)
		{
			tFactor = ((i+2)/2);
			
			if(i%2 == 0)
			{				
				retValue = retValue + x[i]*Math.sin(tFactor*t);
			}
			else
			{
				retValue = retValue + x[i]*Math.cos(tFactor*t);
			}			 
		}
		return retValue;
	}
	
	private double[] mix_ms(double[] m, double[] s)
	{
		double[] ret_A = new double[m.length + m.length];
		
		int j = 0;
		int k = 0;
		for(int i=0; i<ret_A.length; i++)
		{
			if(i%2 == 0)
			{
				ret_A[i] = m[attr_Permutation.get(j)];
				j++;
			}
			else
			{
				ret_A[i] = s[attr_Permutation.get(k)];
				k++;
			}			
		}	
		return ret_A;
	}
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void addRule(int id)
	{
		boolean alreadyIn = false;
		
		for(ParallelAxisViewRule pavr: pavr_List)
		{
			if(pavr.getId() == id) alreadyIn = true;
		}
		
		if(!alreadyIn)
		{
			pavr_List.add(new ParallelAxisViewRule(id
						, session));
			DrawRule dr = new DrawRule(id);
			rulePanelMouseListener.addToMovedList(dr);
			rulePanel.addDrawObject(dr);			
		}
		rulePanel.repaint();
		drawPanel.repaint();
	}
	
	public void removeRule(int index)
	{
		rulePanelMouseListener.removeFromMovedList(index);
		rulePanel.removeDrawObject(index);	
		
		pavr_List.remove(index);
		
		rulePanel.repaint();		
		drawPanel.repaint();
	}
	
	public void clear()
	{
		rulePanelMouseListener.clearMovedList();
		rulePanel.clear();

		pavr_List.clear();
		
		rulePanel.repaint();
		drawPanel.repaint();
	}
	
	public void addStaticRule(int index, int generation)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).addStaticRule(generation);
			
			drawPanel.repaint();
			rulePanel.repaint();
		}		
	}
	
	public void removeStaticRule(int index, int subIndex)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).removeStaticRule(subIndex);
			
			drawPanel.repaint();
			rulePanel.repaint();
		}
	}
	
	public void addDelayRule(int index, int delay)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).addDelayRule(delay);
			
			drawPanel.repaint();
			rulePanel.repaint();
		}		
	}
	
	public void removeDelayRule(int index, int subIndex)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).removeDelayRule(subIndex);
			
			drawPanel.repaint();
			rulePanel.repaint();
		}
	}
	
	//###############################################################################################
	// Override Methods ChangeListener
	//###############################################################################################

	@Override
	public void stateChanged(ChangeEvent e) 
	{
		JSlider slider = (JSlider)e.getSource();
		
		if(slider.getName().equals(y_Slider.getName()))
		{
			this.y_scale = slider.getValue()/100d;
		}
		else if(slider.getName().equals(x_Slider.getName()))
		{
			this.x_scale = slider.getValue()/10d;			
		}
		
		this.repaintDrawPanel();
	}
	
	//###############################################################################################
	// inner Class DrawRule
	//###############################################################################################
	
	private class DrawRule extends DrawObject implements MouseInteractionObject{
		
		public int id;
		
		//###############################################################################################
		// Constructor
		//###############################################################################################
		
		public DrawRule(int id_P)
		{
			this.id = id_P;
		}

		//###############################################################################################
		// Override Methods DrawObject
		//###############################################################################################
		
		@Override
		public void draw(Graphics2D g2d)
		{
			boolean setGray;
			
			for(int i=0; i<pavr_List.size(); i++)
			{			
				if((frCol.getGeneration() >= session.getRuleInfoAt(pavr_List.get(i).getId()).getDeath()
						&&  session.getRuleInfoAt(pavr_List.get(i).getId()).getDeath() > session.getRuleInfoAt(pavr_List.get(i).getId()).getBirth())
						|| frCol.getGeneration() < session.getRuleInfoAt(pavr_List.get(i).getId()).getBirth())
				{
					setGray = true;
				}
				else
				{
					setGray = false;
				}
				
				if(setGray)
				{			
					g2d.setColor(Color.gray);
				}
				else
				{
					g2d.setColor(session.getRuleInfoAt(pavr_List.get(i).getId()).getColor());
				}

				g2d.fillRoundRect((padding_H+rule_Width)*i
					, (int)ruleTransform.getTranslateY() + padding_V
					, rule_Width
					, rule_Height
					, 10
					, 10);
				
				g2d.setColor(Color.black);
				TextLayout textLayout = new TextLayout(pavr_List.get(i).getId()+"", g2d.getFont(), g2d.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				g2d.drawString(pavr_List.get(i).getId()+""
						,(padding_H+rule_Width)*i  + rule_Width/2 -(int)bounds.getCenterX()
						,(int)ruleTransform.getTranslateY() + padding_V + (int) rule_Height/2-(int)bounds.getCenterY());
				
				for(int j=0; j< pavr_List.get(i).getNumStatics(); j++)
				{
					if(setGray)
					{			
						g2d.setColor(Color.gray);
					}
					else
					{
						g2d.setColor(pavr_List.get(i).getSubColor(j));
					}
					g2d.fillRoundRect((padding_H+rule_Width)*i
						, (int)ruleTransform.getTranslateY() + padding_V +  (padding_V + rule_Height)*(j + 1)
						, rule_Width
						, rule_Height
						, 10
						, 10);
					
					g2d.setColor(Color.black);
					g2d.drawRoundRect((padding_H+rule_Width)*i
						, (int)ruleTransform.getTranslateY() + padding_V +  (int)(padding_V + rule_Height)*(j + 1)
						, rule_Width
						, rule_Height
						, 10
						, 10);
					
					g2d.setColor(Color.black);
					textLayout = new TextLayout(pavr_List.get(i).getId()
													+"."
													+pavr_List.get(i).getStaticGenAt(j), g2d.getFont(), g2d.getFontRenderContext());
					bounds = textLayout.getBounds();
					g2d.drawString(pavr_List.get(i).getId()
								+"."
								+pavr_List.get(i).getStaticGenAt(j)
							,(padding_H+rule_Width)*i 
									+ rule_Width/2 
									- (int)bounds.getCenterX()
							,(int)ruleTransform.getTranslateY() + rule_Height/2-(int)bounds.getCenterY()  
									+ padding_V +  (padding_V + rule_Height) * (j + 1));
				}
				
				for(int j=0; j< pavr_List.get(i).getNumDelay(); j++)
				{
					if(setGray)
					{			
						g2d.setColor(Color.gray);
					}
					else
					{
						g2d.setColor(pavr_List.get(i).getSubColor(pavr_List.get(i).getNumStatics()+j));
					}
					g2d.fillRoundRect((padding_H+rule_Width)*i
						,(int)ruleTransform.getTranslateY() +  padding_V + (padding_V + rule_Height)*(pavr_List.get(i).getNumStatics()+j + 1)
						, rule_Width
						, rule_Height
						, 10
						, 10);
					
					g2d.setColor(Color.black);
					g2d.drawRoundRect((padding_H+rule_Width)*i
						, (int)ruleTransform.getTranslateY() + padding_V +  (padding_V + rule_Height)*(pavr_List.get(i).getNumStatics()+j + 1)
						, rule_Width
						, rule_Height
						, 10
						, 10);
					
					g2d.drawRoundRect((padding_H+rule_Width)*i -3
							, (int)ruleTransform.getTranslateY() + padding_V + (padding_V + rule_Height)*(pavr_List.get(i).getNumStatics()+j + 1)-3
							, rule_Width + 6
							, rule_Height + 6
							, 10
							, 10);
					
					g2d.setColor(Color.black);
					textLayout = new TextLayout(pavr_List.get(i).getId()
													+"|"
													+pavr_List.get(i).getDelayAt(j), g2d.getFont(), g2d.getFontRenderContext());
					bounds = textLayout.getBounds();
					g2d.drawString(pavr_List.get(i).getId()
								+"|"
								+pavr_List.get(i).getDelayAt(j)
							,(padding_H+rule_Width)*i
									+ rule_Width/2 
									- (int)bounds.getCenterX()
							,(int)ruleTransform.getTranslateY() + rule_Height/2-(int)bounds.getCenterY()  
									+ padding_V + (padding_V + rule_Height) * (pavr_List.get(i).getNumStatics()+j + 1));
				}
			}		
		}
		
		//###############################################################################################
		// Override Methods MouseInteractionObject
		//###############################################################################################

		@Override
		public boolean action(MouseEvent me, int eventType) 
		{
			double index;
			double x;
	
			switch(eventType){
			
			//############ CLICKED #################################################################
				case CleverMouseListener.CLICKED: 	break;
			//############ ENTERED #################################################################
				case CleverMouseListener.ENTERED:	break;
			//############ EXITED  #################################################################
				case CleverMouseListener.EXITED:	break;		
			//############ PRESSED #################################################################
				case CleverMouseListener.PRESSED:	break;
			//############ RELEASED #################################################################		
				case CleverMouseListener.RELEASED: 	break;
			//############ DRAGGED #################################################################		
				case CleverMouseListener.DRAGGED: 	break;
			//############ MOVED   #################################################################
				case CleverMouseListener.MOVED:
					
					
					index = (int)(me.getX()/(padding_H + rule_Width));
					x = me.getX() - index*(padding_H + rule_Width);
					
					int subIndex = ((int)(me.getY() - (int)ruleTransform.getTranslateY())/(padding_V + rule_Height));
					
					double y = (me.getY()-(int)ruleTransform.getTranslateY()) - subIndex*(padding_V + rule_Height);				
					
					if(index < pavr_List.size()
							&& index >= 0
							&& x < rule_Width
							&& x > 0
							&& subIndex < pavr_List.get((int)index).getNumStatics() 
											+ pavr_List.get((int)index).getNumDelay() + 1
							&& y < rule_Height
							&& y > 0)
					{
						Point point = rulePanel.getLocationOnScreen();						
						InteractionField[] i_Field = null;
						
						if(subIndex == 0)
						{
							i_Field = new InteractionField[2];
							i_Field[0] = new AndrewsView_DeleteInteraction(AndrewsView.this, (int)index, 0, 0);
							i_Field[1] = new AndrewsView_Interaction_2(AndrewsView.this, (int)index);
						}
						else if(subIndex > 0
									&& subIndex < pavr_List.get((int)index).getNumStatics() + 1)
						{
							i_Field = new InteractionField[1];
							i_Field[0] = new AndrewsView_DeleteInteraction(AndrewsView.this, (int)index, 1, subIndex - 1);
						}
						else if(subIndex > pavr_List.get((int)index).getNumStatics()
								&& subIndex < pavr_List.get((int)index).getNumStatics() 
													+ pavr_List.get((int)index).getNumDelay() + 1)
						{
							i_Field = new InteractionField[1];
							i_Field[0] = new AndrewsView_DeleteInteraction(AndrewsView.this, (int)index, 2, subIndex 
																									- pavr_List.get((int)index).getNumStatics() 
																									- 1);
						}
						
						Interaction interaction = Interaction.getInstance(i_Field
													, null
													, point.x
														+ (int)((padding_H+rule_Width)*index)
														+ rule_Width/2
													, point.y + (int)ruleTransform.getTranslateY()+ padding_V
														+ (int)((padding_V+rule_Height)*subIndex)
														+ rule_Height/2
													, 20
													, null);
					}					
					break;				
			}			
			return true;
		}

		@Override
		public boolean hitObject(MouseEvent me) 
		{
			return true;
		}		
	}
	
	//###############################################################################################
	// inner Class DrawRule
	//###############################################################################################
	
	private class DrawAttrBlock extends DrawObject implements MouseInteractionObject{
		
		
		
		//###############################################################################################
		// Constructor
		//###############################################################################################
		


		//###############################################################################################
		// Override Methods DrawObject
		//###############################################################################################
		
		@Override
		public void draw(Graphics2D g2d)
		{
			int ruleSize = session.getFr_premiseSize();
			
			g2d.setColor(new Color(100,100,100));
			for(int i=0; i<ruleSize; i++)
			{
				g2d.fillRoundRect(padding_H*3 + (block_width + padding_H)*i
						, padding_V
						, block_width
						, block_height
						, 5
						, 5);
			}
			g2d.setColor(Color.white);
			for(int i=0; i<ruleSize; i++)
			{
				TextLayout textLayout = new TextLayout(attr_Permutation.get(i)+"", g2d.getFont(), g2d.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				g2d.drawString(attr_Permutation.get(i)+""
						, padding_H*3 + block_width/2 +  (block_width + padding_H)*i  - (int)bounds.getCenterX()
						, padding_V +  block_height/2-(int)bounds.getCenterY());
						
			}	
		}
		
		//###############################################################################################
		// Override Methods MouseInteractionObject
		//###############################################################################################

		@Override
		public boolean action(MouseEvent me, int eventType) 
		{

			boolean retValue = false;
			
			double index;
			double x;

			switch(eventType){
			
			//############ CLICKED #################################################################
				case CleverMouseListener.CLICKED: 	break;
			//############ ENTERED #################################################################
				case CleverMouseListener.ENTERED:	break;
			//############ EXITED  #################################################################
				case CleverMouseListener.EXITED:	break;		
			//############ PRESSED #################################################################
				case CleverMouseListener.PRESSED:
			
					if(padding_V < me.getY()
							&& padding_V +block_height > me.getY())
					{
						index = ((me.getX() - padding_H*3)/(double)(padding_H + block_width));

						x = me.getX() - padding_H*3 -(int)index*(padding_H + block_width);
						
						if((index >= 0 && index < session.getFr_premiseSize())
								&& x < block_width)
						{			
							dragedIndex = (int)index;
							
							Cursor c = rulePanel.getToolkit().createCustomCursor(
									new ImageIcon("res/Icons/dragBlock.png").getImage(),
									new Point(16,16), "move");
							rulePanel.setCursor(c);
						}
						retValue = true;
					}
					break;
			//############ RELEASED #################################################################		
				case CleverMouseListener.RELEASED:
		
					if(dragedIndex > -1)
					{
						index = ((me.getX() - padding_H*3 + padding_H)/(double)(padding_H + block_width));
						
						if(index >= 0 && index < session.getFr_premiseSize()+1)
						{
							int dragValue = attr_Permutation.get(dragedIndex);
							if(dragedIndex+1 < index)
							{
								attr_Permutation.add((int)index, dragValue);
								attr_Permutation.remove(dragedIndex);
							}
							else if(dragedIndex > index)
							{
								attr_Permutation.remove(dragedIndex);
								attr_Permutation.add((int)index,dragValue);
							}
						}
						retValue = true;
				
						rulePanel.setCursor(Cursor.getDefaultCursor());
						rulePanel.repaint();
						drawPanel.repaint();
						dragedIndex = -1;
					}
			
					break;
			//############ DRAGGED #################################################################		
				case CleverMouseListener.DRAGGED: 	break;
			//############ MOVED   #################################################################
				case CleverMouseListener.MOVED:  break;
							
			}
					
			return retValue;
		}

		@Override
		public boolean hitObject(MouseEvent me) {

			return true;
		}	
		
	}
	
	public ParallelAxisViewRule getPAVruleAt(int index)
	{
		return pavr_List.get(index);
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}

}
