package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeSet;

import javax.swing.JPanel;

import listener.CleverMouseListener;
import listener.MouseInteractionObject;

import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.DropTarget;
import guiUtil.Interaction;
import guiUtil.InteractionField;
import guiUtil.NumberFormater;

public class AttributeHistoryView extends RuleVisualizer implements DropTarget, AdjustmentListener{
	
	private int box_Width 	= 20;
	private int box_Height 	= 200;
	
	private int rule_Width = 20;
	private int rule_Height = 20;	
	private int padding_H = 10;
	private int padding_V = 5;
	
	private int attr_Width = 18;
	private int attrPadding_North = 0;
	
	private DrawPanel rulePanel;
	private DrawPanel attrPanel;
	private CleverMouseListener rulePanelMouseListener = new CleverMouseListener();
	private TimeLineSlider timeLineSlider;
	private AttributeHistory_PopUp ahv_PopUp;
	
	private ArrayList<Integer> id_List = new ArrayList<Integer>();
	
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public AttributeHistoryView(Session session_P) {
		super(session_P, new DrawPanel());
		
		ahv_PopUp = new AttributeHistory_PopUp(this);
		
		cm_Listener.addToReleasedList(this);
		cm_Listener.addToPressedList(this);
		
		rulePanel = new DrawPanel();
		rulePanel.setPreferredSize(new Dimension(200, 30));
		rulePanel.addMouseMotionListener(rulePanelMouseListener);
		
		attrPanel = new DrawPanel();
		attrPanel.setPreferredSize(new Dimension(60, drawPanel.getHeight()));
		attrPanel.addDrawObject(new DrawAttribute());
		
		timeLineSlider = new TimeLineSlider(session, drawPanel);
		timeLineSlider.setScale_Dis(box_Width);
		timeLineSlider.setScaleStart(box_Width/2);
		
		scrollPane.getHorizontalScrollBar().addAdjustmentListener(this);
		scrollPane.getVerticalScrollBar().addAdjustmentListener(this);		
		JPanel paddingPanel = new JPanel();
		paddingPanel.setPreferredSize(new Dimension((int)attrPanel.getPreferredSize().getWidth(), 20));
		
		JPanel tlsWrapperPanel = new JPanel();
		tlsWrapperPanel.setLayout(new BorderLayout());
		tlsWrapperPanel.add(paddingPanel, BorderLayout.WEST);
		tlsWrapperPanel.add(timeLineSlider.getComponent(), BorderLayout.CENTER);
		
		mainPanel.add(rulePanel, BorderLayout.SOUTH);
		mainPanel.add(tlsWrapperPanel, BorderLayout.NORTH);
		mainPanel.add(attrPanel, BorderLayout.WEST);
	}

	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) {

		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
								
				if(me.isPopupTrigger())
				{	
					ahv_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED:
				
				if(me.isPopupTrigger())
				{	
					ahv_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;
		}
				
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) {
		// TODO Auto-generated method stub
		return true;
	}

	//###############################################################################################
	// Override Methods DropTarget
	//###############################################################################################
	
	@Override
	public boolean hit(MouseEvent me) 
	{
		Rectangle visible_rect = drawPanel.getVisibleRect();
		
		if(drawPanel.isShowing()
				&& me.getXOnScreen() < drawPanel.getLocationOnScreen().getX()
											+ visible_rect.x
											+ visible_rect.getWidth()
				&& me.getXOnScreen() > drawPanel.getLocationOnScreen().getX()
											+ visible_rect.x
				&& me.getYOnScreen() < drawPanel.getLocationOnScreen().getY()
											+ visible_rect.y
											+ visible_rect.getHeight()
				&& me.getYOnScreen() > drawPanel.getLocationOnScreen().getY()
											+ visible_rect.y)
		{
			return true;
		}		
		return false;
	}

	@Override
	public void drop(Object[] data) 
	{
		if(data != null)
		{
			for(int i=0; i<data.length; i++)
			{
				if(data[i] instanceof Integer)
				{
					this.addRule((Integer)data[i]);
				}
			}
		}
	}
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	public void updateVisualisation() 
	{
		timeLineSlider.updateVisualisation();
		int width = box_Width*session.getNumGenerations();
		int height= box_Height*session.getFr_premiseSize();
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
		rulePanel.repaint();
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2d) {
		
		AffineTransform emptyTrans = g2d.getTransform();
		
		for(int i=0; i< session.getFr_premiseSize(); i++)
		{			
			drawBoxes(g2d);
			drawAttributeBars(g2d, i);
			
			g2d.translate(0, box_Height);
		}
		
		g2d.setTransform(emptyTrans);
		
		drawRedLine(g2d);
		
	}
	
	//###############################################################################################
	// Override Methods AdjustmentListener
	//###############################################################################################

	@Override
	public void adjustmentValueChanged(AdjustmentEvent arg0) {
		
		if(arg0.getSource().equals(scrollPane.getHorizontalScrollBar()))
		{
			timeLineSlider.scroll(arg0);
		}
		else
		{
			Rectangle rect = drawPanel.getVisibleRect();
		
			attrPadding_North = rect.y;
			attrPanel.repaint();
		}	
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void addRule(Integer id_P)
	{
		boolean alreadyIn = false;
		
		for(Integer integer: id_List)
		{
			if(integer.intValue() == id_P.intValue()) alreadyIn = true;
		}

		if(!alreadyIn)
		{		
			id_List.add(id_P);
		
			DrawRule dr = new DrawRule(id_P);
			rulePanelMouseListener.addToMovedList(dr);
			rulePanel.addDrawObject(dr);
		
			drawPanel.repaint();
			rulePanel.repaint();
		}
	}
	
	public void removeRule(int index)
	{
		rulePanelMouseListener.removeFromMovedList(index);
		id_List.remove(index);
		rulePanel.removeDrawObject(index);	
		
		drawPanel.repaint();
		rulePanel.repaint();
	}
	
	public void clear()
	{
		rulePanelMouseListener.clearMovedList();
		id_List.clear();
		rulePanel.clear();
		
		drawPanel.repaint();
		rulePanel.repaint();
	}
	
	public String printToLatex()
	{
		StringBuffer sb = new StringBuffer();
		
		int picHeight = box_Height*session.getFr_premiseSize();
		int picWidth = session.getNumGenerations() * box_Width;
		
		sb.append("\\documentclass{article}");
		sb.append("\n");
		sb.append("\\usepackage{amsmath}");
		sb.append("\n");
		sb.append("\\usepackage{amsfonts}");
		sb.append("\n");
		sb.append("\\usepackage{pst-all}");
		sb.append("\n");
		sb.append("\\usepackage{pstricks}");
		sb.append("\n");
		sb.append("\\usepackage{pstricks-add}");
		sb.append("\n");
		sb.append("\\usepackage{pst-pdf}");
		sb.append("\n");
		
		sb.append("\\begin{document}");
		sb.append("\n");
		
		sb.append("\\psset{xunit=1pt,yunit=1pt}");
		sb.append("\n");
		
		sb.append("\\begin{pspicture}" +
				"(0,0)" +
				"(" + picWidth + "," + picHeight + ")");
		sb.append("\n");
			
		sb.append("% Background-Boxes #######################################################");
		sb.append("\n");
		sb.append("\\newrgbcolor{boxColorA}{0.7529412 0.7529412 0.7529412}");
		sb.append("\n");
		sb.append("\\newrgbcolor{boxColorB}{1 1 1}");
		sb.append("\n");
		
		for(int i=0; i<session.getNumGenerations(); i++)
		{
			if(i%2 == 0)
			{
				sb.append("\\psframe[linestyle=none, fillstyle = solid, fillcolor = boxColorA]" +
						"(" + box_Width*i + "," + 0 +")" +
						"(" + (box_Width*i+box_Width) + "," + picHeight +")");
				sb.append("\n");
			}
			else
			{
				sb.append("\\psframe[linestyle=none, fillstyle = solid, fillcolor = boxColorB]" +
						"(" + box_Width*i + "," + 0 +")" +
						"(" + (box_Width*i+box_Width) + "," + picHeight +")");
				sb.append("\n");
			}
		}
		
		//Attributes---------------------------------------------------
		float[] rgb = new float[4];
		String[] colorNames = new String[id_List.size()];
		int[] colorToID = new int[id_List.size()];
		
		for(int c=0; c<id_List.size(); c++)
		{
			colorNames[c] = NumberFormater.integerToWord(c);
			colorToID[c] = id_List.get(c);
			
			session.getRuleInfoAt(id_List.get(c)).getColor().getRGBComponents(rgb);
			sb.append("\\newrgbcolor{"+ colorNames[c] +"}{"+rgb[0]+" "+rgb[1]+" "+rgb[2]+"}\n");
		}
		
		for(int k=0; k< session.getFr_premiseSize(); k++)
		{	
			sb.append("% Attribute "+k+" ####################################################");
			sb.append("\n");
			int attr = k;
						
			for(int i=0; i<session.getNumGenerations(); i++)
			{		
				int colorID = 0;
	
				LinkedList<BoxSeparator> splitBox = calcSplitBox(i, attr);
				
				ArrayList<Integer> currIDs = new ArrayList<Integer>();
				
				Iterator<BoxSeparator> it = splitBox.iterator();
				
				BoxSeparator bs_0 = null;
				double value_0 = 0;
				if(it.hasNext())
				{
					bs_0 = it.next();
				}

				int helpWidth = 0;
				int sum;
				while(it.hasNext())
				{
					if(bs_0.right > -1)
					{
						currIDs.add(bs_0.right);
					}		
					value_0 = bs_0.value;

					bs_0 = it.next();
					
					sum = 0;
					for(int j=0; j<currIDs.size(); j++)
					{			
						for(int c=0; c<colorNames.length; c++)
						{
							if(colorToID[c] == currIDs.get(j))
								colorID = c;
						}
						
						helpWidth = calcWidth(attr_Width, currIDs.size(), j);
										
						sb.append("\\psframe[linestyle=none, fillstyle = solid, fillcolor = "+colorNames[colorID]+"]" +
								"(" + (box_Width*i+ sum) + "," + (picHeight-(box_Height*k)-value_0) +")" +
								"(" + (box_Width*i+ sum + helpWidth+1) 
									+ "," + (picHeight-(box_Height*k)- bs_0.value-1) +")");
					
						sb.append("\n");
						
						sum = sum + helpWidth;
					}
					
					if(bs_0.left > -1)
					{
						currIDs.remove(new Integer(bs_0.left));
					}
				}
			}
		}
		
		for(int k=0; k< session.getFr_premiseSize()-1; k++)
		{	
			sb.append("\\psline[linewidth = 1pt, linestyle=solid]{-}" +
					"(" + 0 + "," + (box_Height*(k+1)) + ")" +
					"(" + picWidth + "," + (box_Height*(k+1)) + ")");
			sb.append("\n");
		}
		
		sb.append("\\end{pspicture}");
		sb.append("\n");
		sb.append("\\end{document}");
		
		return sb.toString();
	}
	
	//###############################################################################################
	// private DrawMethods
	//###############################################################################################
	
	private void drawRedLine(Graphics2D g2d)
	{
		int x = session.getCurrPopulationIndex()*box_Width + box_Width/2;
		
		g2d.setColor(Color.red);
		g2d.drawLine(x + timeLineSlider.getTransX()
				, 0
				, x + timeLineSlider.getTransX()
				, drawPanel.getHeight());
	}
	
	private void drawBoxes(Graphics2D g2d)
	{
		for(int i=0; i<session.getNumGenerations(); i++)
		{
			if(i%2 == 0)
			{
				g2d.setColor(new Color(240,240,240));
			}
			else
			{
				g2d.setColor(Color.WHITE);
			}
			g2d.fillRect(box_Width * i
					, 0
					, box_Width
					, box_Height);
			g2d.setColor(Color.black);
			g2d.drawLine(box_Width*i
					, box_Height-1
					, box_Width*(i+1)
					, box_Height-1);
		}
	}
	
	private void drawAttributeBars(Graphics2D g2d, int attr)
	{
//		ArrayList<IntersectBox> iBox_List;
//		ArrayList<IntersectBox> toTestBox_List;
		
		for(int i=0; i<session.getNumGenerations(); i++)
		{
//			iBox_List = new ArrayList<IntersectBox>();
//			toTestBox_List = new ArrayList<IntersectBox>();
//			
//			for(int j=0; j<id_List.size(); j++)
//			{
//				int death = session.getRuleInfoAt(id_List.get(j)).getDeath();
//				
//				if(session.getRuleInfoAt(id_List.get(j)).getBirth() <= i
//						&& (death == 0
//								|| death > i))
//				{
//					double center = session.getFuzzyRule(id_List.get(j), i).getCenterFromAttr(attr);
//					center = (center - session.getAttr_RangeAt(attr)[0])/(session.getAttr_RangeAt(attr)[1] - session.getAttr_RangeAt(attr)[0]);
//					center = 1 - center; // y-Axis-draw is from top to bottom
//					
//					double width = session.getFuzzyRule(id_List.get(j), i).getWidthFromAttr(attr);					
//					width = box_Height*width/(session.getAttr_RangeAt(attr)[1] - session.getAttr_RangeAt(attr)[0]);
//					
//					if(((int)width) < 1) width = 1;
//												
//					if(iBox_List.isEmpty())
//					{
//						IntersectBox iBox = new IntersectBox((int)(box_Height*center - (int)width)
//													, (int)width*2
//													, id_List.get(j));
//
//						iBox_List.add(iBox);
//
//					}
//					else
//					{					
//						IntersectBox iBox = new IntersectBox((int)(box_Height*center - width)
//								, (int)(width*2)
//								, id_List.get(j));
//
//						toTestBox_List.add(iBox);	
//						
//						ArrayList<IntersectBox> ibHelp_List = new ArrayList<IntersectBox>();
//						ArrayList<IntersectBox> testHelp_List = new ArrayList<IntersectBox>();
//						
//						while(toTestBox_List.size() > 0)
//						{
//							testHelp_List.clear();
//						
//							for(IntersectBox testBox: toTestBox_List)
//							{					
//								ibHelp_List.clear();
//								
//								for(IntersectBox ib: iBox_List)
//								{
//									IntersectBox newBox = ib.splitBox(testBox);
//									
//									if(newBox != null)
//									{
//										if(newBox.toTest)
//										{
//											testHelp_List.add(newBox);
//										}
//										else
//										{
//											ibHelp_List.add(newBox);
//										}									
//									}
//								}							
//								ibHelp_List.add(testBox);						
//								iBox_List.addAll(ibHelp_List);
//							}						
//							toTestBox_List.clear();
//							toTestBox_List.addAll(testHelp_List);
//
//						}
//					}					
//				}
//			}
//
//			for(IntersectBox ib: iBox_List)
//			{
//				for(int j=0; j<ib.id_List.size(); j++)
//				{			
//					g2d.setColor(session.getRuleInfoAt(ib.id_List.get(j)).getColor());
//				
//					g2d.fillRect(box_Width*i+ (attr_Width/ib.id_List.size())*j
//							, ib.y
//							, (attr_Width/ib.id_List.size())
//							, ib.height);					
//				}
//			}
			
			LinkedList<BoxSeparator> splitBox = calcSplitBox(i, attr);
			
			ArrayList<Integer> currIDs = new ArrayList<Integer>();
			
			Iterator<BoxSeparator> it = splitBox.iterator();
			
			BoxSeparator bs_0 = null;
			double value_0 = 0;
			if(it.hasNext())
			{
				bs_0 = it.next();
			}

			int helpWidth = 0;
			int sum;
			while(it.hasNext())
			{
				if(bs_0.right > -1)
				{
					currIDs.add(bs_0.right);
				}		
				value_0 = bs_0.value;

				bs_0 = it.next();
				
				sum = 0;
				for(int j=0; j<currIDs.size(); j++)
				{			
					helpWidth = calcWidth(attr_Width, currIDs.size(), j);
									
					g2d.setColor(session.getRuleInfoAt(currIDs.get(j)).getColor());
				
					g2d.fillRect(box_Width*i+ sum
							, (int)value_0
							, helpWidth
							, (int)(bs_0.value - value_0));
					
					sum = sum + helpWidth;
				}
				
				if(bs_0.left > -1)
				{
					currIDs.remove(new Integer(bs_0.left));
				}
			}
		}
	}
	
	private LinkedList<BoxSeparator> calcSplitBox(int gen, int attr)
	{
		LinkedList<BoxSeparator> splitBox = new LinkedList<BoxSeparator>();
		
		for(int j=0; j<id_List.size(); j++)
		{
			int death = session.getRuleInfoAt(id_List.get(j)).getDeath();
			
			if(session.getRuleInfoAt(id_List.get(j)).getBirth() <= gen
					&& (death == 0
							|| death > gen))
			{
				double center = session.getFuzzyRule(id_List.get(j), gen).getCenterFromAttr(attr);
				center = (center - session.getAttr_RangeAt(attr)[0])/(session.getAttr_RangeAt(attr)[1] - session.getAttr_RangeAt(attr)[0]);
				center = 1 - center; // y-Axis-draw is from top to bottom
				
				double width = session.getFuzzyRule(id_List.get(j), gen).getWidthFromAttr(attr);					
				width = box_Height*width/(session.getAttr_RangeAt(attr)[1] - session.getAttr_RangeAt(attr)[0]);
				
				if(((int)width) < 1) width = 1;
				
				BoxSeparator bs_left = new BoxSeparator();
				bs_left.value = (int)(box_Height*center - width);
				bs_left.left = -1;
				bs_left.right = id_List.get(j);
				
				int index = 0;
				for(BoxSeparator bs: splitBox)
				{
					if(bs_left.value < bs.value)
					{
						break;
					}
					index++;
				}
				splitBox.add(index, bs_left);
				
				BoxSeparator bs_right = new BoxSeparator();
				bs_right.value = (int)(box_Height*center + width);
				bs_right.left = id_List.get(j);
				bs_right.right = -1;
				
				index = 0;
				for(BoxSeparator bs: splitBox)
				{
					if(bs_right.value < bs.value)
					{
						break;
					}
					index++;
				}
				splitBox.add(index, bs_right);
			}
		}
		return splitBox;
	}
	
	private int calcWidth(int attrWidth, int num, int index)
	{
		int retValue = 0;
		int rest = 0;
		
		retValue = attrWidth/num;
		rest = attrWidth-(retValue*num);
		
		if(index < rest) retValue++;
		
		return retValue;
	}
	
	//###############################################################################################
	// inner Class
	//###############################################################################################
	
	private class BoxSeparator{
		
		public int right;
		public int left;
		public double value;
		
	}
	
	private class IntersectBox{
		
		public int y;
		public int height;
		public ArrayList<Integer> id_List = new ArrayList<Integer>();
		public boolean toTest = false;
		
		//###############################################################################################
		// public Methods
		//###############################################################################################
				
		public IntersectBox(int y_P, int height_P, int id_P)
		{
			this.y 		= y_P;
			this.height = height_P;
			this.id_List.add(id_P);
		}
		
		public IntersectBox(int y_P, int height_P)
		{
			this.y 		= y_P;
			this.height = height_P;
		}
		
		public IntersectBox splitBox(IntersectBox ib)
		{
			if(y < ib.y && y+height > ib.y)
			{
				if( y + height < ib.y + ib.height)
				{
					IntersectBox newIb = new IntersectBox(ib.y, (y+height-ib.y));
					newIb.id_List.addAll(id_List);
					newIb.id_List.addAll(ib.id_List);
					
					if(newIb.height == 0) newIb = null;
					
					int helpY = ib.y;
					
					ib.height = ib.y + ib.height - (y + height);
					ib.y = y + height;
					
					this.height = helpY - y;
							
					return newIb;
				}
				else
				{
					IntersectBox newIb = new IntersectBox(ib.y, ib.height);
					newIb.id_List.addAll(id_List);
					newIb.id_List.addAll(ib.id_List);
					
					if(newIb.height == 0) newIb = null;
					
					int helpY = ib.y;
					int helpH = ib.height;
					
					ib.height = (y + height) - (ib.y + ib.height);
					ib.y = ib.y + helpH;
					ib.id_List.clear();
					ib.id_List.addAll(id_List);
					
					this.height = helpY - y;
							
					return newIb;
				}
			}
			else if(y == ib.y)
			{
				if( y + height < ib.y + ib.height)
				{
					this.id_List.addAll(ib.id_List);
					
					ib.height = ib.y + ib.height - (y + height);
					ib.y = y + height;
										
					return null;
				}
				else
				{
					ib.id_List.addAll(this.id_List);
					
					this.height = this.y + this.height - (ib.y + ib.height);
					this.y = ib.y + ib.height;
				}
			}
			else if(ib.y < y && ib.y+ib.height > y)
			{
				if( y + height > ib.y + ib.height)
				{
					IntersectBox newIb = new IntersectBox(y, ib.y+ib.height-y);
					newIb.id_List.addAll(id_List);
					newIb.id_List.addAll(ib.id_List);
					
					int helpY = this.y;
										
					this.height = this.y + this.height - (ib.y + ib.height);
					this.y = ib.y + ib.height;
					
					ib.height = helpY - ib.y;
					
					return newIb;
				}
				else
				{
					IntersectBox newIb = new IntersectBox(y + height, (ib.y + ib.height) - (this.y + this.height));
					newIb.id_List.addAll(ib.id_List);
					
					int helpY = this.y;
					
					this.id_List.addAll(ib.id_List);
					
					ib.height = helpY - ib.y;
				
					newIb.toTest = true;
							
					return newIb;
				}
			}
			
			return null;
		}
		
		public String toString()
		{
			StringBuffer sb = new StringBuffer();
			
			sb.append("y: " + y + "\n");
			sb.append("height: " + height + "\n");
			sb.append(id_List.toString());
			
			return sb.toString();
		}
	}
	
	//###############################################################################################
	// inner Classes
	//###############################################################################################
	
	private class DrawAttribute extends DrawObject{
		
		//###############################################################################################
		// Override Methods DrawObject
		//###############################################################################################

		@Override
		public void draw(Graphics2D g2d) 
		{
			g2d.setColor(new Color(240,240,255));
			g2d.fillRect(0
					, 0
					, attrPanel.getWidth()
					, attrPanel.getHeight());
			
			g2d.setColor(Color.black);
			for(int i=0; i<session.getFr_premiseSize(); i++)
			{
				g2d.drawString(i+""
					, 10
					, box_Height/2 + box_Height*i - attrPadding_North);
				
				
				//min
				String valueStr = NumberFormater.doubleToString(session.getAttr_RangeAt(i)[0], 2);
				TextLayout textLayout = new TextLayout(valueStr, g2d.getFont(), g2d.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				
				g2d.drawString(valueStr
						, 2
						, box_Height* (i+1)  - 20 - attrPadding_North);
				
				//max
				valueStr = NumberFormater.doubleToString(session.getAttr_RangeAt(i)[1], 2);
				textLayout = new TextLayout(valueStr, g2d.getFont(), g2d.getFontRenderContext());
				bounds = textLayout.getBounds();
				
				g2d.drawString(valueStr
						, 2
						, box_Height*i + (int)bounds.getHeight() + 20 - attrPadding_North);
			}		
		}	
	}
	
	private class DrawRule extends DrawObject implements MouseInteractionObject{
		
		public int id;
		
		//###############################################################################################
		// Constructor
		//###############################################################################################
		
		public DrawRule(int id_P)
		{
			this.id = id_P;
		}

		//###############################################################################################
		// Override Methods DrawObject
		//###############################################################################################
		
		@Override
		public void draw(Graphics2D g2d)
		{
			int index = id_List.indexOf(id);
			
			boolean setGray;
			
			if((frCol.getGeneration() >= session.getRuleInfoAt(id).getDeath()
					&&  session.getRuleInfoAt(id).getDeath() > session.getRuleInfoAt(id).getBirth())
					|| frCol.getGeneration() < session.getRuleInfoAt(id).getBirth())
			{
				setGray = true;
			}
			else
			{
				setGray = false;
			}
			
			if(setGray)
			{			
				g2d.setColor(Color.gray);
			}
			else
			{
				g2d.setColor(session.getRuleInfoAt(id).getColor());
			}

			g2d.fillRoundRect((padding_H+rule_Width)*index
				, padding_V
				, rule_Width
				, rule_Height
				, 10
				, 10);
			
			g2d.setColor(Color.black);
			TextLayout textLayout = new TextLayout(id+"", g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString(id+""
					,(padding_H+rule_Width)*index + rule_Width/2 -(int)bounds.getCenterX()
					,padding_V + rule_Height/2-(int)bounds.getCenterY());			
		}
		
		//###############################################################################################
		// Override Methods MouseInteractionObject
		//###############################################################################################

		@Override
		public boolean action(MouseEvent me, int eventType) 
		{
			switch(eventType){
			
			//############ CLICKED #################################################################
				case CleverMouseListener.CLICKED: 	break;
			//############ ENTERED #################################################################
				case CleverMouseListener.ENTERED:	break;
			//############ EXITED  #################################################################
				case CleverMouseListener.EXITED:	break;		
			//############ PRESSED #################################################################
				case CleverMouseListener.PRESSED:	break;
			//############ RELEASED #################################################################		
				case CleverMouseListener.RELEASED: 	break;
			//############ DRAGGED #################################################################		
				case CleverMouseListener.DRAGGED: 	break;
			//############ MOVED   #################################################################
				case CleverMouseListener.MOVED:
					
					int index = id_List.indexOf(id);
					Point point = rulePanel.getLocationOnScreen();
					
					InteractionField[] i_Field = new InteractionField[1];
					i_Field[0] = new AttributeHistory_DeletInteraction(index, AttributeHistoryView.this);
					
					Interaction interaction = Interaction.getInstance(i_Field
							, null
							, point.x
							 + (padding_H+rule_Width)*index
							 + rule_Width/2
							, point.y
							 + padding_V
							 + rule_Height/2
							, 20
							, null);
					
					break;
				
			}
			
			return true;
		}

		@Override
		public boolean hitObject(MouseEvent me) 
		{
			int index = id_List.indexOf(id);
			
			if(me.getX() > (padding_H + rule_Width)*index
					&& me.getX() < rule_Width + (padding_H + rule_Width)*index
					&& me.getY() > padding_V
					&& me.getY() < padding_V + rule_Height)
			{
				return true;
			}
			return false;
		}		
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}

}
