package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.TreeSet;

import javax.swing.JPanel;

import listener.CleverMouseListener;

import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.DragHandler;
import guiUtil.DragSource;
import guiUtil.DropTarget;
import guiUtil.NumberFormater;

public class FamilyTreeView extends RuleVisualizer implements DragSource, AdjustmentListener{
	
	private DragHandler dragHandler;
	
	private int rule_Width = 20;
	private int rule_Height = 20;
	private int layer_Dis = 10;
	private int column_Dis = 10;
	
	private double scaleFactor = 1;
	
	private int layerCounter = 0;
	private int[] row_A = null;
	
	private TimeLineScale timeLineScale;
	private FamilyTree_PopUp ft_PopUp;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################

	public FamilyTreeView(Session session_P) {
		super(session_P, new DrawPanel());
		
		ft_PopUp = new FamilyTree_PopUp(this);
		
		cm_Listener.addToReleasedList(this);
		cm_Listener.addToPressedList(this);
		
		layerCounter = session.getMaxID();
				
		dragHandler = new DragHandler(this);
		drawPanel.setBackground(Color.white);
		
		scrollPane.getHorizontalScrollBar().addAdjustmentListener(this);
		scrollPane.getVerticalScrollBar().addAdjustmentListener(this);
		
		timeLineScale = new TimeLineScale(session, drawPanel);
		
		timeLineScale = new TimeLineScale(session, drawPanel);
		timeLineScale.setScale_Dis(rule_Width+column_Dis);
		timeLineScale.setScaleStart(column_Dis);
		
		JPanel paddingPanel = new JPanel();
		paddingPanel.setPreferredSize(new Dimension(0, 20));
		
		JPanel tlsWrapperPanel = new JPanel();
		tlsWrapperPanel.setLayout(new BorderLayout());
		tlsWrapperPanel.add(paddingPanel, BorderLayout.WEST);
		tlsWrapperPanel.add(timeLineScale.getComponent(), BorderLayout.CENTER);
		
		mainPanel.add(tlsWrapperPanel, BorderLayout.NORTH);
	}
	
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	protected void updateVisualisation() 
	{
		timeLineScale.updateVisualisation();
		drawPanel.setPreferredSize(new Dimension((int)((rule_Width+column_Dis)* (session.getLastPopIndex()+2)*scaleFactor)
											, (int)((rule_Height + layer_Dis)*(layerCounter+1)*scaleFactor)));
		drawPanel.revalidate();
	}
	
	@Override
	protected void invokeDrawMethods(Graphics2D g2d) 
	{
		g2d.scale(scaleFactor, scaleFactor);
		
		drawTimeMarker(g2d);
		drawFamilyTree(g2d);
		
	}
	
	//###############################################################################################
	// Override Methods CleverMouseListener
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) 
	{
		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
												
				if(me.isPopupTrigger())
				{	

					ft_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED: 
				
				if(me.isPopupTrigger())
				{	

					ft_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;
		}
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) 
	{
		return true;
	}
	
	
	//###############################################################################################
	// Override Methods DragSource
	//###############################################################################################

	@Override
	public Object[] getData(MouseEvent me) {
	
		//compute Index_row
		int row_Index = (int)((me.getY()-layer_Dis*scaleFactor)/((layer_Dis+rule_Height)*scaleFactor));
		//compute Index_column
		int column_Index = (int)((me.getX()-column_Dis*scaleFactor)/((column_Dis+rule_Width)*scaleFactor));
			
		if(row_A != null
			&& row_Index < row_A.length)
		{
			for(int i=0; i<row_A.length; i++)
			{
				if(row_A[i] == row_Index)
				{
					if(session.getRuleInfoAt(i).getBirth() == column_Index)
					{
						Integer[] data = this.selectRuleSubTreeMember(i);					
						return data;
					}
				}
			}
		}		
		return null;
	}

	@Override
	public void addMouseListener(MouseListener ml) 
	{
		this.drawPanel.addMouseListener(ml);		
	}

	@Override
	public void addMouseMotionListener(MouseMotionListener mml)
	{
		this.drawPanel.addMouseMotionListener(mml);		
	}

	@Override
	public void addDropTarget(DropTarget dt) 
	{
		this.dragHandler.addDropTarget(dt);		
	}

	@Override
	public void drawDrop(Graphics2D g2d, MouseEvent me, Object[] data) {
		
		for(int i=0; i<data.length; i++)
		{		
			g2d.setColor(session.getRuleInfoAt((Integer)data[i]).getColor());
			g2d.fillRoundRect((rule_Width+10)*i
					, 0
					, rule_Width
					, rule_Height
					, 10
					, 10);
			
			g2d.setColor(Color.white);
			TextLayout textLayout = new TextLayout((Integer)data[i]+"", g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString((Integer)data[i]+""
					, (rule_Width+10)*i + rule_Width/2 -(int)bounds.getCenterX()
					, rule_Height/2-(int)bounds.getCenterY());
		}
		
	}
	
	//###############################################################################################
	// Override Methods AdjustmentListener
	//###############################################################################################
	
	@Override
	public void adjustmentValueChanged(AdjustmentEvent arg0) {
		// TODO Auto-generated method stub
		
		if(arg0.getSource().equals(scrollPane.getHorizontalScrollBar()))
		{
			timeLineScale.scroll(arg0);
		}
		
		drawPanel.repaint();
	}
	
	//###############################################################################################
	// private draw Methods
	//###############################################################################################
	
	private void drawTimeMarker(Graphics2D g2d)
	{
		for(int i=0; i<=session.getLastPopIndex(); i++)
		{
			g2d.setColor(new Color(240,240,240));
			g2d.drawLine(column_Dis + (column_Dis+rule_Width)*i
				, 0
				, column_Dis + (column_Dis+rule_Width)*i
				, layer_Dis + (layer_Dis + rule_Height)*layerCounter);
		}		
	}
	
	private void drawFamilyTree(Graphics2D g2d)
	{
		ArrayList<Integer> 	visited_List = new ArrayList<Integer>();
		
		int maxID = session.getMaxID();
		int[] layer_A = new int[maxID+1];
		for(int i=0; i<layer_A.length; i++)
		{
			layer_A[i] = -1;
		}
		
		layerCounter = 0;	
		for(int i=maxID; i >= 0; i--)
		{
			if(!visited_List.contains(i))
			{
				drawSingleRule(g2d, visited_List, layer_A, i, -1);
				layerCounter++;
			}			
		}		
	}
	
	private void drawSingleRule(Graphics2D g2d
									, ArrayList<Integer> visited_List
									, int[] layer_A
									, int index
									, int child)
	{
		if(layer_A[index] == -1)
		{
			layer_A[index] = layerCounter;
			visited_List.add(index);
			
			g2d.setColor(session.getRuleInfoAt(index).getColor());
			g2d.fillRoundRect( column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth()
					, layer_Dis + (layer_Dis + rule_Height)*layerCounter
					, rule_Width
					, rule_Height
					, 10
					, 10);
			
			int colorBrightness = session.getRuleInfoAt(index).getColor().getRed()
					+ session.getRuleInfoAt(index).getColor().getGreen()
					+ session.getRuleInfoAt(index).getColor().getBlue();

			if((colorBrightness > 382 || session.getRuleInfoAt(index).getColor().getGreen() > 200)
					&& session.getRuleInfoAt(index).getColor().getGreen() > 150)
			{
				g2d.setColor(Color.black);
			}
			else
			{
				g2d.setColor(Color.white);
			}			
			TextLayout textLayout = new TextLayout(index+"", g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString(index+""
					, column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth() + rule_Width/2 -(int)bounds.getCenterX()
					, layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height/2-(int)bounds.getCenterY());
			
			//check if root is dead
			if(child == -1 )
			{	
				if(session.getRuleInfoAt(index).getBirth() < session.getRuleInfoAt(index).getDeath())
				{
					g2d.setColor(Color.red);
					g2d.drawLine(column_Dis + rule_Width+(column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth()
							, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter
							, column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath()
							, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter);
					
					g2d.setColor(Color.black);
					g2d.drawRoundRect( column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath()
							, layer_Dis + (layer_Dis + rule_Height)*layerCounter
							, rule_Width
							, rule_Height
							, 10
							, 10);
					
					g2d.setColor(Color.black);
					textLayout = new TextLayout(index+"", g2d.getFont(), g2d.getFontRenderContext());
					bounds = textLayout.getBounds();
					g2d.drawString(index+""
							, column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath() + rule_Width/2 -(int)bounds.getCenterX()
							, layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height/2-(int)bounds.getCenterY());
				}
				else
				{
					g2d.setColor(Color.blue);
					g2d.drawLine(column_Dis + rule_Width+(column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth()
							, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter
							, column_Dis + rule_Width + (column_Dis+rule_Width)*session.getLastPopIndex()
							, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter);
				}
			}
			
			
			boolean bothAncestorsChecked = false;
			
			if(session.getRuleInfoAt(index).getAncestors()[0] < index
					&& session.getRuleInfoAt(index).getAncestors()[0] != -1)
			{
				drawSingleRule(g2d, visited_List, layer_A, session.getRuleInfoAt(index).getAncestors()[0], index);
				
			}
			else if(session.getRuleInfoAt(index).getAncestors()[1] < index
					&& session.getRuleInfoAt(index).getAncestors()[1] != -1)
			{
				drawSingleRule(g2d, visited_List, layer_A, session.getRuleInfoAt(index).getAncestors()[1], index);
				bothAncestorsChecked = true;
			}
			
			if(session.getRuleInfoAt(index).getAncestors()[1] < index
					&& session.getRuleInfoAt(index).getAncestors()[1] != -1
					&& !bothAncestorsChecked)
			{

				layerCounter++;
				drawSingleRule(g2d, visited_List, layer_A, session.getRuleInfoAt(index).getAncestors()[1], index);
			}
		}
		
		if(child >= 0)
		{
			if(layer_A[child] > layer_A[index])
			{
				g2d.setColor(Color.GREEN);
			}
			else
			{
				g2d.setColor(Color.black);
			}
						
//			g2d.drawLine(column_Dis + rule_Width +(column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth()
//				, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index]
//				, column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth()
//				, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[child]);
			
			g2d.drawLine(column_Dis +(column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth()
			, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[child]
			, column_Dis +(column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth()
			, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index]);
			
			g2d.fillOval(column_Dis +(column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth() - 2
					, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index] - 2
					, 4
					, 4);
			
			g2d.setColor(Color.black);
			
			g2d.drawLine(column_Dis +(column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth()
			, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index]
			, column_Dis+ rule_Width + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth()
			, layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index]);
		}
		
		row_A = layer_A;
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################

	private Integer[] selectRuleSubTreeMember(int index)
	{
		LinkedList<Integer> member_List = new LinkedList<Integer>();
		
		Integer[] ret_A = null;
		
		addMember(member_List, index);
		
		ret_A = new Integer[member_List.size()];
		
		int i=0;
		for(Integer in: member_List)
		{
			ret_A[i] = in;
			i++;
		}
			
		return ret_A;
	}
	
	private void addMember(LinkedList<Integer> member_List, int index)
	{
		member_List.add(index);
		
		if(session.getRuleInfoAt(index).getAncestors()[0] < index
				&& session.getRuleInfoAt(index).getAncestors()[0] != -1)
		{
			addMember(member_List, session.getRuleInfoAt(index).getAncestors()[0]);
		}
		
		if(session.getRuleInfoAt(index).getAncestors()[1] < index
				&& session.getRuleInfoAt(index).getAncestors()[1] != -1)
		{
			addMember(member_List, session.getRuleInfoAt(index).getAncestors()[1]);
		}
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public String toLatex()
	{
		StringBuffer sb = new StringBuffer();
		
		int picHeight = (rule_Height + layer_Dis)*this.layerCounter;
		int picWidth =  (rule_Width+column_Dis) * session.getNewestCollectionIndex();
		
		sb.append("\\documentclass{article}");
		sb.append("\n");
		sb.append("\\usepackage{amsmath}");
		sb.append("\n");
		sb.append("\\usepackage{amsfonts}");
		sb.append("\n");
		sb.append("\\usepackage{pst-all}");
		sb.append("\n");
		sb.append("\\usepackage{pstricks}");
		sb.append("\n");
		sb.append("\\usepackage{pstricks-add}");
		sb.append("\n");
		sb.append("% preview must be loaded after pstricks");
		sb.append("\n");
		sb.append("\\usepackage[active,tightpage]{preview}");
		sb.append("\n");
		sb.append("\\PreviewEnvironment{pspicture}");
		sb.append("\n");
		sb.append("\\PreviewBorder=12pt%adjust the border if necessary, the default is 0.5bp");
		sb.append("\n");
		
		sb.append("\\begin{document}");
		sb.append("\n");
		
		sb.append("\\psset{xunit=1pt,yunit=1pt}");
		sb.append("\n");
		
		sb.append("\\begin{pspicture}" +
				"(0,0)" +
				"(" + picWidth + "," + picHeight + ")");
		sb.append("\n");
		
		sb.append("% Timemarker ####################################################");
		sb.append("\n");
		
		for(int i=0; i<=session.getLastPopIndex(); i++)
		{
			
			sb.append("\\psline[linestyle=solid, linecolor=lightgray!40]" +
					"(" + (column_Dis + (column_Dis+rule_Width)*i) + "," + 0 + ")" +
					"(" + (column_Dis + (column_Dis+rule_Width)*i) + "," + (layer_Dis + (layer_Dis + rule_Height)*layerCounter) + ")");
			sb.append("\n");
		}
		
		float[] rgb = new float[4];
		
		for(int i=0; i<=session.getMaxID(); i++)
		{		
			session.getRuleInfoAt(i).getColor().getRGBComponents(rgb);
			sb.append("\\newrgbcolor{"+ NumberFormater.integerToWord(i) +"}{"+rgb[0]+" "+rgb[1]+" "+rgb[2]+"}");
			sb.append("\n");
		}
		
		ArrayList<Integer> 	visited_List = new ArrayList<Integer>();
		
		int maxID = session.getMaxID();
		int[] layer_A = new int[maxID+1];
		for(int i=0; i<layer_A.length; i++)
		{
			layer_A[i] = -1;
		}
		layerCounter = 0;
		for(int i=maxID; i >= 0; i--)
		{
			if(!visited_List.contains(i))
			{
				singleRuleToLatex(visited_List, layer_A, i, -1, picHeight, sb);
				layerCounter++;
			}			
		}
		
		sb.append("\\end{pspicture}");
		sb.append("\n");
		sb.append("\\end{document}");
		
		return sb.toString();
	}
	
	public void singleRuleToLatex(ArrayList<Integer> visited_List
														, int[] layer_A
														, int index
														, int child
														, int picHeight
														, StringBuffer sb)
	{
		
		if(layer_A[index] == -1)
		{
			layer_A[index] = layerCounter;
			visited_List.add(index);
			sb.append("% Rule "+index+"############################################################");
			sb.append("\n");
			sb.append("\\psframe[framearc = 0.4, fillstyle=solid, fillcolor =" + NumberFormater.integerToWord(index) + "]" +
					"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth()) 
								+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter)) +")" +
					"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth() + rule_Width)
								+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height)) + ")");
			sb.append("\n");
			
			int colorBrightness = session.getRuleInfoAt(index).getColor().getRed()
									+ session.getRuleInfoAt(index).getColor().getGreen()
									+ session.getRuleInfoAt(index).getColor().getBlue();

			if((colorBrightness > 382 || session.getRuleInfoAt(index).getColor().getGreen() > 200)
					&& session.getRuleInfoAt(index).getColor().getGreen() > 150)
			{
				sb.append("\\rput" +
						"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth() + rule_Width/2)
									+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height/2)) + ")" +
						"{" + "\\large{" + index + "}}");
			}
			else
			{
				sb.append("\\rput" +
						"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth() + rule_Width/2)
									+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height/2)) + ")" +
						"{" + "\\large{" + "\\textcolor{white}{" + index + "}}}");
			}
			sb.append("\n");
			
			//check if root is dead
			if(child == -1 )
			{	
				if(session.getRuleInfoAt(index).getBirth() < session.getRuleInfoAt(index).getDeath())
				{
					sb.append("\\psline[linestyle=solid, linecolor=red]{-}" +
							"(" + (column_Dis + rule_Width+(column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth())
									+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter)) + ")" +
							"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath())
									+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter)) + ")");
					sb.append("\n");
				
					sb.append("\\psframe[framearc = 0.4, linecolor = black]" +
							"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath()) 
										+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter)) +")" +
							"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath() + rule_Width)
										+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height)) + ")");
					sb.append("\n");

					sb.append("\\rput" +
							"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getDeath() + rule_Width/2)
										+ "," + (picHeight - (layer_Dis + (layer_Dis + rule_Height)*layerCounter + rule_Height/2)) + ")" +
							"{" + "\\large{" + index + "}}");
				}
				else
				{
					sb.append("\\psline[linestyle=solid, linecolor=blue]{-}" +
							"(" + (column_Dis + rule_Width+(column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth())
									+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter)) + ")" +
							"(" + (column_Dis + rule_Width/2 + (column_Dis+rule_Width)*session.getNewestCollectionIndex())
									+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layerCounter)) + ")");
					sb.append("\n");
				}
			}
			
			
			boolean bothAncestorsChecked = false;
			
			if(session.getRuleInfoAt(index).getAncestors()[0] < index
					&& session.getRuleInfoAt(index).getAncestors()[0] != -1)
			{
				singleRuleToLatex(visited_List, layer_A, session.getRuleInfoAt(index).getAncestors()[0], index, picHeight, sb);
				
			}
			else if(session.getRuleInfoAt(index).getAncestors()[1] < index
					&& session.getRuleInfoAt(index).getAncestors()[1] != -1)
			{
				singleRuleToLatex(visited_List, layer_A, session.getRuleInfoAt(index).getAncestors()[1], index, picHeight,sb);
				bothAncestorsChecked = true;
			}
			
			if(session.getRuleInfoAt(index).getAncestors()[1] < index
					&& session.getRuleInfoAt(index).getAncestors()[1] != -1
					&& !bothAncestorsChecked)
			{

				layerCounter++;
				singleRuleToLatex(visited_List, layer_A, session.getRuleInfoAt(index).getAncestors()[1], index, picHeight,sb);
			}
		}
		
		if(child >= 0)
		{
			if(layer_A[child] > layer_A[index])
			{
				sb.append("\\psline[linestyle=solid, linecolor=green]{-}" +
						"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth())
								+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[child])) + ")" +
						"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth())
								+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index])) + ")");
				sb.append("\n");
				
				sb.append("\\pscircle[linestyle = none, fillstyle = solid, fillcolor= green]" +
						"(" + (column_Dis +(column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth()) + "," 
								+ (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index])) + ")" +
						"{" + 2 +"pt}");	
				sb.append("\n");
			}
			else
			{
				sb.append("\\psline[linestyle=solid, linecolor=black]{-}" +
						"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth())
								+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[child])) + ")" +
						"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth())
								+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index])) + ")");
				sb.append("\n");
				
				sb.append("\\pscircle[linestyle = none, fillstyle = solid, fillcolor= black]" +
						"(" + (column_Dis +(column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth()) + "," 
								+ (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index])) + ")" +
						"{" + 2 +"pt}");	
				sb.append("\n");
			}
			
			sb.append("\\psline[linestyle=solid, linecolor=black]{-}" +
					"(" + (column_Dis + (column_Dis+rule_Width)*session.getRuleInfoAt(child).getBirth())
							+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index])) + ")" +
					"(" + (column_Dis + rule_Width + (column_Dis+rule_Width)*session.getRuleInfoAt(index).getBirth())
							+ "," + (picHeight - (layer_Dis + rule_Height/2 + (layer_Dis + rule_Height)*layer_A[index])) + ")");
			sb.append("\n");
		}
		
		row_A = layer_A;											
	}
	
	public void zoomIN(){
		
		scaleFactor = scaleFactor*1.1;
		timeLineScale.setScaleFactor(scaleFactor);
		timeLineScale.repaintDrawPanel();
		drawPanel.repaint();
		
		int width = (int)((rule_Width+column_Dis)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_Height+column_Dis)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
	}
	
	public void zoomOut(){
		
		scaleFactor = scaleFactor/1.1;
		timeLineScale.setScaleFactor(scaleFactor);
		timeLineScale.repaintDrawPanel();
		drawPanel.repaint();
		
		int width = (int)((rule_Width+column_Dis)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_Height+column_Dis)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
	}


	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}
}
