package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.util.TreeSet;

import guiCore.RuleVisualizer;
import guiCore.Session;

public class FisDifMapView extends RuleVisualizer{
	
	private int map_Width 	= 400;
	private int map_Height 	= 400;
	
	private int resolutionX = 50;
	private int resolutionY = 50;
	
	private double maxPredValue = 0;
	private double minPredValue = 0;
	
	private double[] protoDatapoint;
	private int selectedAttr_1;


	private int selectedAttr_2;
	
	private int selectedFIS_1 = 0;
	private int selectedFIS_2 = 0;
	
	private FisDifMap_ToolBar fdm_ToolBar;
	private FisDifMap_SliderPanel fdm_SliderPanel;
	

	//###############################################################################################
	// Constructor
	//###############################################################################################

	public FisDifMapView(Session session_P) {
		super(session_P, new DrawPanel());
		
		protoDatapoint = new double[session.getFr_premiseSize()];
		for(int i=0; i<protoDatapoint.length; i++)
		{
			protoDatapoint[i] = session.getProtoDatapoint()[i];
		}
		
		fdm_ToolBar = new FisDifMap_ToolBar(this);
		fdm_SliderPanel = new FisDifMap_SliderPanel(this, session);
		
		mainPanel.add(fdm_ToolBar, BorderLayout.NORTH);
		mainPanel.add(fdm_SliderPanel, BorderLayout.EAST);
	}
	
	//###############################################################################################
	// Override Methods CleverMouseListener
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) {
		// TODO Auto-generated method stub
		return false;
	}
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	protected void updateVisualisation() 
	{
		fdm_ToolBar.setSpinnerMaxValue(session.getLastPopIndex());
		this.findMaxMinPredDiff();
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2d) {
		
		if(selectedFIS_1 > -1 && selectedFIS_2 > -1)
		{
			drawMap(g2d);
		}
		else
		{
//			drawSelectOrder(Graphics2D g2d); TODO
		}
	}
	
	//###############################################################################################
	// private Draw-Methods
	//###############################################################################################
	
	private void drawMap(Graphics2D g2d)
	{
		double origAttrValue_1 = protoDatapoint[selectedAttr_1];
		double origAttrValue_2 = protoDatapoint[selectedAttr_2];
		
		double rangeStep_1 = (session.getAttr_RangeAt(selectedAttr_1)[1] 
				- session.getAttr_RangeAt(selectedAttr_1)[0])/resolutionX;
		double rangeStep_2 = (session.getAttr_RangeAt(selectedAttr_2)[1] 
						- session.getAttr_RangeAt(selectedAttr_2)[0])/resolutionY;
		
		int width = (int)(map_Width/resolutionX);
		int height = (int)(map_Height/resolutionY);
		
		for(int i=0; i<resolutionX; i++)
		{
			for(int j=0; j<resolutionY; j++)
			{
				protoDatapoint[selectedAttr_1] = session.getAttr_RangeAt(selectedAttr_1)[0] + rangeStep_1/2 + rangeStep_1 * i;
				protoDatapoint[selectedAttr_2] = session.getAttr_RangeAt(selectedAttr_2)[0] + rangeStep_2/2 + rangeStep_2 * j;
				
				//compute Color
				Color color = new Color(255,255,255);;
				//1. compute prediction differenz
				double predDiff = session.getFuzzyRuleCollectionAt(selectedFIS_1).getPrediction(protoDatapoint)
									- session.getFuzzyRuleCollectionAt(selectedFIS_2).getPrediction(protoDatapoint);
				
				if(predDiff < 0)
				{
					float faktor = (float)(predDiff/this.minPredValue);
					
					color = new Color(faktor, faktor, 1f);
				}
				else if(predDiff > 0)
				{
					float faktor = (float)(predDiff/this.maxPredValue);
					
					color = new Color(1f, faktor, faktor);
				}				
				g2d.setColor(color);
				g2d.fillRect(width * i
						, height * j
						, width
						, height);
			}
		}
		
		protoDatapoint[selectedAttr_1] = origAttrValue_1;
		protoDatapoint[selectedAttr_2] = origAttrValue_2;
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void buildSilder()
	{
		this.fdm_SliderPanel.buildSlider();
	}
	
	public void findMaxMinPredDiff()
	{
		maxPredValue = 0;
		minPredValue = 0;
		double helpValue;
		
		double origAttrValue_1 = protoDatapoint[selectedAttr_1];
		double origAttrValue_2 = protoDatapoint[selectedAttr_2];
		
		double rangeStep_1 = (session.getAttr_RangeAt(selectedAttr_1)[1] 
										- session.getAttr_RangeAt(selectedAttr_1)[0])/resolutionX;
		double rangeStep_2 = (session.getAttr_RangeAt(selectedAttr_2)[1] 
										- session.getAttr_RangeAt(selectedAttr_2)[0])/resolutionY;
		
		for(int i=0; i<resolutionX; i++)
		{
			for(int j=0; j<resolutionY; j++)
			{
				protoDatapoint[selectedAttr_1] = session.getAttr_RangeAt(selectedAttr_1)[0] + rangeStep_1/2 + rangeStep_1 * i;
				protoDatapoint[selectedAttr_2] = session.getAttr_RangeAt(selectedAttr_2)[0] + rangeStep_2/2 + rangeStep_2 * j;
				
				helpValue = session.getFuzzyRuleCollectionAt(selectedFIS_1).getPrediction(protoDatapoint)
								- session.getFuzzyRuleCollectionAt(selectedFIS_2).getPrediction(protoDatapoint);
				
				if(helpValue > maxPredValue)
				{
					maxPredValue = helpValue;
				}
				else if(helpValue < minPredValue)
				{
					minPredValue = helpValue;
				}
			}
		}
		protoDatapoint[selectedAttr_1] = origAttrValue_1;
		protoDatapoint[selectedAttr_2] = origAttrValue_2;		
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public Session getSession()
	{
		return session;
	}
	
	public int getSelectedAttr_1() {
		return selectedAttr_1;
	}

	public void setSelectedAttr_1(int selectedAttr_1) {
		this.selectedAttr_1 = selectedAttr_1;
	}

	public int getSelectedAttr_2() {
		return selectedAttr_2;
	}

	public void setSelectedAttr_2(int selectedAttr_2) {
		this.selectedAttr_2 = selectedAttr_2;
	}

	public int getSelectedFIS_1() {
		return selectedFIS_1;
	}

	public void setSelectedFIS_1(int selectedFIS_1) {
		this.selectedFIS_1 = selectedFIS_1;
	}

	public int getSelectedFIS_2() {
		return selectedFIS_2;
	}

	public void setSelectedFIS_2(int selectedFIS_2) {
		this.selectedFIS_2 = selectedFIS_2;
	}

	public double[] getProtoDatapoint() {
		return protoDatapoint;
	}

	public void setProtoDatapoint(double[] protoDatapoint) {
		this.protoDatapoint = protoDatapoint;
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}

}
