package draw;

import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;

public class GaussPaint implements Paint{
	
	private float[] color;
	private double mean;
	private double width;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public GaussPaint(float[] color, double mean, double width)
	{
		this.color = color;
		this.mean = mean;
		this.width = width;
	}
	
	//###############################################################################################
	// Override Methods Paint
	//###############################################################################################

	@Override
	public int getTransparency() {
		
		return OPAQUE;
	}

	@Override
	public PaintContext createContext(ColorModel cm, Rectangle arg1,
			Rectangle2D arg2, AffineTransform atrans, RenderingHints arg4) {

		return new Context(cm, atrans);
	}
	
	//###############################################################################################
	// Inner Class Context
	//###############################################################################################
	
	class Context implements PaintContext{
		
		private AffineTransform transform;
		
		public Context(ColorModel cm_, AffineTransform atrans_)
		{
			this.transform = atrans_;
		}
		
		//###############################################################################################
		// Override Methods PaintContext
		//###############################################################################################

		@Override
		public void dispose() {
			
		}

		@Override
		public ColorModel getColorModel() {
			
			return ColorModel.getRGBdefault();
		}

		@Override
		public Raster getRaster(int xOffset, int yOffset, int w, int h) {

			WritableRaster raster = getColorModel().createCompatibleWritableRaster(w, h);
				
			for(int i=0; i<h; i++)
			{
				for(int j=0; j<w; j++)
				{					
					//transform mean
					double mean_t = mean+transform.getTranslateY();
					color[3] = gaussAlpha((i+yOffset), mean_t, width);
					
if(color[3] > 255) System.err.println("ERROR alpha grer 255 "+color[3]);
					raster.setPixel(j
									, i
									, color);
				}
			}
			
			return raster;
		}
		
		private float gaussAlpha(int x, double mean, double width)
		{
			float ret_Value = (float)(255*Math.exp(-0.5*(x-mean)*(x-mean)/(width*width)));
			
			return ret_Value;
		}
	}
}
