package draw;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import drawUtil.MathDraws;

public class GlobalPlot extends DrawObject{
	
	private String title;
	
	private ArrayList<ArrayList<Double>> value_LL = new ArrayList<ArrayList<Double>>();
	private ArrayList<Color> color_List = new ArrayList<Color>();
	
	public static int dis_H = 10;
	private int y_height = 100;
	
	private double currMax = Double.MIN_VALUE;
	
	private AffineTransform aTrans = new AffineTransform();
	
	public GlobalPlot(String title_P, int numPlots)
	{
		title = title_P;
		
		for(int i=0; i < numPlots; i++)
		{
			value_LL.add(new ArrayList<Double>());
		}
		
		color_List.add(Color.blue);
	}
	
	//###############################################################################################
	// Override Method DrawObject
	//###############################################################################################

	@Override
	public void draw(Graphics2D g2d) 
	{
		g2d.transform(aTrans);
		
		drawAxes(g2d);
		drawPlot(g2d);					
	}
	
	//###############################################################################################
	// private draw Methods
	//###############################################################################################
	
	private void drawPlot(Graphics2D g2d)
	{
		for(int j=0; j<value_LL.size(); j++)
		{	
			g2d.setColor(color_List.get(j));
			
			int yValue_0;
			int yValue_1;
			
			for(int i=1; i<value_LL.get(j).size(); i++)
			{			
				yValue_0 = y_height - (int)(value_LL.get(j).get(i-1) / currMax * y_height); 
				yValue_1 = y_height - (int)(value_LL.get(j).get(i) / currMax * y_height); 
				
				g2d.drawLine(dis_H * (i-1)
						, yValue_0
						, dis_H * i
						, yValue_1);
			}
		}
	}
	
	private void drawAxes(Graphics2D g2d)
	{		
		g2d.setColor(Color.black);
		g2d.drawString(title
				, (value_LL.get(0).size() - 1)*dis_H/2
				, -10);
		
		g2d.setColor(Color.white);
		g2d.fillRect(0
				, 0
				, dis_H * (value_LL.get(0).size()-1)
				, y_height);

		
		MathDraws.drawSimpleAxis(g2d
				, (value_LL.get(0).size() - 1)*dis_H
				, y_height
				, new double[]{0, value_LL.get(0).size()-1, 0, currMax}
				, new int[]{(value_LL.get(0).size()-1)*dis_H / 60,5});
	}

	//###############################################################################################
	// GET and SET
	//###############################################################################################

	public ArrayList<ArrayList<Double>> getValue_List() {
		return value_LL;
	}

	public void setValue_List(ArrayList<Double> value_List, double maxValue, int index) 
	{
		this.value_LL.set(index, value_List);
		currMax = maxValue;
	}
	
	public void setColor(Color color, int index)
	{
		color_List.set(index, color);
	}
	
	public void addColor(Color color)
	{
		color_List.add(color);
	}

	public AffineTransform getaTrans() {
		return aTrans;
	}

	public void setaTrans(AffineTransform aTrans) {
		this.aTrans = aTrans;
	}
	
}