package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.TreeSet;

import listener.CleverMouseListener;
import math.Metric;

import fuzzy.FuzzyMath;
import fuzzy.FuzzyRule;
import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.DropTarget;

public class LocalWalkView extends RuleVisualizer implements DropTarget{
	
	private double distanceOfOne = 20;
	private int diameter = 5;
	private int translateX = 0;
	private int translateY = 0;
	private int startX = 0;
	private int startY = 0;
	
	private int rule_Width = 20;
	private int rule_Height = 20;
	private int padding_H = 10;
	private int padding_V = 10;
	
	private boolean[] enabledAttr_A;
	private int distanceType = 0;
	private int numHLSamples = 100;
	
	private boolean showWidth = true;
	
	LocalWalk_PopUp lw_PopUp;
	
	private int id;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################

	public LocalWalkView(Session session_P) {
		super(session_P, new DrawPanel());
		
		lw_PopUp = new LocalWalk_PopUp(this);
		
		cm_Listener.addToReleasedList(this);
		cm_Listener.addToPressedList(this);
		cm_Listener.addToDraggedList(this);
		
		enabledAttr_A = new boolean[session.getFr_premiseSize()];
		
		for(int i=0; i<enabledAttr_A.length; i++)
		{
			enabledAttr_A[i] = true;
		}
		
		LocalWalkView_ToolBar lwv_ToolBar = new LocalWalkView_ToolBar(this);
		mainPanel.add(lwv_ToolBar, BorderLayout.NORTH);
	}
	
	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) {
		

		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
												
				if(me.isPopupTrigger())
				{	

//					lw_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				else
				{
					startX = me.getX();
					startY = me.getY();
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED: 
				
				if(me.isPopupTrigger())
				{	

//					lw_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	
				translateX = translateX-(startX-me.getX());
				translateY = translateY-(startY-me.getY());
				
				startX = me.getX();
				startY = me.getY();
				break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;
		}
		
		drawPanel.repaint();
		
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) {
		// TODO Auto-generated method stub
		return true;
	}
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	protected void updateVisualisation() {
		
		
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2d) {
		
		g2d.translate(translateX, translateY);
		drawPolarAxis(g2d);
		drawWalk(g2d);
//		drawCenter(g2d);
		g2d.translate(-translateX, -translateY);
		drawRule(g2d);
	}
	
	//###############################################################################################
	// Override Methods DropTarget
	//###############################################################################################

	@Override
	public boolean hit(MouseEvent me) 
	{
		Rectangle visible_rect = drawPanel.getVisibleRect();
		
		if(drawPanel.isShowing()		
				&& me.getXOnScreen() < drawPanel.getLocationOnScreen().getX()+visible_rect.getWidth()
				&& me.getXOnScreen() > drawPanel.getLocationOnScreen().getX()
				&& me.getYOnScreen() < drawPanel.getLocationOnScreen().getY()+visible_rect.getHeight()
				&& me.getYOnScreen() > drawPanel.getLocationOnScreen().getY())
		{
			return true;
		}		
		return false;
	}

	@Override
	public void drop(Object[] data) 
	{
		if(data[0] instanceof Integer)
		{
			id = (Integer)data[0];
			
			drawPanel.repaint();
		}		
	}
	
	//###############################################################################################
	// private Draw-Methods
	//###############################################################################################
	
	private void drawCenter(Graphics2D g2d)
	{
		double[] c_1 = null;
		double[] c = null;
		double[] c_s = null;
		
		if(session.getCurrPopulationIndex() >= 0)
		{
			c_1 = session.getFuzzyRuleCollectionAt(0).getFuzzyRule(id).getCenter();
		}
		
		g2d.setColor(Color.red);
		for(int i=1; i <= session.getCurrPopulationIndex(); i++)
		{
			c = session.getFuzzyRuleCollectionAt(i).getFuzzyRule(id).getCenter();
			c_s = session.getFuzzyRuleCollectionAt(i-1).getFuzzyRule(id).getCenter();
			
			g2d.drawLine((int)((c[0]-c_1[0])*distanceOfOne+drawPanel.getWidth()/2-diameter/2)
					, (int)((c[1]-c_1[1])*distanceOfOne+drawPanel.getHeight()/2-diameter/2)
					, (int)((c_s[0]-c_1[0])*distanceOfOne+drawPanel.getWidth()/2-diameter/2)
					, (int)((c_s[1]-c_1[1])*distanceOfOne+drawPanel.getHeight()/2-diameter/2));
			
			g2d.drawOval((int)((c[0]-c_1[0])*distanceOfOne+drawPanel.getWidth()/2-diameter/2)
					, (int)((c[1]-c_1[1])*distanceOfOne+drawPanel.getHeight()/2-diameter/2)
					, diameter
					, diameter);
		}
	}
	
	private void drawWalk(Graphics2D g2d)
	{
		FuzzyRule fr_1 		= null;	//first
		FuzzyRule fr_2 		= null;	//last
		FuzzyRule fr_curr 	= null;	//current
		FuzzyRule fr_m		= null; //magnet
		
		Point.Double p_1 	= new Point.Double();
		Point.Double p_2 	= new Point.Double();
		Point.Double p_curr = new Point.Double();
		Point.Double p_m 	= new Point.Double();
		
		double dis_1 		= 0;
		double dis_2 		= 0;
		double dis_m		= 0;
		double dis_s		= 0;
		
		double[] v_1;
		double[] v_2;
		
		double a_1_curr;
		double a_curr_1;
		double a_curr_2;
		double a_2_curr;

		g2d.setColor(Color.black);
		
		if(session.getCurrPopulationIndex() >= session.getRuleInfoAt(id).getBirth())
		{
			fr_1 = session.getFuzzyRuleCollectionAt(session.getRuleInfoAt(id).getBirth()).getFuzzyRule(id);
			
			p_1.setLocation(0
					, 0);
			
			g2d.drawOval((int)(p_1.getX()+drawPanel.getWidth()/2-diameter/2)
					, (int)(p_1.getY()+drawPanel.getHeight()/2-diameter/2)
					, diameter
					, diameter);			
		}
		
		if(session.getRuleInfoAt(id).getBirth() >= session.getRuleInfoAt(id).getDeath()
				|| session.getRuleInfoAt(id).getBirth()+1 < session.getRuleInfoAt(id).getDeath())
		{
			if(session.getCurrPopulationIndex() > session.getRuleInfoAt(id).getBirth())
			{
				fr_curr = session.getFuzzyRuleCollectionAt(session.getRuleInfoAt(id).getBirth()+1).getFuzzyRule(id);
				
				switch(distanceType)
				{
				case 0:
					dis_1 = Session.calcDistance(fr_1, fr_curr, enabledAttr_A);
					break;
				case 1:
					dis_1 = 1 - FuzzyRule.compSimilarityMD2(fr_1, fr_curr, enabledAttr_A);
					break;
				case 2:
					dis_1 = 1 - FuzzyRule.compSimilarityHL(fr_1
															, fr_curr
															, enabledAttr_A
															, session.getAttr_Range()
															, numHLSamples);
					break;
				}
				
				
				p_curr.setLocation(dis_1
						, 0);
				
				g2d.drawLine((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2)
						, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2)
						, (int)(p_1.getX() * distanceOfOne + drawPanel.getWidth()/2)
						, (int)(p_1.getY() + drawPanel.getHeight()/2));
				
				g2d.drawOval((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2-diameter/2)
						, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2-diameter/2)
						, diameter
						, diameter);
				
				//draw first width
				if(showWidth)
				{
					if(session.getCurrPopulationIndex() < session.getRuleInfoAt(id).getBirth()+2)
					{					
						v_1 = Metric.subtractVector(fr_1.getCenter(), fr_curr.getCenter());
						
						Metric.normalizeVector(v_1);
						
						a_1_curr = FuzzyMath.directedWidth(v_1, fr_1.getWidth(), enabledAttr_A);
						a_curr_1 = FuzzyMath.directedWidth(v_1, fr_curr.getWidth(), enabledAttr_A);
						
						g2d.setColor(Color.blue);
						g2d.drawOval((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2- a_curr_1/2*distanceOfOne)
								, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2-a_curr_1/2*distanceOfOne)
								, (int)(a_curr_1 * distanceOfOne)
								, (int)(a_curr_1 * distanceOfOne));
						
						g2d.setColor(Color.yellow);
						g2d.drawOval((int)(p_1.getX() * distanceOfOne + drawPanel.getWidth()/2- a_1_curr/2*distanceOfOne)
								, (int)(p_1.getY() * distanceOfOne + drawPanel.getHeight()/2 - a_1_curr/2*distanceOfOne)
								, (int)(a_1_curr * distanceOfOne)
								, (int)(a_1_curr * distanceOfOne));
					}
				}
				
				fr_m = fr_1;
				fr_2 = fr_curr;
				
				p_m = p_1;
				p_2 = p_curr;
			}
			
			int top = session.getCurrPopulationIndex();
			
			if(top > session.getRuleInfoAt(id).getDeath()-1
					&& session.getRuleInfoAt(id).getDeath() > 0)
			{
				top = session.getRuleInfoAt(id).getDeath()-1;
			}
			
			for(int i=session.getRuleInfoAt(id).getBirth()+2; i <= top; i++)
			{						
				fr_curr = session.getFuzzyRuleCollectionAt(i).getFuzzyRule(id);
				
				switch(distanceType)
				{
				case 0:
					dis_1 = Session.calcDistance(fr_1, fr_curr, enabledAttr_A);
					dis_2 = Session.calcDistance(fr_2, fr_curr, enabledAttr_A);
					dis_m = Session.calcDistance(fr_curr, fr_m, enabledAttr_A);
					dis_s = Session.calcDistance(fr_1, fr_2, enabledAttr_A);
					break;
				case 1:
					dis_1 = 1 - FuzzyRule.compSimilarityMD2(fr_1, fr_curr, enabledAttr_A);
					dis_2 = 1 - FuzzyRule.compSimilarityMD2(fr_2, fr_curr, enabledAttr_A);
					dis_m = 1 - FuzzyRule.compSimilarityMD2(fr_curr, fr_m, enabledAttr_A);
					dis_s = 1 - FuzzyRule.compSimilarityMD2(fr_1, fr_2, enabledAttr_A);
					break;
				case 2:
					dis_1 = 1 - FuzzyRule.compSimilarityHL(fr_1, fr_curr, enabledAttr_A, session.getAttr_Range(), numHLSamples);
					dis_2 = 1 - FuzzyRule.compSimilarityHL(fr_2, fr_curr, enabledAttr_A, session.getAttr_Range(), numHLSamples);
					dis_m = 1 - FuzzyRule.compSimilarityHL(fr_curr, fr_m, enabledAttr_A, session.getAttr_Range(), numHLSamples);
					dis_s = 1 - FuzzyRule.compSimilarityHL(fr_1, fr_2, enabledAttr_A, session.getAttr_Range(), numHLSamples);
					break;
				}
				
				

				if(dis_2 == 0)
				{
					p_curr = p_2;
				}
				else if(dis_1 == 0)
				{
					p_curr = p_1;
				}
				else if(checkIntersect(p_1.x
						, p_1.y
						, p_2.x
						, p_2.y
						, dis_1
						, dis_2))
				{	
					Point.Double[] intersectPoints;
					//TODO while.. falls p_1 == p_2 muss auf p_2-1 usw. zurckgegriffen werden.
					if(p_1.x == p_2.x && p_1.y == p_2.y)
					{							
						intersectPoints = LocalWalkView.intersectCircle(p_1.x
																			, p_1.y
																			, p_m.x
																			, p_m.y
																			, dis_1
																			, dis_m);
						
						if(intersectPoints == null)
						{
							p_curr.x = dis_1;
							p_curr.y = 0;
						}
						else
						{
							p_curr = intersectPoints[0];
						}						
					}
					else
					{
						intersectPoints = LocalWalkView.intersectCircle(p_1.x
																		, p_1.y
																		, p_2.x
																		, p_2.y
																		, dis_1
																		, dis_2);
					
						p_curr = nearestPoint(intersectPoints, p_m, dis_m);
						
						//Caused by numerical problems
						if(new Double(p_curr.x).isNaN())
						{
							p_curr = p_2;
						}
					}
				}
				else
				{
					p_curr = pointOnLine(p_1.x
											, p_1.y
											, p_2.x
											, p_2.y
											, dis_1);
				}
				
				if(i > top - 2)
				{
					g2d.setColor(Color.black);
				}
				else
				{
					g2d.setColor(Color.LIGHT_GRAY);
				}
				
				g2d.drawLine((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2)
						, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2)
						, (int)(p_2.getX() * distanceOfOne + drawPanel.getWidth()/2)
						, (int)(p_2.getY() * distanceOfOne + drawPanel.getHeight()/2));
				
				g2d.fillOval((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2-diameter/2)
						, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2-diameter/2)
						, diameter
						, diameter);
				
				if(i == top)
				{
					if(dis_1 > dis_s)
					{
						g2d.setColor(Color.red);
					}
					else
					{
						g2d.setColor(Color.blue);
					}
					
					g2d.drawLine((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2)
							, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2)
							, (int)(p_1.getX() * distanceOfOne + drawPanel.getWidth()/2)
							, (int)(p_1.getY() * distanceOfOne + drawPanel.getHeight()/2));
					
					g2d.setColor(Color.green);
					g2d.drawLine((int)(p_2.getX() * distanceOfOne + drawPanel.getWidth()/2)
							, (int)(p_2.getY() * distanceOfOne + drawPanel.getHeight()/2)
							, (int)(p_1.getX() * distanceOfOne + drawPanel.getWidth()/2)
							, (int)(p_1.getY() * distanceOfOne + drawPanel.getHeight()/2));
					
					//draw Width
					if(showWidth)
					{
						v_1 = Metric.subtractVector(fr_1.getCenter(), fr_curr.getCenter());
						v_2 = Metric.subtractVector(fr_curr.getCenter(), fr_2.getCenter());
						
						Metric.normalizeVector(v_1);
						Metric.normalizeVector(v_2);
						
						a_1_curr = FuzzyMath.directedWidth(v_1, fr_1.getWidth(), enabledAttr_A);
						a_curr_2 = FuzzyMath.directedWidth(v_2, fr_curr.getWidth(), enabledAttr_A);
						a_2_curr = FuzzyMath.directedWidth(v_2, fr_2.getWidth(), enabledAttr_A);
						
						g2d.setColor(Color.blue);
						g2d.drawOval((int)(p_curr.getX() * distanceOfOne + drawPanel.getWidth()/2 - a_curr_2/2*distanceOfOne)
								, (int)(p_curr.getY() * distanceOfOne + drawPanel.getHeight()/2 - a_curr_2/2*distanceOfOne)
								, (int)(a_curr_2 * distanceOfOne)
								, (int)(a_curr_2 * distanceOfOne));
						
						g2d.setColor(Color.green);
						g2d.drawOval((int)(p_2.getX() * distanceOfOne + drawPanel.getWidth()/2 - a_2_curr/2*distanceOfOne)
								, (int)(p_2.getY() * distanceOfOne + drawPanel.getHeight()/2 - a_2_curr/2*distanceOfOne)
								, (int)(a_2_curr * distanceOfOne)
								, (int)(a_2_curr * distanceOfOne));
						
						g2d.setColor(Color.yellow);
						g2d.drawOval((int)(p_1.getX()* distanceOfOne + drawPanel.getWidth()/2- a_1_curr/2*distanceOfOne)
								, (int)(p_1.getY() * distanceOfOne  + drawPanel.getHeight()/2-a_1_curr/2*distanceOfOne)
								, (int)(a_1_curr * distanceOfOne)
								, (int)(a_1_curr * distanceOfOne));
					}
				}
							
				fr_m = fr_2;
				fr_2 = fr_curr;
				
				p_m = p_2;
				p_2 = p_curr;
				
			}
		}
	}
	
	private void drawRule(Graphics2D g2d)
	{
		g2d.setColor(session.getRuleInfoAt(id).getColor());
		g2d.fillRoundRect(padding_H
			, padding_V
			, rule_Width
			, rule_Height
			, 10
			, 10);
		
		g2d.setColor(Color.black);
		TextLayout textLayout = new TextLayout(id+"", g2d.getFont(), g2d.getFontRenderContext());
		Rectangle2D bounds = textLayout.getBounds();
		g2d.drawString(id+""
				,padding_H + rule_Width/2 -(int)bounds.getCenterX()
				,padding_V + rule_Height/2-(int)bounds.getCenterY());
	}
	
	private void drawPolarAxis(Graphics2D g2d)
	{
		int diameter = 50;
		
		g2d.setColor(Color.white);
		for(int i=1; i<25; i++)
		{
			g2d.drawOval(drawPanel.getWidth()/2-(diameter/2)*i
					, drawPanel.getHeight()/2-(diameter/2)*i
					, diameter*i
					, diameter*i);
		}
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public Point.Double nearestPoint(Point.Double[] p_A, Point.Double p_m, double dis_m)
	{				
		double dis_1 = Point.distance(p_A[0].x, p_A[0].y, p_m.x, p_m.y);
		double dis_2 = Point.distance(p_A[1].x, p_A[1].y, p_m.x, p_m.y);
		
		double dis_dis_1 = Math.abs(dis_1-dis_m);
		double dis_dis_2 = Math.abs(dis_2-dis_m);
		
		if(dis_dis_1 < dis_dis_2)
		{
			return p_A[0];
		}
		else
		{
			return p_A[1];
		}
	}
	
	public static boolean checkIntersect(double x1, double y1, double x2, double y2, double r1, double r2)
	{
		double dis = Metric.euclidean(new double[]{x1,y1}, new double[]{x2,y2});
		
		if(dis > r1+r2) return false;
		
		return true;
	}
	
	public static Point.Double pointOnLine(double x1, double y1, double x2, double y2, double r1)
	{
		Point.Double p = new Point.Double();
		
		double[] v_1 = Metric.subtractVector(new double[]{x1,y1}, new double[]{x2,y2});
		Metric.normalizeVector(v_1);
		
		p.x = v_1[0] * r1;
		p.y = v_1[1] * r1;
		
		return p;
	}
	
	public static Point.Double[] intersectCircle(double x1, double y1, double x2, double y2, double r1, double r2)
	{
		Point.Double[] p = new Point.Double[2];
		
		p[0] = new Point.Double();
		p[1] = new Point.Double();
		
		if(x1 != x2)
		{
			double a = (y2-y1)/(x2-x1);
			double b = ((r1*r1-r2*r2)-(x1*x1-x2*x2)-(y1*y1-y2*y2))/(2*(x2-x1));
		
			double e = a*a+1;
			double f = -2*y1-2*a*(b-x1);
			double g = y1*y1+(b-x1)*(b-x1)-r1*r1;
			
			p[0].y = (-f + Math.sqrt(f*f-4*g*e))/(2*e);
			p[1].y = (-f - Math.sqrt(f*f-4*g*e))/(2*e);
		
			p[0].x = b - p[0].y*a;
			p[1].x = b - p[1].y*a;
		}
		else if(y1 != y2)
		{

			double a = (x2-y1)/(y2-y1);
			double b = ((r1*r1-r2*r2)-(y1*y1-y2*y2)-(x1*x1-x2*x2))/(2*(y2-y1));
		
			double e = a*a+1;
			double f = -2*x1-2*a*(b-y1);
			double g = x1*x1+(b-y1)*(b-y1)-r1*r1;
			
			p[0].y = (-f + Math.sqrt(f*f-4*g*e))/(2*e);
			p[1].y = (-f - Math.sqrt(f*f-4*g*e))/(2*e);
		
			p[0].y = b - p[0].x*a;
			p[1].y = b - p[1].x*a;
		}
		else
		{
			return null;
		}
		
		return p;
	}
	
	public void zoomIN(){
		
		distanceOfOne = distanceOfOne*1.1;
	}
	
	public void zoomOut(){
		
		distanceOfOne = distanceOfOne/1.1;
	}

	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public Session getSession()
	{
		return session;
	}

	public boolean[] getEnabledAttr_A() {
		return enabledAttr_A;
	}

	public void setEnabledAttr_A(boolean[] enabledAttr_A) {
		this.enabledAttr_A = enabledAttr_A;
	}

	public int getDistanceType() {
		return distanceType;
	}

	public void setDistanceType(int distanceType) {
		this.distanceType = distanceType;
	}

	public int getNumHLSamples() {
		return numHLSamples;
	}

	public void setNumHLSamples(int numHLSamples) {
		this.numHLSamples = numHLSamples;
	}

	public boolean isShowWidth() {
		return showWidth;
	}

	public void setShowWidth(boolean showWidth) {
		this.showWidth = showWidth;
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}
}
