package draw;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.TreeSet;

import listener.CleverMouseListener;
import math.Metric;

import fuzzy.FuzzyRule;
import guiCore.ColorGenerator;
import guiCore.RuleVisualizer;
import guiCore.Session;

public class MatrixView extends RuleVisualizer {
	
	private int rule_width = 20;
	private int rule_height = 20;	
	private int padding = 0;
	
	private int translateX = 0;
	private int translateY = 0;
	private int startX = 0;
	private int startY = 0;
	
	private float[] bottomColor = {0,0,0,1};
	private float[] topColor = {1,1,1,1};
	
	MatrixView_PopUp mv_popup;
	
	private int distanceType = 0;
	private int numHLSamples = 100;
	private boolean showAngle = false;
	private boolean outerMarks = false;
	
	//Zoom stuff
	private double scaleFactor = 1;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public MatrixView(Session session_P)
	{	
		super(session_P, new DrawPanel());
		
		mv_popup = new MatrixView_PopUp(this);
		
		cm_Listener.addToReleasedList(this);
		cm_Listener.addToPressedList(this);
		cm_Listener.addToDraggedList(this);
	}
		
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################
	
	@Override
	protected void invokeDrawMethods(Graphics2D g2D) 
	{	
		g2D.translate(translateX, translateY);
		g2D.translate(20, 20);
//		drawGrid(g2D);
		g2D.scale(scaleFactor, scaleFactor);
		drawMatrixViewRules(g2D);
		drawInnerMatrixFilds(g2D);
	}

	@Override
	protected void updateVisualisation()
	{		
	
	}
	
	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) 
	{
		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
												
				if(me.isPopupTrigger())
				{	

					mv_popup.show(me.getComponent(), me.getX(), me.getY());
				}
				else
				{
					startX = me.getX();
					startY = me.getY();
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED: 
				
				if(me.isPopupTrigger())
				{	

					mv_popup.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	
				translateX = translateX-(startX-me.getX());
				translateY = translateY-(startY-me.getY());
				
				startX = me.getX();
				startY = me.getY();
				break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;
		}
		drawPanel.repaint();
		
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) 
	{		
		return true;
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawInnerMatrixFilds(Graphics2D g2D)
	{	
		//first calc largest Distance
		double max_Dis = 0;
		double curr_Dis = 0;
		for(int i=0; i<frCol.collectionSize(); i++)
		{
			for(int j=0; j<frCol.collectionSize(); j++)
			{
				curr_Dis = Session.calcDistance(frCol.getFuzzyRuleAt(i), frCol.getFuzzyRuleAt(j));

				if(curr_Dis > max_Dis)
				{
					max_Dis = curr_Dis;
				}
			}
		}
		//draw filds
		AffineTransform aTrans = g2D.getTransform();
		AffineTransform bTrans;
		
		for(int i=0; i<frCol.collectionSize(); i++)
		{
			g2D.translate(0, (rule_height+padding*2)*(i+1));
			
			for(int j=0; j<frCol.collectionSize(); j++)
			{
				Color color = null;
				
				FuzzyRule fr = frCol.getFuzzyRuleAt(i);
				FuzzyRule fr_2 = frCol.getFuzzyRuleAt(j);
				
				switch(distanceType)
				{
				case 0: 
					curr_Dis = Session.calcDistance(fr, fr_2);
				
					color = ColorGenerator.calculateRuleDisColor(curr_Dis
																		, max_Dis
																		, topColor
																		, bottomColor);
					
					break;
					
				case 1:
					
					double sim = (FuzzyRule.compSimilarityErrorFunc(fr, fr_2, null));
					
					color = new Color((float)sim,(float)sim,(float)sim);
				
					break;
					
				case 4:
					sim = (FuzzyRule.compConsequensSim(fr, fr_2));
					
					color = new Color((float)sim,(float)sim,(float)sim);
					
					break;
				
				case 5:
					color = new Color(150,150,150);
					break;
				}
				
				g2D.setColor(color);
				
				g2D.translate(rule_width+padding*2, 0);
				
				g2D.fillRect(padding
						, padding
						, rule_width
						, rule_height);
				
				//Conclusion  Angle		
				if(showAngle)
				{
					double angle = FuzzyRule.compConsequensAngle(fr, fr_2);
					
					if(angle > Math.PI/2) angle = Math.PI - angle;
					
					bTrans = g2D.getTransform();
					g2D.translate(padding + rule_width/2, padding + rule_height/2);
					g2D.rotate(-angle);
					
					g2D.setColor(Color.red);
					g2D.drawLine( - (rule_width/2)
						, 0
						, (rule_height/2)
						, 0);
					
					g2D.setTransform(bTrans);
				}
				
			}
			g2D.setTransform(aTrans);
		}
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawMatrixViewRules(Graphics2D g2D)
	{
		AffineTransform aTrans = g2D.getTransform();
		//fill x-direction
		for(FuzzyRule fr: frCol.getFrList())
		{
			g2D.translate(rule_width+padding*2, 0);
			g2D.setColor(session.getRuleInfoAt(fr.getId()).getColor());
			g2D.fillRoundRect(padding
					, padding
					, rule_width
					, rule_height
					, 10
					, 10);
			
			g2D.setColor(Color.white);
			TextLayout textLayout = new TextLayout(fr.getId()+"", g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(fr.getId()+""
					, padding + rule_width/2 - (int)bounds.getCenterX()
					, padding + rule_height/2 - (int)bounds.getCenterY());
		}
		
		g2D.setTransform(aTrans);
		
		//fill y-direction
		for(FuzzyRule fr: frCol.getFrList())
		{
			g2D.translate(0, rule_height+padding*2);
			g2D.setColor(session.getRuleInfoAt(fr.getId()).getColor());
			g2D.fillRoundRect(padding
					, padding
					, rule_width
					, rule_height
					, 10
					, 10);
			
			g2D.setColor(Color.white);
			TextLayout textLayout = new TextLayout(fr.getId()+"", g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(fr.getId()+""
					, padding + rule_width/2 - (int)bounds.getCenterX()
					, padding + rule_height/2 - (int)bounds.getCenterY());
		}
		
		g2D.setTransform(aTrans);
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawGrid(Graphics2D g2D)
	{
		g2D.setColor(Color.black);
		for(int i=1; i <= frCol.collectionSize(); i++)
		{
			//horizontal lines
			g2D.drawLine(0
					, (rule_height+padding*2)*i
					, (rule_width+padding*2)*(frCol.collectionSize()+1)
					, (rule_height+padding*2)*i);
			
			//vertical lines
			g2D.drawLine((rule_width+padding*2)*i
					, 0
					, (rule_width+padding*2)*i
					, (rule_height+padding*2)*(frCol.collectionSize()+1));
		}
		
	}
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void drawImage(Graphics2D g2D, int gen)
	{		
		g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING
				, RenderingHints.VALUE_ANTIALIAS_ON );

		session.setCurrPopulationIndex(gen);
		super.refreshCollection(session.getCurrPopulation());
		updateVisualisation();	

//		drawGrid(g2D);
		g2D.scale(scaleFactor, scaleFactor);
		if(!outerMarks)
		{
			drawMatrixViewRules(g2D);
		}
		else
		{
			g2D.translate(-rule_width, -rule_height);
		}
		drawInnerMatrixFilds(g2D);		
	}
	
	public void drawMarksX(Graphics2D g2D, int xSize, int offset)
	{
		g2D.setColor(Color.black);
		int j=1;
		int xOffset = 0;
		for(int i=0; i<xSize; i++)
		{		
			for(FuzzyRule fr: session.getFuzzyRuleCollectionAt(i).getFrList())
			{
				TextLayout textLayout = new TextLayout(fr.getId()+"", g2D.getFont(), g2D.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				g2D.drawString(fr.getId()+""
						, padding + rule_width/2 - (int)bounds.getCenterX() + (rule_width)*j + xOffset
						, (padding + rule_height/2 - (int)bounds.getCenterY()));
				j++;
			}
			
			xOffset = xOffset + offset;
		}
	}
	
	public void drawMarksY(Graphics2D g2D, int ySize, int offset)
	{
		g2D.setColor(Color.black);
		int j=1;
		int yOffset = 0;
		for(int i=0; i<ySize; i++)
		{
			for(FuzzyRule fr: frCol.getFrList())
			{
				TextLayout textLayout = new TextLayout(fr.getId()+"", g2D.getFont(), g2D.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				g2D.drawString(fr.getId()+""
						, padding + rule_width/2 - (int)bounds.getCenterX()
						, padding + rule_height/2 - (int)bounds.getCenterY() + (rule_height)*j + yOffset);
				j++;
			}
			yOffset = yOffset + offset;
		}
	}
	
	public void zoomIN(){
		
		scaleFactor = scaleFactor*1.1;
		
		int width = (int)((rule_width+padding)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_height+padding)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();		
	}
	
	public void zoomOut(){
		
		scaleFactor = scaleFactor/1.1;
		
		int width = (int)((rule_width+padding)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_height+padding)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public void setRule_width(int width) {
		this.rule_width = width;
	}

	public void setRule_height(int height) {
		this.rule_height = height;
	}

	public int getRule_width() {
		return rule_width;
	}

	public int getRule_height() {
		return rule_height;
	}

	public int getPadding() {
		return padding;
	}

	public void setPadding(int padding) {
		this.padding = padding;
	}
	public int getNumHLSamples() {
		return numHLSamples;
	}

	public void setNumHLSamples(int numHLSamples) {
		this.numHLSamples = numHLSamples;
	}

	public boolean isShowAngle() {
		return showAngle;
	}

	public void setShowAngle(boolean showAngle) {
		this.showAngle = showAngle;
	}
	
	public int getDistanceType() {
		return distanceType;
	}

	public void setDistanceType(int distanceType) {
		this.distanceType = distanceType;
	}
	
	public Session getSession()
	{
		return super.session;
	}
	
	public void setOuterMarks(boolean set)
	{
		outerMarks = set;
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}




}
