package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.imageio.ImageIO;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

public class MatrixView_ToImageDialog extends JDialog implements ActionListener, DocumentListener {
	private MatrixView mv;
	
	JPanel mainPanel;
	
	JPanel middlePanel;
	JPanel centerPanel;
	
	JPanel matrixPanel;
	JPanel singlePanel;
	JPanel outerMarksPanel;
	JPanel textAreaPanel;
	
	JPanel buttonPanel;
	
	private JCheckBox asMatrixBox;
	private JSpinner maxSpinner;
	private JRadioButton rowRadio;
	private JRadioButton columnRadio;
	
	private JCheckBox singleBox;
	
	private JCheckBox outerMarks;
	
	private JTextArea genTextArea;
	
	private JButton okButton;
	private JButton cancelButton;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public MatrixView_ToImageDialog(MatrixView mv)
	{
		this.mv = mv;
		
		asMatrixBox = new JCheckBox("as matrix");
		
		rowRadio = new JRadioButton("row");
		rowRadio.setSelected(true);
		
		columnRadio = new JRadioButton("column");
		
		maxSpinner = new JSpinner(new SpinnerNumberModel(1,1,10000,1));
		maxSpinner.setPreferredSize(new Dimension(40,20));
		
		matrixPanel = new JPanel();
		matrixPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		matrixPanel.add(asMatrixBox);
		matrixPanel.add(rowRadio);
		matrixPanel.add(columnRadio);
		matrixPanel.add(maxSpinner);
		
		singleBox = new JCheckBox("single detention");
		
		singlePanel = new JPanel();
		singlePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		singlePanel.add(singleBox);
		
		outerMarks = new JCheckBox("reduced Marks");
		
		outerMarksPanel = new JPanel();
		outerMarksPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
		outerMarksPanel.add(outerMarks);
		
		middlePanel = new JPanel();
		middlePanel.setLayout(new GridLayout(2,1));
		middlePanel.add(singlePanel);
		middlePanel.add(outerMarksPanel);
	
		genTextArea = new JTextArea();
		genTextArea.getDocument().addDocumentListener(this);
		genTextArea.setLineWrap(true);
		genTextArea.setPreferredSize(new Dimension(150, 30));
		
		textAreaPanel = new JPanel();
		textAreaPanel.setLayout(new BorderLayout());
		textAreaPanel.setBorder(new TitledBorder("generations"));
		textAreaPanel.add(genTextArea, BorderLayout.CENTER);
	
		centerPanel = new JPanel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(matrixPanel, BorderLayout.NORTH);
		centerPanel.add(middlePanel, BorderLayout.CENTER);
		centerPanel.add(textAreaPanel, BorderLayout.SOUTH);
		
		okButton = new JButton("Ok");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		
		cancelButton = new JButton("cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		buttonPanel.add(cancelButton);
		buttonPanel.add(okButton);
		
		mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(centerPanel, BorderLayout.CENTER);
		mainPanel.add(buttonPanel, BorderLayout.SOUTH);
		
		Container cp = this.getContentPane();
		cp.setLayout(new BorderLayout());
		cp.add(mainPanel, BorderLayout.CENTER);
		
		this.setSize(300, 350);
		this.setLocation((int)((this.getToolkit().getScreenSize().getWidth() - this.getWidth())/2),
				(int)((this.getToolkit().getScreenSize().getHeight() - this.getHeight())/2));
		this.setVisible(true);
		
	}
	
	private TreeSet<Integer> parseGenText(String text)
	{
		TreeSet<Integer> ret_List = new TreeSet<Integer>();
		
		String[] preNumbers = text.split(",");
		String[] number;
		
		String[] onlyEvery = preNumbers[preNumbers.length-1].split(":");
		int mod = 1;
		int count = 0;
		
		preNumbers[preNumbers.length-1] = onlyEvery[0];
		
		if(onlyEvery.length == 2) mod = Integer.parseInt(onlyEvery[1]);
		
		for(int i=0; i<preNumbers.length; i++)
		{
			
			number = preNumbers[i].split("-");
			
			if(number.length == 1)
			{
				if(count%mod == 0) ret_List.add(Integer.parseInt(number[0]));
			}
			else
			{
				for(int j= Integer.parseInt(number[0]); j<=Integer.parseInt(number[1]); j++)
				{
					if(count%mod == 0) ret_List.add(j);
					count++;
				}
			}
			count++;
		}	
		return ret_List;
	}
	
	public boolean testExpression(String string)
	{
		 Pattern p = Pattern.compile("((\\d+,)*(\\d+-\\d+,)*)*((\\d)+|(\\d+-\\d+)+)(:\\d+)?");
		 Matcher m = p.matcher(string);
		 
		 return m.matches();
	}
	
	//###############################################################################################
	// Override Methods ActionListener
	//###############################################################################################

	@Override
	public void actionPerformed(ActionEvent arg0) 
	{
		if(arg0.getActionCommand().equals(okButton.getActionCommand()))
		{			
			int width;
			
			if(outerMarks.isSelected())
			{
				mv.setOuterMarks(true);
				width = mv.getRule_width()*mv.getSession().getCurrPopulationSize();
			}
			else
			{
				mv.setOuterMarks(false);
				width = mv.getRule_width()*(mv.getSession().getCurrPopulationSize()+1);
			}

			File file = null;

			JFileChooser jfc = new JFileChooser();
			
			int selection = jfc.showSaveDialog(this);
			
			if(selection == JFileChooser.APPROVE_OPTION)
			{
				file = jfc.getSelectedFile();
			}
		
			TreeSet<Integer> gen_List = parseGenText(genTextArea.getText());
			
			BufferedImage matrixImgA = null;
			Graphics2D g2dMA = null;
			int maxSize = (Integer)maxSpinner.getValue();
			
			if(asMatrixBox.isSelected())
			{
				if(outerMarks.isSelected())
				{
					matrixImgA = new BufferedImage(width*(maxSize)+2*(maxSize-1)+mv.getRule_width()
							, width*((gen_List.size()-1)/maxSize+1)+2*(gen_List.size()/maxSize) + mv.getRule_width()
							, BufferedImage.TYPE_INT_ARGB);
				}
				else
				{
					matrixImgA = new BufferedImage(width*(maxSize)+2*(maxSize-1)
							, width*((gen_List.size()-1)/maxSize+1)+2*(gen_List.size()/maxSize)
							, BufferedImage.TYPE_INT_ARGB);
				}

				g2dMA = matrixImgA.createGraphics();
			}
			
			if(outerMarks.isSelected() && asMatrixBox.isSelected())
			{
				
				mv.drawMarksX(g2dMA, maxSize, 2);
				mv.drawMarksY(g2dMA, (gen_List.size()-1)/maxSize+1, 2);
			}
			
			g2dMA.translate(mv.getRule_width(), mv.getRule_height());
		
			int j=0;
			for(Integer i: gen_List)
			{	
				try
				{		
				    // TYPE_INT_ARGB specifies the image format: 8-bit RGBA packed
				    // into integer pixels
				    BufferedImage bi = new BufferedImage(width, width, BufferedImage.TYPE_INT_ARGB);
		
				    Graphics2D g2d = bi.createGraphics();

				    mv.drawImage(g2d, i);
				    
				    if(asMatrixBox.isSelected())
				    {
				    	g2dMA.drawImage(bi, (width+2)*(j%maxSize), (width+2)*(j/maxSize), null);
				    	g2dMA.setColor(Color.DARK_GRAY);

				    	g2dMA.fillRect((width+2)*(j%maxSize)-2
			    				, 0
			    				, 2
			    				, width*((gen_List.size()-1)/maxSize+2)+2*(gen_List.size()/maxSize));

				    	g2dMA.fillRect(0
			    				, (width+2)*(j/maxSize)-2
			    				, width*(maxSize)+2*maxSize
			    				,2);					    		
					}	
				    if(singleBox.isSelected())
					    ImageIO.write(bi, "PNG", new File(file.toString() + i +".PNG"));
				    
			    } catch (IOException ie) {
			      ie.printStackTrace();
			    };
			    j++;
			}
			if(asMatrixBox.isSelected())
				try {
					ImageIO.write(matrixImgA, "PNG", new File(file.toString() +"_M"  +".PNG"));

				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			
			this.dispose();
		}
		
	}
	
	//###############################################################################################
	// Override Methods DocumentListener
	//###############################################################################################

	@Override
	public void changedUpdate(DocumentEvent arg0) 
	{
		if(testExpression(genTextArea.getText()))
		{
			okButton.setEnabled(true);
		}
		else
		{
			okButton.setEnabled(false);
		}
		
	}

	@Override
	public void insertUpdate(DocumentEvent arg0) 
	{
		if(testExpression(genTextArea.getText()))
		{
			okButton.setEnabled(true);
		}
		else
		{
			okButton.setEnabled(false);
		}
	}

	@Override
	public void removeUpdate(DocumentEvent arg0) 
	{
		if(testExpression(genTextArea.getText()))
		{
			okButton.setEnabled(true);
		}
		else
		{
			okButton.setEnabled(false);
		}
	}

}
