package draw;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Path2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeSet;

import javax.swing.ImageIcon;

import listener.CleverMouseListener;

import fuzzy.FuzzyRule;
import fuzzy.FuzzySet;
import guiCore.FuzzyRuleCollection;
import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.DropTarget;
import guiUtil.Interaction;
import guiUtil.InteractionField;
import guiUtil.NumberFormater;
import listener.CleverMouseListener;
import math.Polar;

public class ParallelAxisView extends RuleVisualizer implements DropTarget{
	
	public static final  int DRAG_ATTRIBUTE = 0;
	public static final int DRAG_WEIGHT = 1;
	
	public static final int OVERLAP_BAR = 0;
	public static final int SPLIT_BAR = 1;
	public static final int BEZIER_TUBE = 2;
	public static final int TUBE = 3;
	public static final int SPLIT_BAR_2 = 4;
	
	private int rule_Width = 20;
	private int rule_Height = 20;	
	private int padding = 10;
	private int padding_V = 10;
	private int left_padding = 100;
	private int global_y = 80;
	 
	private int block_width = 30;
	private int block_height = 12;
	
	private int axisA_Length = 200;		//antecedence Axis
	private int axisC_Length = 100;		//conclusion Axis
	private int axis_Dis = 60;
	private int rangeStr_y = 15;
	private int bar_Halfwidth = 15;
	
	private int width = 0;
	private int height = 0;
	
	private int alphaValue = 80;
	
	//drag and drop stuff
	private int dragedIndex = -1;
	private int dragObj = -1;
	
	//Pupup
	ParallelAxisView_PopUp pav_popup;
	
	//Bar mode	
	private int barMode = OVERLAP_BAR;
	private int tubeMode = TUBE;
	
	private AffineTransform globalTransform = new AffineTransform();
	private AffineTransform weightBlocksTransform = new AffineTransform();
	private AffineTransform weightTransform = new AffineTransform();
	private AffineTransform attrBlocksTransform = new AffineTransform();
	private AffineTransform ruleTransform = new AffineTransform();
	
	private ArrayList<Integer> attr_Permutation = new ArrayList<Integer>();
	private ArrayList<Integer> weights_Permutation = new ArrayList<Integer>();
	
	private ArrayList<ParallelAxisViewRule> pavr_List = new ArrayList<ParallelAxisViewRule>();
	private ArrayList<RuleAndColor> currFr_List = new ArrayList<RuleAndColor>();
	private int selectedRule = -1;
	private int selectedLockedRule = -1;
	private boolean showReferenceIntercept = true;
	private boolean showSpherical = true;
	private boolean logSelectedRule = false;
	private boolean showCenterPath = false;
	private boolean showWidthBars = true;
	private boolean showWidthTube = false;
	private boolean showData = false;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public ParallelAxisView(Session session_P)
	{
		super(session_P, new DrawPanel());
		
		pav_popup = new ParallelAxisView_PopUp(this);
		
		cm_Listener.addToReleasedList(this);
		cm_Listener.addToPressedList(this);
		cm_Listener.addToMovedList(this);
		
		width = axis_Dis * session.getFr_premiseSize() + left_padding + 100;
		height = axisA_Length + axisC_Length + block_height*2 + rangeStr_y*5 + 150;
		
		drawPanel.setPreferredSize(new Dimension(width, height));
		
		for(int i=0; i<session.getFr_premiseSize(); i++)
		{
			attr_Permutation.add(i);
		}
		for(int i=0; i<session.getFr_conclusionSize(); i++)
		{
			weights_Permutation.add(i);
		}
		
		globalTransform.translate(left_padding, global_y);
		attrBlocksTransform.translate(- block_width/2
				, axisA_Length+rangeStr_y);
		weightBlocksTransform.translate(-(block_width/2) - axis_Dis
								, axisA_Length+rangeStr_y+block_height+5);
		weightTransform.translate( 0 
									, axisA_Length + rangeStr_y*2 + block_height*2 + 5);
		ruleTransform.translate( - axis_Dis*3/2
								, axisA_Length + axisC_Length + block_height*2 + rangeStr_y*5 + 5 + 10);
		
	}
	
	//###############################################################################################
	// Override Methods DropTarget
	//###############################################################################################

	@Override
	public boolean hit(MouseEvent me) 
	{	
		Rectangle visible_rect = drawPanel.getVisibleRect();
		
		if(drawPanel.isShowing())
		if(me.getXOnScreen() < drawPanel.getLocationOnScreen().getX()+visible_rect.getWidth()
				&& me.getXOnScreen() > drawPanel.getLocationOnScreen().getX()
				&& me.getYOnScreen() < drawPanel.getLocationOnScreen().getY()+visible_rect.getHeight()
				&& me.getYOnScreen() > drawPanel.getLocationOnScreen().getY())
		{
			return true;
		}		
		return false;
	}

	@Override
	public void drop(Object[] data)
	{
		if(data != null)
		{
			for(int i=0; i<data.length; i++)
			{
				if(data[i] instanceof Integer)
				{
					this.addRule((Integer)data[i]);
				}
			}
		}
	}

	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################
	
	@Override
	protected void invokeDrawMethods(Graphics2D g2D)
	{				
		g2D.transform(globalTransform);
		
//		drawAxisBackground(g2D);		
		drawParallelAxis(g2D);
		
		if(showData)
		{
			drawData(g2D);
		}
				
		//draw Bars
		if(showWidthBars)
		{
			switch(barMode)
			{
				case OVERLAP_BAR:
					drawRuleFuzzyWidth(g2D);
					break;
					
				case SPLIT_BAR:
					drawRuleFuzzySplitBar(g2D);
					break;
				case SPLIT_BAR_2:
					drawRuleFuzzySplitBar2(g2D);
					break;

			}
		}
		if(showWidthTube)
		{
			switch(tubeMode)
			{
				
				case BEZIER_TUBE:
					drawCurvedTube(g2D);
					break;
				case TUBE:
					drawTube(g2D);
					break;
			}
		}
			
		drawAttributeBlocks(g2D);
			
		if(showCenterPath)
		{
			drawRuleCenter(g2D);
		}
								
		drawWeightBlocks(g2D);
		drawWeightAxis(g2D);
		
		if(showSpherical)
		{
			drawPolarWeights(g2D);
		}
		else
		{
			drawWeights(g2D);
		}
				
		drawRules(g2D);
				
		drawValueBackground(g2D);
		if(selectedRule != -1) drawValues(g2D);
		if(selectedLockedRule != -1 ) drawLockedValues(g2D);
	}

	@Override
	protected synchronized void updateVisualisation()
	{
		currFr_List.clear();
		for(ParallelAxisViewRule pavr: pavr_List)
		{			
			int birth = session.getRuleInfoAt(pavr.getId()).getBirth();
			int death = session.getRuleInfoAt(pavr.getId()).getDeath() - 1;
			int gen = this.frCol.getGeneration();
			
			if(gen > death && death > -1)
			{
				gen = death;
			}
			else if(gen < birth)
			{
				gen = birth;
			}
			
			FuzzyRule fr = session.getFuzzyRule(pavr.getId(), gen);
					
			currFr_List.add(new RuleAndColor(fr,
									pavr.getColor()));
			
			for(int i=0; i<pavr.getNumStatics(); i++)
			{
				currFr_List.add(new RuleAndColor(session.getFuzzyRule(pavr.getId(), pavr.getStaticGenAt(i)),
									pavr.getSubColor(i)));
			}
			
			for(int i=0; i<pavr.getNumDelay(); i++)
			{
				gen = gen - pavr.getDelayAt(i);
				
				if(gen < session.getRuleInfoAt(pavr.getId()).getBirth()
						|| (gen >= session.getRuleInfoAt(pavr.getId()).getDeath()
							&& session.getRuleInfoAt(pavr.getId()).getDeath() > 0))
				{
					gen = session.getRuleInfoAt(pavr.getId()).getBirth();
				}
								
				currFr_List.add(new RuleAndColor(session.getFuzzyRule(pavr.getId(), gen),
									pavr.getSubColor(pavr.getNumStatics()+i)));
				gen = this.frCol.getGeneration();
			}
		}
	}
	
	@Override
	protected void setIDList(TreeSet<Integer> ID_List) 
	{
		clear();
		for(Integer i: ID_List)
		{
			this.addRule(i);
		}
	}
	
	//###############################################################################################
	// Override Methods CleverMouseListener
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType)
	{
		boolean retValue = false;
		
		double index;
		double x;

		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
								
				if(me.isPopupTrigger())
				{	
					pav_popup.show(me.getComponent(), me.getX(), me.getY());
				}
				else if(globalTransform.getTranslateY()+weightBlocksTransform.getTranslateY() < me.getY()
						&& globalTransform.getTranslateY()+weightBlocksTransform.getTranslateY()+block_height > me.getY())
				{
					index = ((me.getX()-globalTransform.getTranslateX()+axis_Dis*3/2)/axis_Dis);
					x = me.getX()-globalTransform.getTranslateX()+axis_Dis*3/2-(int)index*axis_Dis;
					
					if((index >= 0 && index < session.getFr_conclusionSize())
							&& (x < (axis_Dis+block_width)/2 && x > (axis_Dis-block_width)/2))
					{			
						this.dragedIndex = (int)index;
						this.dragObj = DRAG_WEIGHT;
						
						Cursor c = drawPanel.getToolkit().createCustomCursor(
								new ImageIcon("res/Icons/dragBlock.png").getImage(),
								new Point(16,16), "move");
						drawPanel.setCursor(c);
					}
					retValue = true;
				}
				else if(globalTransform.getTranslateY()+attrBlocksTransform.getTranslateY() < me.getY()
						&& globalTransform.getTranslateY()+attrBlocksTransform.getTranslateY()+block_height > me.getY())
				{
					index = ((me.getX()-globalTransform.getTranslateX()+axis_Dis/2)/axis_Dis);
					x = me.getX()-globalTransform.getTranslateX()+axis_Dis/2-(int)index*axis_Dis;
					
					if((index >= 0 && index < session.getFr_premiseSize())
							&& (x < (axis_Dis+block_width)/2 && x > (axis_Dis-block_width)/2))
					{			
						this.dragedIndex = (int)index;
						this.dragObj = DRAG_ATTRIBUTE;
						
						Cursor c = drawPanel.getToolkit().createCustomCursor(
								new ImageIcon("res/Icons/dragBlock.png").getImage(),
								new Point(16,16), "move");
						drawPanel.setCursor(c);
					}
					retValue = true;
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED:
				
				if(me.isPopupTrigger())
				{	
					pav_popup.show(me.getComponent(), me.getX(), me.getY());
				}
				else if(dragObj == DRAG_WEIGHT
						&& dragedIndex > -1)
				{
					index = ((me.getX()-globalTransform.getTranslateX()+2*axis_Dis)/axis_Dis);
					
					if(index >= 0 && index < session.getFr_conclusionSize()+1)
					{
						int dragValue = weights_Permutation.get(dragedIndex);
						if(dragedIndex+1 < index)
						{
							weights_Permutation.add((int)index, dragValue);
							weights_Permutation.remove(dragedIndex);
						}
						else if(dragedIndex > index)
						{
							weights_Permutation.remove(dragedIndex);
							weights_Permutation.add((int)index,dragValue);
						}
					}
					retValue = true;
				}
				else if(dragObj == DRAG_ATTRIBUTE
						&& dragedIndex > -1)
				{
					index = ((me.getX()-globalTransform.getTranslateX()+axis_Dis)/axis_Dis);
					
					if(index >= 0 && index < session.getFr_premiseSize()+1)
					{
						int dragValue = attr_Permutation.get(dragedIndex);
						if(dragedIndex+1 < index)
						{
							attr_Permutation.add((int)index, dragValue);
							attr_Permutation.remove(dragedIndex);
						}
						else if(dragedIndex > index)
						{
							attr_Permutation.remove(dragedIndex);
							attr_Permutation.add((int)index,dragValue);
						}
					}
					retValue = true;
				}
				drawPanel.setCursor(Cursor.getDefaultCursor());
				drawPanel.repaint();
				dragedIndex = -1;
				dragObj = -1;
				
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	
				
				index = (int)((me.getX() 
									- ruleTransform.getTranslateX() 
									- globalTransform.getTranslateX())/(padding + rule_Width));
				x = me.getX() 
						- ruleTransform.getTranslateX() 
						- globalTransform.getTranslateX()
						- index*(padding + rule_Width);
				
				int subIndex = (int)((me.getY() 
									- ruleTransform.getTranslateY()
									- globalTransform.getTranslateY())/(padding_V + rule_Height));
				
				double y = me.getY() 
						- ruleTransform.getTranslateY() 
						- globalTransform.getTranslateY()
						- subIndex*(padding_V + rule_Height);				
				
				if(index < pavr_List.size()
						&& index >= 0
						&& x < rule_Width
						&& x > 0
						&& subIndex < pavr_List.get((int)index).getNumStatics() 
										+ pavr_List.get((int)index).getNumDelay() + 1
						&& y < rule_Height
						&& y > 0)
				{
					Point point = drawPanel.getLocationOnScreen();						
					InteractionField[] i_Field = null;
					
					if(subIndex == 0)
					{
						i_Field = new InteractionField[3];
						i_Field[0] = new ParallelAxis_DeleteInteraction(this, (int)index, 0, 0);
						i_Field[1] = new ParallelAxis_Interaction_2(this, (int)index);
						i_Field[2] = new ParallelAxis_Interaction_3(this, (int)index);
					}
					else if(subIndex > 0
								&& subIndex < pavr_List.get((int)index).getNumStatics() + 1)
					{
						i_Field = new InteractionField[1];
						i_Field[0] = new ParallelAxis_DeleteInteraction(this, (int)index, 1, subIndex - 1);
					}
					else if(subIndex > pavr_List.get((int)index).getNumStatics()
							&& subIndex < pavr_List.get((int)index).getNumStatics() 
												+ pavr_List.get((int)index).getNumDelay() + 1)
					{
						i_Field = new InteractionField[1];
						i_Field[0] = new ParallelAxis_DeleteInteraction(this, (int)index, 2, subIndex 
																								- pavr_List.get((int)index).getNumStatics() 
																								- 1);
					}
					
					
					
					ParallelAxis_InteractionMove paim = new ParallelAxis_InteractionMove(this, (int)index);
					
						Interaction interaction = Interaction.getInstance(i_Field
													, paim
													, point.x
														+ (int)globalTransform.getTranslateX()
														+ (int)ruleTransform.getTranslateX()
														+ (int)((padding+rule_Width)*index)
														+ rule_Width/2
													, point.y 
														+ (int)ruleTransform.getTranslateY()
														+ (int)globalTransform.getTranslateY()
														+ (int)((padding_V+rule_Height)*subIndex)
														+ rule_Height/2
													, 20
													, null);
				}
				
				break;
		}		
		return retValue;
	}

	@Override
	public boolean hitObject(MouseEvent me) 
	{
		return true;
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void drawImageAntecedence(Graphics2D g2D, int gen, boolean withMarks)
	{
		g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING
				, RenderingHints.VALUE_ANTIALIAS_ON );

		session.setCurrPopulationIndex(gen);
		super.refreshCollection(session.getCurrPopulation());
		updateVisualisation();	

		g2D.translate((axis_Dis/2), 0);
		
		if(withMarks)
		{
			g2D.translate(0, 15);
		}

		drawParallelAxis(g2D);
		
		if(showData)
		{
			drawData(g2D);
		}
				
		//draw Bars
		if(showWidthBars)
		{
			switch(barMode)
			{
				case OVERLAP_BAR:
					drawRuleFuzzyWidth(g2D);
					break;
					
				case SPLIT_BAR:
					drawRuleFuzzySplitBar(g2D);
					break;
				case SPLIT_BAR_2:
					drawRuleFuzzySplitBar2(g2D);
					break;
			}
		}	
	}
	
	public void drawImageConclusion(Graphics2D g2D, int gen, boolean withMarks)
	{		
		g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING
				, RenderingHints.VALUE_ANTIALIAS_ON );

		session.setCurrPopulationIndex(gen);
		super.refreshCollection(session.getCurrPopulation());
		updateVisualisation();	

		g2D.translate((axis_Dis/2*3), -weightTransform.getTranslateY());
		
		if(withMarks)
		{
			g2D.translate(0, 15);
		}
		
		drawWeightAxis(g2D);
		if(showSpherical)
		{
			drawPolarWeights(g2D);
		}
		else
		{
			drawWeights(g2D);
		}		
	}
	
	public void addRule(int id)
	{
		boolean alreadyIn = false;
		
		for(ParallelAxisViewRule pavr: pavr_List)
		{
			if(pavr.getId() == id) alreadyIn = true;
		}
		
		if(!alreadyIn) pavr_List.add(
				new ParallelAxisViewRule(id
						, session));
		
		this.customize();
		
		drawPanel.repaint();
	}
	
	public void removeRule(int index)
	{
		pavr_List.remove(index);
		if(index == this.selectedLockedRule)
		{
			this.selectedLockedRule = -1;
			this.logSelectedRule = false;
		}
		
		if(index == this.selectedRule)
		{
			this.selectedRule = -1;
		}
		
		this.customize();
		
		drawPanel.repaint();
	}
	
	public void clear()
	{
		pavr_List.clear();
		this.selectedLockedRule = -1;
		this.selectedRule		= -1;
		this.logSelectedRule = false;
		
		drawPanel.repaint();
	}
	
	public void getRulesInRange()
	{
		session.openFindRulesDialog(this);
	}
	
	public void addStaticRule(int index, int generation)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).addStaticRule(generation);
			
			drawPanel.repaint();
		}
		
		this.updateHeight();
	}
	
	public void removeStaticRule(int index, int subIndex)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).removeStaticRule(subIndex);
			
			drawPanel.repaint();
		}
		
		this.updateHeight();
	}
	
	public void addDelayRule(int index, int delay)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).addDelayRule(delay);
			
			drawPanel.repaint();
		}	
		
		this.updateHeight();
	}
	
	public void removeDelayRule(int index, int subIndex)
	{
		if(index < pavr_List.size())
		{
			pavr_List.get(index).removeDelayRule(subIndex);
			
			drawPanel.repaint();
		}
		
		this.updateHeight();
	}
	
	public void customize()
	{
		if(session.isAutoCustomFlag())
		{
			int[] id_A = new int[this.getPavr_List().size()];

			for(int i=0; i<id_A.length; i++)
			{
				id_A[i] = this.getPAVruleAt(i).getId();
			}
			
			this.autoCustomizeRange(id_A);
		}
	}
	
	//###############################################################################################
	// draw Methods
	//###############################################################################################
	
	private void drawRules(Graphics2D g2d)
	{		
		boolean setGray = false;
		
		for(int i=0; i<pavr_List.size(); i++)
		{	
			if((frCol.getGeneration() >= session.getRuleInfoAt(pavr_List.get(i).getId()).getDeath()
					&&  session.getRuleInfoAt(pavr_List.get(i).getId()).getDeath() > session.getRuleInfoAt(pavr_List.get(i).getId()).getBirth())
					|| frCol.getGeneration() < session.getRuleInfoAt(pavr_List.get(i).getId()).getBirth())
			{
				setGray = true;
			}
			else
			{
				setGray = false;
			}
			
			if(setGray)
			{
				g2d.setColor(Color.gray);
			}
			else
			{
				g2d.setColor(pavr_List.get(i).getColor());
			}
			
			g2d.fillRoundRect((padding+rule_Width)*i + (int)ruleTransform.getTranslateX()
				, (int)ruleTransform.getTranslateY()
				, rule_Width
				, rule_Height
				, 10
				, 10);
			
			g2d.setColor(Color.black);
			TextLayout textLayout = new TextLayout(pavr_List.get(i).getId()+"", g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString(pavr_List.get(i).getId()+""
					,(padding+rule_Width)*i + (int)ruleTransform.getTranslateX() + rule_Width/2 -(int)bounds.getCenterX()
					,(int)ruleTransform.getTranslateY() + rule_Height/2-(int)bounds.getCenterY());
			
			for(int j=0; j< pavr_List.get(i).getNumStatics(); j++)
			{
				if(setGray)
				{
					g2d.setColor(Color.gray);
				}
				else
				{
					g2d.setColor(pavr_List.get(i).getSubColor(j));
				}
				g2d.fillRoundRect((padding+rule_Width)*i + (int)ruleTransform.getTranslateX()
					, (int)ruleTransform.getTranslateY() + (padding_V + rule_Height)*(j + 1)
					, rule_Width
					, rule_Height
					, 10
					, 10);
				
				g2d.setColor(Color.black);
				g2d.drawRoundRect((padding+rule_Width)*i + (int)ruleTransform.getTranslateX()
					, (int)ruleTransform.getTranslateY() + (padding_V + rule_Height)*(j + 1)
					, rule_Width
					, rule_Height
					, 10
					, 10);
				
				g2d.setColor(Color.black);
				textLayout = new TextLayout(pavr_List.get(i).getId()
												+"."
												+pavr_List.get(i).getStaticGenAt(j), g2d.getFont(), g2d.getFontRenderContext());
				bounds = textLayout.getBounds();
				g2d.drawString(pavr_List.get(i).getId()
							+"."
							+pavr_List.get(i).getStaticGenAt(j)
						,(padding+rule_Width)*i 
						 		+ (int)ruleTransform.getTranslateX()
								+ rule_Width/2 
								- (int)bounds.getCenterX()
						,(int)ruleTransform.getTranslateY() 
								+ rule_Height/2-(int)bounds.getCenterY()  
								+ (padding_V + rule_Height) * (j + 1));
			}
			
			for(int j=0; j< pavr_List.get(i).getNumDelay(); j++)
			{
				if(setGray)
				{
					g2d.setColor(Color.gray);
				}
				else
				{
					g2d.setColor(pavr_List.get(i).getSubColor(pavr_List.get(i).getNumStatics()+j));
				}
				g2d.fillRoundRect((padding+rule_Width)*i + (int)ruleTransform.getTranslateX()
					, (int)ruleTransform.getTranslateY() + (padding_V + rule_Height)*(pavr_List.get(i).getNumStatics()+j + 1)
					, rule_Width
					, rule_Height
					, 10
					, 10);
				
				g2d.setColor(Color.black);
				g2d.drawRoundRect((padding+rule_Width)*i + (int)ruleTransform.getTranslateX()
					, (int)ruleTransform.getTranslateY() + (padding_V + rule_Height)*(pavr_List.get(i).getNumStatics()+j + 1)
					, rule_Width
					, rule_Height
					, 10
					, 10);
				
				g2d.drawRoundRect((padding+rule_Width)*i + (int)ruleTransform.getTranslateX()-3
						, (int)ruleTransform.getTranslateY() + (padding_V + rule_Height)*(pavr_List.get(i).getNumStatics()+j + 1)-3
						, rule_Width + 6
						, rule_Height + 6
						, 10
						, 10);
				
				g2d.setColor(Color.black);
				textLayout = new TextLayout(pavr_List.get(i).getId()
												+"|"
												+pavr_List.get(i).getDelayAt(j), g2d.getFont(), g2d.getFontRenderContext());
				bounds = textLayout.getBounds();
				g2d.drawString(pavr_List.get(i).getId()
							+"|"
							+pavr_List.get(i).getDelayAt(j)
						,(padding+rule_Width)*i 
						 		+ (int)ruleTransform.getTranslateX()
								+ rule_Width/2 
								- (int)bounds.getCenterX()
						,(int)ruleTransform.getTranslateY() 
								+ rule_Height/2-(int)bounds.getCenterY()  
								+ (padding_V + rule_Height) * (pavr_List.get(i).getNumStatics()+j + 1));
			}
		}		
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawWeights(Graphics2D g2D)
	{
		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				if(rac.isSub)
				{
					//TODO
				}
				else 
				{
					g2D.setColor(session.getRuleInfoAt(rac.fr.getId()).getColor());
				}
				
	
				double[] weights = rac.fr.getFc().getCharacteristics();
				for(int i=0; i<weights.length-1; i++)
				{				
					g2D.drawLine(axis_Dis*i - axis_Dis
							, (int)(axisC_Length * (1 - (weights[weights_Permutation.get((i+weights.length-1)%weights.length)] - session.getOut_Range()[weights_Permutation.get((i+weights.length-1)%weights.length)][0])/(session.getOut_Range()[weights_Permutation.get((i+weights.length-1)%weights.length)][1]-session.getOut_Range()[weights_Permutation.get((i+weights.length-1)%weights.length)][0])))
								+ (int)weightTransform.getTranslateY()
							, axis_Dis*(i+1) - axis_Dis
							, (int)(axisC_Length * (1 - (weights[weights_Permutation.get((i+1+weights.length-1)%weights.length)] - session.getOut_Range()[weights_Permutation.get((i+1+weights.length-1)%weights.length)][0])/(session.getOut_Range()[weights_Permutation.get((i+1+weights.length-1)%weights.length)][1]-session.getOut_Range()[weights_Permutation.get((i+1+weights.length-1)%weights.length)][0])))
								+ (int)weightTransform.getTranslateY());
					
					g2D.fillOval(axis_Dis*i - axis_Dis-3
							, (int)(axisC_Length * (1 - (weights[weights_Permutation.get((i+weights.length-1)%weights.length)] - session.getOut_Range()[weights_Permutation.get((i+weights.length-1)%weights.length)][0])/(session.getOut_Range()[weights_Permutation.get((i+weights.length-1)%weights.length)][1]-session.getOut_Range()[weights_Permutation.get((i+weights.length-1)%weights.length)][0])))
								+ (int)weightTransform.getTranslateY()-3
							, 6
							, 6);
				}
				g2D.fillOval(axis_Dis*(weights.length-1) - axis_Dis-3
						, (int)(axisC_Length * (1 - (weights[weights_Permutation.get(weights.length-2)] - session.getOut_Range()[weights_Permutation.get(weights.length-2)][0])/(session.getOut_Range()[weights_Permutation.get(weights.length-2)][1]-session.getOut_Range()[weights_Permutation.get(weights.length-2)][0])))
							+ (int)weightTransform.getTranslateY()-3
						, 6
						, 6);
			}
		}
	}
	
	private void drawPolarWeights(Graphics2D g2D)
	{
		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				if(rac.isSub)
				{
					//TODO
				}
				else 
				{
					g2D.setColor(session.getRuleInfoAt(rac.fr.getId()).getColor());
				}
				
	
				double[] weights = rac.fr.getFc().getCharacteristics();
				
				if(showReferenceIntercept)
				{
					//centered y-intercept
					g2D.drawLine( - axis_Dis - axis_Dis/4
							, (int)(axisC_Length * (1- (rac.fr.getFc().getValue(rac.fr.getCenter()) - session.getRefOut_Range()[0])/(session.getRefOut_Range()[1]-session.getRefOut_Range()[0])))
								+ (int)weightTransform.getTranslateY()
							, - axis_Dis + axis_Dis/4
							, (int)(axisC_Length * (1-(rac.fr.getFc().getValue(rac.fr.getCenter()) - session.getRefOut_Range()[0])/(session.getRefOut_Range()[1]-session.getRefOut_Range()[0])))
								+ (int)weightTransform.getTranslateY());					
				}
				else
				{
					//y-intercept
					g2D.drawLine( - axis_Dis - axis_Dis/4
						, (int)(axisC_Length * (1-(weights[session.getFr_conclusionSize()-1] - session.getOut_Range()[session.getFr_conclusionSize()-1][0])/(session.getOut_Range()[session.getFr_conclusionSize()-1][1]-session.getOut_Range()[session.getFr_conclusionSize()-1][0])))
							+ (int)weightTransform.getTranslateY()
						, - axis_Dis + axis_Dis/4
						, (int)(axisC_Length * (1-(weights[session.getFr_conclusionSize()-1] - session.getOut_Range()[session.getFr_conclusionSize()-1][0])/(session.getOut_Range()[session.getFr_conclusionSize()-1][1]-session.getOut_Range()[session.getFr_conclusionSize()-1][0])))
							+ (int)weightTransform.getTranslateY());				
				}
				
				weights[weights.length-1] = -1;
				double[] permWeights = permutate(weights);
				
				double[] polar = Polar.cartesianToPolar(permWeights);
				
				g2D.drawLine(0
						, (int)(axisC_Length * (1 - (polar[1] + Math.PI)/(2*Math.PI)))
							+ (int)weightTransform.getTranslateY()
						, axis_Dis
						, (int)(axisC_Length * (1 - (polar[2])/(Math.PI)))
							+ (int)weightTransform.getTranslateY());
				
				g2D.fillOval(-3
						, (int)(axisC_Length * (1 - (polar[1] + Math.PI)/(2*Math.PI)))
							+ (int)weightTransform.getTranslateY()-3
						, 6
						, 6);
				
				for(int i=2; i<weights.length-1; i++)
				{				
					g2D.drawLine(axis_Dis*i - axis_Dis
							, (int)(axisC_Length * (1 - (polar[i])/(Math.PI)))
								+ (int)weightTransform.getTranslateY()
							, axis_Dis*(i+1) - axis_Dis
							, (int)(axisC_Length * (1 - (polar[i+1])/(Math.PI)))
								+ (int)weightTransform.getTranslateY());
					
					g2D.fillOval(axis_Dis*i - axis_Dis-3
							, (int)(axisC_Length * (1 - (polar[i])/(Math.PI)))
								+ (int)weightTransform.getTranslateY()-3
							, 6
							, 6);
				}
				
				g2D.fillOval(axis_Dis*(weights.length-1) - axis_Dis-3
						, (int)(axisC_Length * (1 - (polar[polar.length-1])/(Math.PI)))
							+ (int)weightTransform.getTranslateY()-3
						, 6
						, 6);
			}
		}
	}
	
	private double[] permutate(double[] array)
	{
		double[] retArray = new double[array.length];
		
		for(int i=0; i<retArray.length; i++)
		{
			retArray[i] = array[weights_Permutation.get(i)];
		}
		
		return retArray;
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawWeightAxis(Graphics2D g2D)
	{
		int conclusionSize = session.getFr_conclusionSize();
		int i=2;
		String drawValue = null;
		TextLayout textLayout = null;
		Rectangle2D bounds = null;
		
		if(showSpherical)
		{
			//Background
			g2D.setColor(new Color(230,230,230));
			
			g2D.fillRect( - axis_Dis/2 - axis_Dis
					, (int)weightTransform.getTranslateY()
					, axis_Dis
					, axisC_Length);	
			
			//Axis
			g2D.setColor(Color.black);
			g2D.drawLine( - axis_Dis
					, (int)weightTransform.getTranslateY()
					, -axis_Dis
					, (int)weightTransform.getTranslateY() + axisC_Length);
	
			//---------------------------------------------------------------
			g2D.setColor(Color.white);
			
			g2D.fillRect( - axis_Dis/2
					, (int)weightTransform.getTranslateY()
					, axis_Dis
					, axisC_Length);	
			
			//Axis
			g2D.setColor(Color.black);
			g2D.drawLine(0
					, (int)weightTransform.getTranslateY()
					, 0
					, (int)weightTransform.getTranslateY() + axisC_Length);
			
			//Range intercept
			g2D.setColor(new Color(0,0,200));
			
			if(this.showReferenceIntercept)
			{
				drawValue = NumberFormater.doubleToString(session.getRefOut_Range()[0], 2);
			}
			else
			{
				drawValue = NumberFormater.doubleToString(session.getOut_Range()[session.getFr_conclusionSize()-1][0], 2);
			}
			
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() + axisC_Length + rangeStr_y/2 - (int)bounds.getCenterY());
			
			if(showReferenceIntercept)
			{
				drawValue = NumberFormater.doubleToString(session.getRefOut_Range()[1], 2);
			}
			else
			{
				drawValue = NumberFormater.doubleToString(session.getOut_Range()[session.getFr_conclusionSize()-1][1], 2);
			}
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() - rangeStr_y/2 - (int)bounds.getCenterY());
			
	
			//Range first angle
			g2D.setColor(new Color(200,0,0));
			
	//		String drawValue = NumberFormater.doubleToString(session.getOut_Range()[weights_Permutation.get((i+session.getFr_conclusionSize()-1)%session.getFr_conclusionSize())][0], 2);
			
			drawValue = "-180";
			
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*1 - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() + axisC_Length + rangeStr_y/2 - (int)bounds.getCenterY());
			
	//		drawValue = NumberFormater.doubleToString(session.getOut_Range()[weights_Permutation.get((i+session.getFr_conclusionSize()-1)%session.getFr_conclusionSize())][1], 2);
			drawValue = "180";
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*1 - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() - rangeStr_y/2 - (int)bounds.getCenterY());
			
		}
		else
		{
			 i=0;
		}
	
		for(; i<conclusionSize ; i++)
		{
			//Background
			if(i%2 != 0)
			{
				g2D.setColor(Color.white);
			}
			else
			{
				g2D.setColor(new Color(230,230,230));
			}
			
			g2D.fillRect(axis_Dis*i - axis_Dis/2 -axis_Dis
					, (int)weightTransform.getTranslateY()
					, axis_Dis
					, axisC_Length);	
			
			//Axis
			g2D.setColor(Color.black);
			g2D.drawLine(axis_Dis*i - axis_Dis
					, (int)weightTransform.getTranslateY()
					, axis_Dis*i - axis_Dis
					, (int)weightTransform.getTranslateY() + axisC_Length);
			
			//Range
			if(i%2 != 0)
			{
				g2D.setColor(new Color(200,0,0));
			}
			else
			{
				g2D.setColor(new Color(0,0,200));
			}

			if(!showSpherical)
			{
				drawValue = NumberFormater.doubleToString(session.getOut_Range()[weights_Permutation.get((i+session.getFr_conclusionSize()-1)%session.getFr_conclusionSize())][0], 2);
			}
			else
			{
				drawValue = "0";
			}
			
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() + axisC_Length + rangeStr_y/2 - (int)bounds.getCenterY());
			
			if(!showSpherical)
			{
				drawValue = NumberFormater.doubleToString(session.getOut_Range()[weights_Permutation.get((i+session.getFr_conclusionSize()-1)%session.getFr_conclusionSize())][1], 2);
			}
			else
			{
				drawValue = "180";
			}
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() - rangeStr_y/2 - (int)bounds.getCenterY());
	
		}
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawWeightBlocks(Graphics2D g2D)
	{
		int conclusionSize = session.getFr_conclusionSize();
		
		g2D.setColor(Color.DARK_GRAY);
		for(int i=0; i<conclusionSize ; i++)
		{
			g2D.fillRoundRect(axis_Dis*i+(int)weightBlocksTransform.getTranslateX()
					, (int)weightBlocksTransform.getTranslateY()
					, block_width
					, block_height
					, 5
					, 5);
		}
		g2D.setColor(Color.white);
		for(int i=0; i<conclusionSize; i++)
		{
			TextLayout textLayout = new TextLayout(weights_Permutation.get(i)+"", g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(weights_Permutation.get(i)+""
					, axis_Dis*i +(int)weightBlocksTransform.getTranslateX() + block_width/2 - (int)bounds.getCenterX()
					, (int)weightBlocksTransform.getTranslateY()+block_height/2-(int)bounds.getCenterY());
					
		}
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawAttributeBlocks(Graphics2D g2D)
	{		
		int ruleSize = session.getFr_premiseSize();
		
		g2D.setColor(new Color(100,100,100));
		for(int i=0; i<ruleSize; i++)
		{
			g2D.fillRoundRect(axis_Dis*i + (int)attrBlocksTransform.getTranslateX()
					, (int)attrBlocksTransform.getTranslateY()
					, block_width
					, block_height
					, 5
					, 5);
		}
		g2D.setColor(Color.white);
		for(int i=0; i<ruleSize; i++)
		{
			TextLayout textLayout = new TextLayout(attr_Permutation.get(i)+"", g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(attr_Permutation.get(i)+""
					, axis_Dis*i-(int)bounds.getCenterX()
					, (int)attrBlocksTransform.getTranslateY() + block_height/2-(int)bounds.getCenterY());
					
		}		
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawRuleFuzzyWidth(Graphics2D g2D)
	{
		double[] center;
		double[] width;
				
//		float[] color;
		Color color;
		
		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				center = rac.fr.getCenter();
				width = rac.fr.getWidth();
						
//				color = new float[4];
//				color[0] = rac.color.getRed();
//				color[1] = rac.color.getGreen();
//				color[2] = rac.color.getBlue();
//				color[3] = 0xff;
						
				for(int i=0; i<center.length; i++)
				{
	//				GaussPaint gp = new GaussPaint(color
	//								, axis_Length*(center[attr_Permutation.get(i)] - session.getAttr_Range()[attr_Permutation.get(i)][0])/(session.getAttr_Range()[attr_Permutation.get(i)][1]-session.getAttr_Range()[attr_Permutation.get(i)][0])
	//								, axis_Length*width[attr_Permutation.get(i)]/(session.getAttr_Range()[attr_Permutation.get(i)][1]-session.getAttr_Range()[attr_Permutation.get(i)][0]));
	//				g2D.setPaint(gp);							
	//				g2D.fillRect(axis_Dis*i-bar_Halfwidth
	//						, (int)(axis_Length*(center[attr_Permutation.get(i)] - session.getAttr_Range()[attr_Permutation.get(i)][0])/(session.getAttr_Range()[attr_Permutation.get(i)][1]-session.getAttr_Range()[attr_Permutation.get(i)][0]))-100
	//						, bar_Halfwidth*2
	//						, 200);
					
					double helpWidth =  axisA_Length*width[attr_Permutation.get(i)]/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0]);
					if(helpWidth < 1) helpWidth = 1;
					
					int helpY = (int)(axisA_Length*(1 - (center[attr_Permutation.get(i)] - session.getAttr_RangeAt(attr_Permutation.get(i))[0])
							/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0])))-((int)helpWidth);
		
					
					if(helpY < 0)
					{
						helpWidth = helpWidth +  helpY;
						helpY = 0;
					}
					
					if(helpY + helpWidth > axisA_Length)
					{
						helpWidth = axisA_Length - helpY;
					}
					
					color = new Color(rac.color.getRed()
							, rac.color.getGreen()
							, rac.color.getBlue()
							, alphaValue);
					
					g2D.setColor(color);
					g2D.fillRect(axis_Dis*i-bar_Halfwidth
							, helpY
							, bar_Halfwidth*2
							, (int)helpWidth*2);
				}
			}
		}
	}
	
	private void drawRuleFuzzySplitBar(Graphics2D g2D)
	{
		double bar_width = bar_Halfwidth*2d/currFr_List.size();
		
		double[] center;
		double[] width;
						
		int j=1;
		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
					center = rac.fr.getCenter();
					width = rac.fr.getWidth();
														
				for(int i=0; i<center.length; i++)
				{

					
					double helpWidth =  axisA_Length*width[attr_Permutation.get(i)]/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0]);
					if(helpWidth < 1) helpWidth = 1;
					
					int helpY = (int)(axisA_Length * (1 - (center[attr_Permutation.get(i)] - session.getAttr_RangeAt(attr_Permutation.get(i))[0])
							/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0])))-((int)helpWidth);
		
					
					if(helpY < 0)
					{
						helpWidth = helpWidth +  helpY;
						helpY = 0;
					}
					
					if(helpY + helpWidth > axisA_Length)
					{
						helpWidth = axisA_Length - helpY;
					}
					g2D.setColor(rac.color);
					g2D.fillRect(axis_Dis*i+bar_Halfwidth-(int)(bar_width*j)
							, helpY
							, (int)bar_width
							, (int)helpWidth*2);
				}
				j++;
			}
		}
	}
	
	private void drawRuleFuzzySplitBar2(Graphics2D g2d)
	{
		for(int attr=0; attr< session.getFr_premiseSize(); attr++)
		{	
			LinkedList<BoxSeparator> splitBox = calcSplitBox(attr_Permutation.get(attr));
			
			ArrayList<Integer> currIDs = new ArrayList<Integer>();
			
			Iterator<BoxSeparator> it = splitBox.iterator();
			
			BoxSeparator bs_0 = null;
			double value_0 = 0;
			if(it.hasNext())
			{
				bs_0 = it.next();
			}
	
			int helpWidth = 0;
			int sum;
			while(it.hasNext())
			{
				if(bs_0.right > -1)
				{
					currIDs.add(bs_0.right);
				}		
				value_0 = bs_0.value;
	
				bs_0 = it.next();
				
				sum = 0;
				for(int j=0; j<currIDs.size(); j++)
				{			
					helpWidth = calcWidth(bar_Halfwidth*2, currIDs.size(), j);
									
					g2d.setColor(session.getRuleInfoAt(currIDs.get(j)).getColor());
				
					g2d.fillRect(axis_Dis * attr - bar_Halfwidth+ sum
							, (int)value_0
							, helpWidth
							, (int)(bs_0.value - value_0));
					
					sum = sum + helpWidth;
				}
				
				if(bs_0.left > -1)
				{
					currIDs.remove(new Integer(bs_0.left));
				}
			}
		}
	}
	
	private int calcWidth(int attrWidth, int num, int index)
	{
		int retValue = 0;
		int rest = 0;
		
		retValue = attrWidth/num;
		rest = attrWidth-(retValue*num);
		
		if(index < rest) retValue++;
		
		return retValue;
	}
	
	private LinkedList<BoxSeparator> calcSplitBox(int attr)
	{
		LinkedList<BoxSeparator> splitBox = new LinkedList<BoxSeparator>();
		
		for(RuleAndColor rac: currFr_List)
		{
			int death = session.getRuleInfoAt(rac.fr.getId()).getDeath();
			
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				double center = rac.fr.getCenter()[attr];
				center = (center - session.getAttr_RangeAt(attr)[0])/(session.getAttr_RangeAt(attr)[1] - session.getAttr_RangeAt(attr)[0]);
				center = 1 - center; // y-Axis-draw is from top to bottom
				
				double width = rac.fr.getWidth()[attr];				
				width = axisA_Length*width/(session.getAttr_RangeAt(attr)[1] - session.getAttr_RangeAt(attr)[0]);
				
				if(((int)width) < 1) width = 1;
				
				BoxSeparator bs_left = new BoxSeparator();
				bs_left.value = (int)(axisA_Length*center - width);
				bs_left.left = -1;
				bs_left.right = rac.fr.getId();
				
				int index = 0;
				for(BoxSeparator bs: splitBox)
				{
					if(bs_left.value < bs.value)
					{
						break;
					}
					index++;
				}
				splitBox.add(index, bs_left);
				
				BoxSeparator bs_right = new BoxSeparator();
				bs_right.value = (int)(axisA_Length*center + width);
				bs_right.left = rac.fr.getId();
				bs_right.right = -1;
				
				index = 0;
				for(BoxSeparator bs: splitBox)
				{
					if(bs_right.value < bs.value)
					{
						break;
					}
					index++;
				}
				splitBox.add(index, bs_right);
			}
		}
		return splitBox;
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawRuleCenter(Graphics2D g2D)
	{

		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				g2D.setColor(rac.color);
				double[] center = rac.fr.getCenter();
				
				for(int i=0; i<center.length-1; i++)
				{				
					g2D.drawLine(axis_Dis*i
							, (int)(axisA_Length*(1 - (center[attr_Permutation.get(i)] - session.getAttr_RangeAt(attr_Permutation.get(i))[0])/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0])))
							, axis_Dis*(i+1)
							, (int)(axisA_Length*(1 - (center[attr_Permutation.get(i+1)] - session.getAttr_RangeAt(attr_Permutation.get(i+1))[0])/(session.getAttr_RangeAt(attr_Permutation.get(i+1))[1]-session.getAttr_RangeAt(attr_Permutation.get(i+1))[0]))));
				}
			}
		}
	}
	
	/**
	 * 
	 * @param g2D
	 */
	private void drawParallelAxis(Graphics2D g2D)
	{
		int ruleSize = session.getFr_premiseSize();

		
		for(int i=0; i<ruleSize; i++)
		{				
			//Background
			if(i%2 == 0)
			{
				g2D.setColor(Color.white);
			}
			else
			{
				g2D.setColor(new Color(240,240,240));
			}
			
			g2D.fillRect(axis_Dis*i - axis_Dis/2
					, 0
					, axis_Dis
					, axisA_Length);
			
			g2D.setColor(Color.black);
			//Axis
			g2D.drawLine(axis_Dis*i
					, 0
					, axis_Dis*i
					, axisA_Length);
			
			//Range
			if(i%2 == 0)
			{
				g2D.setColor(new Color(200,0,0));
			}
			else
			{
				g2D.setColor(new Color(0,0,200));
			}
			String drawValue = NumberFormater.doubleToString(session.getAttr_RangeAt(attr_Permutation.get(i))[1], 2);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - (int)bounds.getCenterX()
					, -3);
			
			drawValue = NumberFormater.doubleToString(session.getAttr_RangeAt(attr_Permutation.get(i))[0], 2);
			textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - (int)bounds.getCenterX()
					, axisA_Length + rangeStr_y/2 - (int)bounds.getCenterY());	
		}
	}
	
	private void drawValues(Graphics2D g2D)
	{
		g2D.setColor(Color.black);

		double[] weighthelp = currFr_List.get(selectedRule).fr.getFc().getCharacteristics();
		double[] centers = currFr_List.get(selectedRule).fr.getCenter();
		double[] widths  = currFr_List.get(selectedRule).fr.getWidth();
		
		double[] weights;
		
		if(showSpherical)
		{
			double yIntercept = weighthelp[session.getFr_conclusionSize()-1];
			weighthelp[session.getFr_conclusionSize()-1] = -1;
			double[] permWeights = permutate(weighthelp);
			weights = Polar.cartesianToPolar(weighthelp);
			
			if(this.showReferenceIntercept)
			{
				weights[0] = currFr_List.get(selectedRule).fr.getFc().getValue(centers);
			}
			else
			{
				weights[0] = yIntercept;
			}
System.out.println("draw poloar values: " + (weights[0]/Math.PI*180)+ " "+weights[1] + " "+weights[2]/Math.PI*180);
		}
		else
		{
			weights = weighthelp;
		}
		

		
				
		for(int i=0; i<weights.length; i++)
		{	
			String drawValue = NumberFormater.doubleToString(weights[(i+weights.length-1)%weights.length], 3);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() + axisC_Length + rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		}
		
		for(int i=0; i<centers.length; i++)
		{
			String drawValue = NumberFormater.doubleToString(centers[i], 3);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - (int)bounds.getCenterX()
					, - 3 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		}
		
		for(int i=0; i<widths.length; i++)
		{
			String drawValue = NumberFormater.doubleToString(widths[i], 3);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					,  axis_Dis*i - (int)bounds.getCenterX()
					, - 2 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		}		
	}
	
	private void drawLockedValues(Graphics2D g2D)
	{
		g2D.setColor(Color.black);

		double[] weights = currFr_List.get(selectedLockedRule).fr.getFc().getCharacteristics();
		double[] centers = currFr_List.get(selectedLockedRule).fr.getCenter();
		double[] widths  = currFr_List.get(selectedLockedRule).fr.getWidth();
		
				
		for(int i=0; i<weights.length; i++)
		{	
			String drawValue = NumberFormater.doubleToString(weights[i], 3);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - axis_Dis - (int)bounds.getCenterX()
					, (int)weightTransform.getTranslateY() + axisC_Length + 2 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		}
		
		for(int i=0; i<centers.length; i++)
		{
			String drawValue = NumberFormater.doubleToString(centers[i], 3);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					, axis_Dis*i - (int)bounds.getCenterX()
					, - 5 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		}
		
		for(int i=0; i<widths.length; i++)
		{
			String drawValue = NumberFormater.doubleToString(widths[i], 3);
			TextLayout textLayout = new TextLayout(drawValue, g2D.getFont(), g2D.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2D.drawString(drawValue
					,  axis_Dis*i - (int)bounds.getCenterX()
					, - 4 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		}		
	}
	
	private void drawValueBackground(Graphics2D g2d)
	{	
		//Center 1 -------------------------------------
		g2d.setColor(new Color(255, 200, 200));
		g2d.fillRect( - axis_Dis  - axis_Dis/2
				, - 5 * rangeStr_y
				, axis_Dis * session.getFr_premiseSize() + axis_Dis
				, rangeStr_y);
		
		g2d.setColor(Color.black);
		String drawValue = "Center";
		TextLayout textLayout = new TextLayout(drawValue, g2d.getFont(), g2d.getFontRenderContext());
		Rectangle2D bounds = textLayout.getBounds();
		g2d.drawString(drawValue
				, - 50  - axis_Dis/2
				, - 5 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		
		//Width 1 -------------------------------------		
		g2d.setColor(new Color(200, 200, 255));
		g2d.fillRect( - axis_Dis  - axis_Dis/2
				, - 4 * rangeStr_y
				, axis_Dis * session.getFr_premiseSize() + axis_Dis
				, rangeStr_y);
		
		g2d.setColor(Color.black);
		drawValue = "Width";
		textLayout = new TextLayout(drawValue, g2d.getFont(), g2d.getFontRenderContext());
		bounds = textLayout.getBounds();
		g2d.drawString(drawValue
				,  - 50  - axis_Dis/2
				, - 4 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		
		//Center 2 -------------------------------------
		g2d.setColor(new Color(255, 200, 200));
		g2d.fillRect( - axis_Dis  - axis_Dis/2
				, - 3 * rangeStr_y
				, axis_Dis * session.getFr_premiseSize() + axis_Dis
				, rangeStr_y);
		
		g2d.setColor(Color.black);
		drawValue = "Center";
		textLayout = new TextLayout(drawValue, g2d.getFont(), g2d.getFontRenderContext());
		bounds = textLayout.getBounds();
		g2d.drawString(drawValue
				, - 50  - axis_Dis/2
				, - 3 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		
		//Width 2 -------------------------------------
		g2d.setColor(new Color(200, 200, 255));
		g2d.fillRect( - axis_Dis  - axis_Dis/2
				, - 2 * rangeStr_y
				, axis_Dis * session.getFr_premiseSize() + axis_Dis
				, rangeStr_y);
		
		g2d.setColor(Color.black);
		drawValue = "Width";
		textLayout = new TextLayout(drawValue, g2d.getFont(), g2d.getFontRenderContext());
		bounds = textLayout.getBounds();
		g2d.drawString(drawValue
				,  - 50  - axis_Dis/2
				, - 2 * rangeStr_y + rangeStr_y/2 - (int)bounds.getCenterY());
		
		//Weight 1 -------------------------------------
		g2d.setColor(new Color(255, 200, 200));
		g2d.fillRect( - axis_Dis  - axis_Dis/2
				, rangeStr_y + (int)weightTransform.getTranslateY() + axisC_Length
				, axis_Dis * session.getFr_premiseSize() + axis_Dis
				, rangeStr_y );
		
		//Weight 2 -------------------------------------
		g2d.setColor(new Color(200, 200, 255));
		g2d.fillRect( - axis_Dis  - axis_Dis/2
				, 2 * rangeStr_y + (int)weightTransform.getTranslateY() + axisC_Length
				, axis_Dis * session.getFr_premiseSize() + axis_Dis
				, rangeStr_y );
	}
	
	private void drawData(Graphics2D g2d)
	{
//		double[][] data = session.getData();
		double[][] data = session.getCurrData();
		int winID = 0;
		
		for(int i = 0; i< data.length; i++)
		{			
			for(int j=0; j<data[i].length - 1; j++)
			{			
				winID = FuzzyRuleCollection.compWinningRule(data[i], frCol);
				
				if(isInCurrList(winID))
				{
					g2d.setColor(session.getRuleInfoAt(winID).getColor());
					g2d.drawLine(axis_Dis*j
							, (int)(axisA_Length*(1 - (data[i][attr_Permutation.get(j)] - session.getAttr_RangeAt(attr_Permutation.get(j))[0])/(session.getAttr_RangeAt(attr_Permutation.get(j))[1]-session.getAttr_RangeAt(attr_Permutation.get(j))[0])))
							, axis_Dis*(j+1)
							, (int)(axisA_Length*(1 - (data[i][attr_Permutation.get(j+1)] - session.getAttr_RangeAt(attr_Permutation.get(j+1))[0])/(session.getAttr_RangeAt(attr_Permutation.get(j+1))[1]-session.getAttr_RangeAt(attr_Permutation.get(j+1))[0]))));
				}
			}
		}
	}
	
	private void drawCurvedTube(Graphics2D g2d)
	{
		CubicCurve2D c = new CubicCurve2D.Double();
		CubicCurve2D c2 = new CubicCurve2D.Double();

		Color color;
		
		double[] center;
		double[] width;

		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				center = rac.fr.getCenter();
				width = rac.fr.getWidth();
						
				for(int i=0; i<center.length-1; i++)
				{					
					double helpWidth =  axisA_Length*width[attr_Permutation.get(i)]/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0]);
					if(helpWidth < 1) helpWidth = 1;
					
					double helpWidth2 =  axisA_Length*width[attr_Permutation.get(i+1)]/(session.getAttr_RangeAt(attr_Permutation.get(i+1))[1]-session.getAttr_RangeAt(attr_Permutation.get(i+1))[0]);
					if(helpWidth2 < 1) helpWidth2 = 1;
					
					int helpY = (int)(axisA_Length*(1 - (center[attr_Permutation.get(i)] - session.getAttr_RangeAt(attr_Permutation.get(i))[0])
							/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0])));
					int helpY2 = (int)(axisA_Length*(1 - (center[attr_Permutation.get(i+1)] - session.getAttr_RangeAt(attr_Permutation.get(i+1))[0])
							/(session.getAttr_RangeAt(attr_Permutation.get(i+1))[1]-session.getAttr_RangeAt(attr_Permutation.get(i+1))[0])));
		
					
					if(helpY < 0)
					{
						helpWidth = helpWidth +  helpY;
						helpY = 0;
					}
					
					if(helpY + helpWidth > axisA_Length)
					{
						helpWidth = axisA_Length - helpY;
					}
										
					color = new Color(rac.color.getRed()
									, rac.color.getGreen()
									, rac.color.getBlue()
									, 80);
					
					g2d.setColor(color);
					
					// draw CubicCurve2D.Double with set coordinates
					c.setCurve(axis_Dis*i
							, helpY-helpWidth
							, axis_Dis*i
							, helpY
							, axis_Dis*(i+1)
							, helpY2
							, axis_Dis*(i+1)
							, helpY2-helpWidth2);

					
					c2.setCurve(axis_Dis*(i+1)
							, helpY2+helpWidth2							
							, axis_Dis*(i+1)
							, helpY2
							, axis_Dis*i
							, helpY
							, axis_Dis*i
							, helpY+helpWidth
							);
					
					Path2D path2D = new Path2D.Double();
				    path2D.append(c, true);
				    path2D.append(c2, true);
				    path2D.closePath();
					
					g2d.fill(path2D);

				}
			}
		}
	}
	
	private void drawTube(Graphics2D g2d)
	{
		Color color;
		
		double[] center;
		double[] width;
		
		int[] xPoints = new int[4];
		int[] yPoints = new int[4];

		for(RuleAndColor rac: currFr_List)
		{
			if((frCol.getGeneration() < session.getRuleInfoAt(rac.fr.getId()).getDeath()
					||  session.getRuleInfoAt(rac.fr.getId()).getDeath() <= session.getRuleInfoAt(rac.fr.getId()).getBirth())
					&& frCol.getGeneration() >= session.getRuleInfoAt(rac.fr.getId()).getBirth())
			{
				center = rac.fr.getCenter();
				width = rac.fr.getWidth();
						
				for(int i=0; i<center.length-1; i++)
				{					
					double helpWidth =  axisA_Length*width[attr_Permutation.get(i)]/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0]);
					if(helpWidth < 1) helpWidth = 1;
					
					double helpWidth2 =  axisA_Length*width[attr_Permutation.get(i+1)]/(session.getAttr_RangeAt(attr_Permutation.get(i+1))[1]-session.getAttr_RangeAt(attr_Permutation.get(i+1))[0]);
					if(helpWidth2 < 1) helpWidth2 = 1;
					
					int helpY = (int)(axisA_Length*(1 - (center[attr_Permutation.get(i)] - session.getAttr_RangeAt(attr_Permutation.get(i))[0])
							/(session.getAttr_RangeAt(attr_Permutation.get(i))[1]-session.getAttr_RangeAt(attr_Permutation.get(i))[0])));
					int helpY2 = (int)(axisA_Length*(1 - (center[attr_Permutation.get(i+1)] - session.getAttr_RangeAt(attr_Permutation.get(i+1))[0])
							/(session.getAttr_RangeAt(attr_Permutation.get(i+1))[1]-session.getAttr_RangeAt(attr_Permutation.get(i+1))[0])));
		
					
					if(helpY < 0)
					{
						helpWidth = helpWidth +  helpY;
						helpY = 0;
					}
					
					if(helpY + helpWidth > axisA_Length)
					{
						helpWidth = axisA_Length - helpY;
					}
										
					color = new Color(rac.color.getRed()
									, rac.color.getGreen()
									, rac.color.getBlue()
									, 80);
					
					g2d.setColor(color);
					
					xPoints[0] = axis_Dis*i;
					xPoints[1] = axis_Dis*i;
					xPoints[2] = axis_Dis*(i+1);
					xPoints[3] = axis_Dis*(i+1);
					
					yPoints[0] = (int)(helpY-helpWidth);
					yPoints[1] = (int)(helpY+helpWidth);
					yPoints[2] = (int)(helpY2+helpWidth2);
					yPoints[3] = (int)(helpY2-helpWidth2);
					
					g2d.fillPolygon(xPoints, yPoints, 4);

				}
			}
		}
	}
	
	private boolean isInCurrList(int id)
	{
		for(RuleAndColor rac: currFr_List)
		{
			if(rac.fr.getId() == id) return true;
		}
		return false;
	}
	
	private void updateHeight()
	{
		int maxNum = 0;
		int num = 0;
		
		for(ParallelAxisViewRule pavr: pavr_List)
		{
			num = pavr.getNumDelay() + pavr.getNumStatics();
			
			if(maxNum <num) maxNum = num;
		}
		
		
		height = axisA_Length + axisC_Length 
				+ block_height*2 
				+ rangeStr_y*5 
				+ 150 
				+ (rule_Height + padding_V)* maxNum;
		
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	public RuleInfo getRuleInfoAt(int index)
	{
		return session.getRuleInfoAt(index);
	}
	
	public ParallelAxisViewRule getPAVruleAt(int index)
	{
		return pavr_List.get(index);
	}
	
	public ArrayList<ParallelAxisViewRule> getPavr_List() {
		return pavr_List;
	}

	public int getCurr_t()
	{
		return session.getCurrPopulationIndex();
	}
	
	public int getRule_Width() {
		return rule_Width;
	}


	public int getRule_Height() {
		return rule_Height;
	}


	public int getPadding() {
		return padding;
	}

	public void setBarMode(int barMode) {
		this.barMode = barMode;
	}
	
	public void setTubeMode(int tubeMode) {
		this.tubeMode = tubeMode;
	}
	
	public void setSelectedRule(int selectedRule) {
		this.selectedRule = selectedRule;
	}

	public int getSelectedRule() {
		return selectedRule;
	}
	
	public void setSelectedLockedRule(int selectedLockedRule) {
		this.selectedLockedRule = selectedLockedRule;
	}

	public int getSelectedLockedRule() {
		return selectedLockedRule;
	}

	public boolean isLogSelectedRule() {
		return logSelectedRule;
	}

	public void setLogSelectedRule(boolean logSelectedRule) {
		this.logSelectedRule = logSelectedRule;
	}

	public boolean isShowCenterPath() {
		return showCenterPath;
	}

	public void setShowCenterPath(boolean showCenterPath) {
		this.showCenterPath = showCenterPath;
	}

	public boolean isShowWidthBars() {
		return showWidthBars;
	}

	public void setShowWidthBars(boolean showWidthBars) {
		this.showWidthBars = showWidthBars;
	}
	
	public boolean isShowWidthTube() {
		return showWidthTube;
	}

	public void setShowWidthTube(boolean showWidthTube) {
		this.showWidthTube = showWidthTube;
	}

	public boolean isShowData() {
		return showData;
	}

	public void setShowData(boolean showData) {
		this.showData = showData;
	}

	public boolean isShowReferenceIntercept() {
		return showReferenceIntercept;
	}

	public void setShowReferenceIntercept(boolean showReferenceIntercept) {
		this.showReferenceIntercept = showReferenceIntercept;
	}

	public boolean isShowSpherical() {
		return showSpherical;
	}

	public void setShowSpherical(boolean showSpherical) {
		this.showSpherical = showSpherical;
	}

	public int getAlphaValue() {
		return alphaValue;
	}

	public void setAlphaValue(int alphaValue) 
	{
		this.repaintDrawPanel();
		this.alphaValue = alphaValue;
	}

	public int getAxisA_Length() {
		return axisA_Length;
	}

	public int getAxisC_Length() {
		return axisC_Length;
	}

	public int getAxis_Dis() {
		return axis_Dis;
	}
	
	public Session getSession()
	{
		return super.session;
	}

	public void autoCustomizeRange(int[] id_A)
	{
		session.autoCustomizeRange(id_A);
		session.setAutoCustomFlag(true);
	}

	//###############################################################################################
	// inner Class
	//###############################################################################################
	
	private class BoxSeparator{
		
		public int right;
		public int left;
		public double value;
		
	}

}
