package draw;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import fuzzy.FuzzyRule;
import guiCore.ColorGenerator;
import guiCore.Session;

public class ParallelAxisViewRule {
	
	private int id;
	
	private Session session;
	
	private final ArrayList<Integer> static_List = new ArrayList<Integer>();
	private final ArrayList<Integer> delay_List = new ArrayList<Integer>();
	
	//###############################################################################################
	// Constructor
	//###############################################################################################

	public ParallelAxisViewRule(int id_P, Session session)
	{
		this.id 	= id_P;
		this.session = session;	
		
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void addStaticRule(int generation)
	{
		static_List.add(generation);
	}
	
	public void removeStaticRule(int subIndex)
	{
		static_List.remove(subIndex);
	}
	
	public void addDelayRule(int delay)
	{
		delay_List.add(delay);
	}
	
	public void removeDelayRule(int subIndex)
	{
		delay_List.remove(subIndex);
	}
	
	public int getNumStatics()
	{
		return static_List.size();
	}
	
	public Integer getStaticGenAt(int index)
	{
		return static_List.get(index);
	}
	
	public int getNumDelay()
	{
		return delay_List.size();
	}
	
	public Integer getDelayAt(int index)
	{
		return delay_List.get(index);
	}
	
	public Color getSubColor(int index)
	{
		Color color = session.getRuleInfoAt(id).getColor();
		
		int red		= color.getRed();
		int green	= color.getGreen();
		int blue	= color.getBlue();
		
		float[][] genColor = new float[2][3];
		
		genColor[0][0] = 0f; genColor[0][1] = 0f; genColor[0][2] = 0f;
		genColor[1][0] = 1f; genColor[1][1] = 1f; genColor[1][2] = 1f;
		
		Color bottomColor = new ColorGenerator(genColor).calculateRuleColor(id);
		
		int redInter 	= red - bottomColor.getRed();
		int greenInter 	= green - bottomColor.getGreen();
		int blueInter	= blue - bottomColor.getBlue();		
		
		red 	= red - (redInter/((static_List.size()+delay_List.size())+1)) * (index+1);
		green 	= green - (greenInter/((static_List.size()+delay_List.size())+1)) * (index+1);
		blue 	= blue - (blueInter/((static_List.size()+delay_List.size())+1)) * (index+1);
		
		return new Color(red, green, blue);
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################

	public Color getColor() {
		return session.getRuleInfoAt(id).getColor();
	}


	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}
}
