package draw;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class ParallelAxisView_PopUp extends JPopupMenu implements ActionListener, ChangeListener{
	
	private ParallelAxisView pav;
	
	private JRadioButton overlapBars_Radio;
	private JRadioButton splitBars_Radio;
	private JRadioButton splitBars2_Radio;
	
	private JRadioButton tube_Radio;
	private JRadioButton bezierTube_Radio;
	
	private JRadioButton yIntercept_Radio;
	private JRadioButton referenceInter_Radio;
	
	private JRadioButton spherical_Radio;
	private JRadioButton cartesian_Radio;
	
	private ButtonGroup group_1;
	private ButtonGroup group_2;
	private ButtonGroup group_3;
	private ButtonGroup group_4;
	
	private JPanel panel_1;
	private JPanel panel_2;
	private JPanel panel_3;
	private JPanel panel_4;
	private JPanel panel_5;
	
	private JCheckBox showWidthBars_Check;
	private JCheckBox showWidthTube_Check;
	private JCheckBox showCenterPath_Check;
	private JCheckBox showData_Check;
	
	private JSpinner alphaValue_Spinner;
	private JLabel alphaValue_Label;
	
	private JPanel alphaValue_Panel;
	
	private JMenuItem autoCustomizeItem;
	private JMenuItem clearItem;
	private JMenuItem rulesInRangeItem;
	private JMenuItem saveAsImageItem;
	

	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public ParallelAxisView_PopUp(ParallelAxisView pav_P)
	{
		pav = pav_P;
		
		overlapBars_Radio = new JRadioButton("overlap Bars");
		overlapBars_Radio.setActionCommand("overlap");
		overlapBars_Radio.addActionListener(this);
		
		splitBars_Radio = new JRadioButton("split Bars");
		splitBars_Radio.setActionCommand("split");
		splitBars_Radio.addActionListener(this);
		
		splitBars2_Radio = new JRadioButton("split Bars 2");
		splitBars2_Radio.setActionCommand("split2");
		splitBars2_Radio.addActionListener(this);
		
		bezierTube_Radio = new JRadioButton("Bezier-Tube");
		bezierTube_Radio.setActionCommand("bezier");
		bezierTube_Radio.addActionListener(this);
		
		tube_Radio = new JRadioButton("Tube");
		tube_Radio.setActionCommand("tube");
		tube_Radio.addActionListener(this);
		
		yIntercept_Radio = new JRadioButton("y-Intercept");
		yIntercept_Radio.setActionCommand("yInter");
		yIntercept_Radio.addActionListener(this);
		
		referenceInter_Radio = new JRadioButton("Center-Intercept");
		referenceInter_Radio.setActionCommand("cInter");
		referenceInter_Radio.addActionListener(this);
		
		spherical_Radio = new JRadioButton("Spherical");
		spherical_Radio.setActionCommand("spherical");
		spherical_Radio.addActionListener(this);
		
		cartesian_Radio = new JRadioButton("Cartesian");
		cartesian_Radio.setActionCommand("cartesian");
		cartesian_Radio.addActionListener(this);
		
		showWidthBars_Check = new JCheckBox("show Width-Bars");
		showWidthBars_Check.setActionCommand("showWidth");
		showWidthBars_Check.addActionListener(this);
		showWidthBars_Check.setSelected(true);
		
		showWidthTube_Check = new JCheckBox("show Width-Tube");
		showWidthTube_Check.setActionCommand("showWidthTube");
		showWidthTube_Check.addActionListener(this);
		showWidthTube_Check.setSelected(false);
		
		showCenterPath_Check = new JCheckBox("show Center-Path");
		showCenterPath_Check.setActionCommand("showPath");
		showCenterPath_Check.addActionListener(this);
		
		showData_Check = new JCheckBox("show Data");
		showData_Check.setActionCommand("showData");
		showData_Check.addActionListener(this);
		
		alphaValue_Label = new JLabel("Alpha Value ");
		
		alphaValue_Spinner = new JSpinner(new SpinnerNumberModel(80,0,255,1));
		alphaValue_Spinner.addChangeListener(this);
		alphaValue_Spinner.setName("alphaValue");
		alphaValue_Spinner.setPreferredSize(new Dimension(50,20));
		
		alphaValue_Panel = new JPanel();
		alphaValue_Panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		alphaValue_Panel.add(alphaValue_Label);
		alphaValue_Panel.add(alphaValue_Spinner);
		
		autoCustomizeItem = new JMenuItem("Auto Custimze Range");
		autoCustomizeItem.setActionCommand("autoC");
		autoCustomizeItem.addActionListener(this);
		
		clearItem = new JMenuItem("Clear");
		clearItem.setActionCommand("clear");
		clearItem.addActionListener(this);
		
		rulesInRangeItem = new JMenuItem("Select all rules in Range");
		rulesInRangeItem.setActionCommand("rulesInRange");
		rulesInRangeItem.addActionListener(this);
		
		saveAsImageItem = new JMenuItem("save as image");
		saveAsImageItem.setActionCommand("saveAsImage");
		saveAsImageItem.addActionListener(this);
		
		group_1 = new ButtonGroup();
		group_1.add(overlapBars_Radio);
		group_1.add(splitBars_Radio);
		group_1.add(splitBars2_Radio);
		overlapBars_Radio.setSelected(true);
		
		group_2 = new ButtonGroup();
		group_2.add(bezierTube_Radio);
		group_2.add(tube_Radio);
		tube_Radio.setSelected(true);
		
		group_3 = new ButtonGroup();
		group_3.add(yIntercept_Radio);
		group_3.add(referenceInter_Radio);
		referenceInter_Radio.setSelected(true);
		
		group_4 = new ButtonGroup();
		group_4.add(spherical_Radio);
		group_4.add(cartesian_Radio);
		spherical_Radio.setSelected(true);
		
		panel_1 = new JPanel();
		panel_1.setLayout(new GridLayout(3,1));
		panel_1.setBorder(new TitledBorder("Bar"));
		panel_1.add(overlapBars_Radio);
		panel_1.add(splitBars_Radio);
		panel_1.add(splitBars2_Radio);
		
		panel_2 = new JPanel();
		panel_2.setLayout(new GridLayout(2,1));
		panel_2.setBorder(new TitledBorder("Tube"));
		panel_2.add(bezierTube_Radio);
		panel_2.add(tube_Radio);
		
		panel_3 = new JPanel();
		panel_3.setLayout(new GridLayout(2,1));
		panel_3.setBorder(new TitledBorder("Intercept"));
		panel_3.add(referenceInter_Radio);
		panel_3.add(yIntercept_Radio);
		
		panel_4 = new JPanel();
		panel_4.setLayout(new GridLayout(4,1));
		panel_4.setBorder(new TitledBorder("Visibility"));
		panel_4.add(showWidthBars_Check);
		panel_4.add(showWidthTube_Check);
		panel_4.add(showCenterPath_Check);
		panel_4.add(showData_Check);
		
		panel_5 = new JPanel();
		panel_5.setLayout(new GridLayout(2,1));
		panel_5.setBorder(new TitledBorder("Conclusion"));
		panel_5.add(spherical_Radio);
		panel_5.add(cartesian_Radio);
		
		this.add(panel_1);
		this.add(panel_2);
		this.add(panel_3);
		this.add(panel_5);
		this.add(panel_4);
		this.add(alphaValue_Panel);
		this.add(autoCustomizeItem);
		this.add(clearItem);
		this.add(rulesInRangeItem);
		this.add(saveAsImageItem);
	}
	
	//###############################################################################################
	// Override Methods ActionListener
	//###############################################################################################

	@Override
	public void actionPerformed(ActionEvent arg0) {
		
		if(arg0.getActionCommand().equals("overlap"))
		{
			pav.setBarMode(ParallelAxisView.OVERLAP_BAR);
		}
		else if(arg0.getActionCommand().equals("split"))
		{
			pav.setBarMode(ParallelAxisView.SPLIT_BAR);
		}
		else if(arg0.getActionCommand().equals("split2"))
		{
			pav.setBarMode(ParallelAxisView.SPLIT_BAR_2);
		}
		else if(arg0.getActionCommand().equals(tube_Radio.getActionCommand()))
		{
			pav.setTubeMode(ParallelAxisView.TUBE);
		}
		else if(arg0.getActionCommand().equals(bezierTube_Radio.getActionCommand()))
		{
			pav.setTubeMode(ParallelAxisView.BEZIER_TUBE);
		}
		else if(arg0.getActionCommand().equals(referenceInter_Radio.getActionCommand()))
		{
			pav.setShowReferenceIntercept(true);
		}
		else if(arg0.getActionCommand().equals(yIntercept_Radio.getActionCommand()))
		{
			pav.setShowReferenceIntercept(false);
		}
		else if(arg0.getActionCommand().equals(spherical_Radio.getActionCommand()))
		{
			pav.setShowSpherical(true);
		}
		else if(arg0.getActionCommand().equals(cartesian_Radio.getActionCommand()))
		{
			pav.setShowSpherical(false);
		}
		else if(arg0.getActionCommand().equals("autoC"))
		{
			int[] id_A = new int[pav.getPavr_List().size()];

			for(int i=0; i<id_A.length; i++)
			{
				id_A[i] = pav.getPAVruleAt(i).getId();
			}
			
			pav.autoCustomizeRange(id_A);
		}
		else if(arg0.getActionCommand().equals(clearItem.getActionCommand()))
		{
			pav.clear();
		}
		else if(arg0.getActionCommand().equals(showCenterPath_Check.getActionCommand()))
		{
			pav.setShowCenterPath(showCenterPath_Check.isSelected());
		}
		else if(arg0.getActionCommand().equals(showWidthBars_Check.getActionCommand()))
		{
			pav.setShowWidthBars(showWidthBars_Check.isSelected());
		}
		else if(arg0.getActionCommand().equals(showWidthTube_Check.getActionCommand()))
		{
			pav.setShowWidthTube(showWidthTube_Check.isSelected());
		}
		else if(arg0.getActionCommand().equals(showData_Check.getActionCommand()))
		{
			pav.setShowData(showData_Check.isSelected());
		}
		else if(arg0.getActionCommand().equals(rulesInRangeItem.getActionCommand()))
		{
			pav.getRulesInRange();
		}
		else if(arg0.getActionCommand().equals(saveAsImageItem.getActionCommand()))
		{
			ParallelAxis_ToImageDialog tid = new ParallelAxis_ToImageDialog(pav);
		}
		
		pav.repaintDrawPanel();
	}

	@Override
	public void stateChanged(ChangeEvent arg0) 
	{
		JSpinner spinner = (JSpinner)arg0.getSource();
		
		if(spinner.getName().equals(alphaValue_Spinner.getName()))
		{
			pav.setAlphaValue((Integer)spinner.getValue());
		}		
	}

}
