package draw;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.TreeSet;

import javax.swing.ImageIcon;

import listener.CleverMouseListener;

import fuzzy.FuzzyRule;
import guiCore.ColorGenerator;
import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.DragHandler;
import guiUtil.DragSource;
import guiUtil.DropTarget;
import guiUtil.Interaction;
import guiUtil.InteractionField;

public class RuleListView extends RuleVisualizer implements DragSource{
	
	private DragHandler dragHandler;
	
	private int rule_Width = 20;
	private int rule_Height = 20;
	private int rule_Dis = 10;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################

	public RuleListView(Session session_P) {
		super(session_P, new DrawPanel());
		
		drawPanel.setPreferredSize(new Dimension(60,200));
		
		cm_Listener.addToPressedList(this);
		cm_Listener.addToReleasedList(this);
		
		dragHandler = new DragHandler(this);
	}
	
	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) {
		boolean retValue = false;
		
		int index;

		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED:
				
				index = me.getY()/(rule_Dis+rule_Height);
				
				if(session.isOnlyLiving())
				{
					if(index < frCol.collectionSize()
							&& (me.getY()-(index*(rule_Dis+rule_Height)) > rule_Dis))
					{
						index = new Integer(frCol.getFuzzyRuleAt(index).getId());
					}
				}
				
				if(me.isPopupTrigger())
				{
					RuleList_PopUp rl_PopUp = new RuleList_PopUp(this, index);
					rl_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED:
				
				index = me.getY()/(rule_Dis+rule_Height);
				
				if(session.isOnlyLiving())
				{
					if(index < frCol.collectionSize()
							&& (me.getY()-(index*(rule_Dis+rule_Height)) > rule_Dis))
					{
						index = new Integer(frCol.getFuzzyRuleAt(index).getId());
					}
				}
				
				if(me.isPopupTrigger())
				{
					RuleList_PopUp rl_PopUp = new RuleList_PopUp(this, index);
					rl_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;

		}		
		return retValue;
	}

	@Override
	public boolean hitObject(MouseEvent me) 
	{
		return true;
	}
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	protected void updateVisualisation() 
	{
		drawPanel.setPreferredSize(new Dimension(65, (rule_Height + rule_Dis)*(session.getMaxID()+1)));
		drawPanel.revalidate();
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2d) 
	{
		drawRules(g2d);
	}
		
	//###############################################################################################
	// Override Methods DragSource
	//###############################################################################################

	@Override
	public Object[] getData(MouseEvent me) {
		
		Object[] ret_obj_A = new Object[1];
	
		int index = me.getY()/(rule_Dis+rule_Height);
		
		if(!session.isOnlyLiving())
		{
			if(index <= session.getMaxID()
					&& (me.getY()-(index*(rule_Dis+rule_Height)) > rule_Dis))
			{
				ret_obj_A[0] = new Integer(index);
				
				return ret_obj_A;
			}
		}
		else
		{
			if(index < frCol.collectionSize()
					&& (me.getY()-(index*(rule_Dis+rule_Height)) > rule_Dis))
			{
				ret_obj_A[0] = new Integer(frCol.getFuzzyRuleAt(index).getId());
				
				return ret_obj_A;
			}
		}
			
		
		return null;
	}

	@Override
	public void addMouseListener(MouseListener ml) 
	{
		this.drawPanel.addMouseListener(ml);
	}
	
	@Override
	public void addMouseMotionListener(MouseMotionListener mml)
	{
		this.drawPanel.addMouseMotionListener(mml);		
	}
	

	@Override
	public void addDropTarget(DropTarget dt) 
	{
		this.dragHandler.addDropTarget(dt);
	}

	@Override
	public void drawDrop(Graphics2D g2d, MouseEvent me, Object[] data) 
	{
		int index = me.getY()/(rule_Dis+rule_Height);
		
		if(session.isOnlyLiving())
		{
			index = frCol.getFuzzyRuleAt(index).getId();
		}
		
		g2d.setColor(session.getRuleInfoAt(index).getColor());
		g2d.fillRoundRect(0
				, 0
				, rule_Width
				, rule_Height
				, 10
				, 10);
		
		g2d.setColor(Color.white);
		TextLayout textLayout = new TextLayout(index+"", g2d.getFont(), g2d.getFontRenderContext());
		Rectangle2D bounds = textLayout.getBounds();
		g2d.drawString(index+""
				, rule_Width/2 -(int)bounds.getCenterX()
				, rule_Height/2-(int)bounds.getCenterY());
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################
	
	private void drawRules(Graphics2D g2d)
	{
		if(!session.isOnlyLiving())
		{
			for(int i=0; i<=session.getMaxID(); i++)
			{			
				g2d.setColor(session.getRuleInfoAt(i).getColor());
				g2d.fillRoundRect( 5
						, rule_Dis + (rule_Dis + rule_Height)*i
						, rule_Width
						, rule_Height
						, 10
						, 10);
				
				g2d.setColor(Color.white);
				TextLayout textLayout = new TextLayout(i+"", g2d.getFont(), g2d.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				g2d.drawString(i+""
						, 5 + rule_Width/2 -(int)bounds.getCenterX()
						, rule_Dis + (rule_Dis + rule_Height)*i + rule_Height/2-(int)bounds.getCenterY());
				
				
				g2d.setColor(Color.black);
				g2d.drawString(session.getRuleInfoAt(i).getBirth()+""
						, 2 + 5 + rule_Width
						, rule_Dis + (rule_Dis + rule_Height)*i + (int)bounds.getHeight());
				
				String death = null;
				if(session.getRuleInfoAt(i).getDeath() == 0)
				{
					death = "~";
				}
				else
				{
					death = session.getRuleInfoAt(i).getDeath() + "";
				}
				g2d.drawString(death
						, 2 + 5 + rule_Width
						, rule_Dis + (rule_Dis + rule_Height)*i + rule_Height);
				
				g2d.drawString(session.getRuleInfoAt(i).getAncestors()[0]+""
						, 4
						,  (rule_Dis + rule_Height)*i + (int)bounds.getHeight());
				g2d.drawString(session.getRuleInfoAt(i).getAncestors()[1]+""
						, 4 + 12
						,  (rule_Dis + rule_Height)*i + (int)bounds.getHeight());
			}
		}
		else
		{
			int i = 0;
			for(FuzzyRule fr: this.frCol.getFrList())
			{			
				g2d.setColor(session.getRuleInfoAt(fr.getId()).getColor());
				g2d.fillRoundRect( 5
						, rule_Dis + (rule_Dis + rule_Height)*i
						, rule_Width
						, rule_Height
						, 10
						, 10);
				
				g2d.setColor(Color.white);
				TextLayout textLayout = new TextLayout(fr.getId()+"", g2d.getFont(), g2d.getFontRenderContext());
				Rectangle2D bounds = textLayout.getBounds();
				g2d.drawString(fr.getId()+""
						, 5 + rule_Width/2 -(int)bounds.getCenterX()
						, rule_Dis + (rule_Dis + rule_Height)*i + rule_Height/2-(int)bounds.getCenterY());
				
				
				g2d.setColor(Color.black);
				g2d.drawString(session.getRuleInfoAt(fr.getId()).getBirth()+""
						, 2 + 5 + rule_Width
						, rule_Dis + (rule_Dis + rule_Height)*i + (int)bounds.getHeight());
				
				String death = null;
				if(session.getRuleInfoAt(fr.getId()).getDeath() == 0)
				{
					death = "~";
				}
				else
				{
					death = session.getRuleInfoAt(fr.getId()).getDeath() + "";
				}
				g2d.drawString(death
						, 2 + 5 + rule_Width
						, rule_Dis + (rule_Dis + rule_Height)*i + rule_Height);
				
				g2d.drawString(session.getRuleInfoAt(fr.getId()).getAncestors()[0]+""
						, 4
						,  (rule_Dis + rule_Height)*i + (int)bounds.getHeight());
				g2d.drawString(session.getRuleInfoAt(fr.getId()).getAncestors()[1]+""
						, 4 + 12
						,  (rule_Dis + rule_Height)*i + (int)bounds.getHeight());
				i++;
			}
		}		
	}
	
	public Session getSession()
	{
		return super.session;
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}
}
