package draw;

import guiCore.RuleVisualizer;
import guiCore.Session;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.AdjustmentEvent;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.TreeSet;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import listener.CleverMouseListener;

public class TimeLineScale extends RuleVisualizer{
	
	private int scale_Dis = 20;
	private int scaleStart = 0;
	private double scaleFactor = 1;

	private int transX 	= 0;
	private int startX 	= 0;
	private int g		= 0;
	private boolean reglerPressed = false;
	
	private JPanel partnerPanel;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public TimeLineScale(Session session_P, JPanel partnerPanel_P)
	{
		super(session_P, new DrawPanel());
		
		this.partnerPanel = partnerPanel_P;
		
		scrollPane.getHorizontalScrollBar().setPreferredSize(new Dimension(0,0));
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		this.drawPanel.setPreferredSize(new Dimension(partnerPanel_P.getWidth(),30));
	}
	
	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) 
	{
		return reglerPressed;
	}

	@Override
	public boolean hitObject(MouseEvent me) 
	{		
		return false;
	}
	
	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################

	@Override
	protected void updateVisualisation() 
	{
		drawPanel.setPreferredSize(new Dimension(partnerPanel.getWidth(), 30));
		drawPanel.revalidate();
		drawPanel.repaint();	
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2d) 
	{		
		drawScale(g2d);
	}
	
	//###############################################################################################
	// private drawMethods
	//###############################################################################################
	
	public void drawScale(Graphics2D g2d)
	{
		int[] visibleRules = computeVisibleRules((int)(scale_Dis*scaleFactor), drawPanel, session);
		
		Rectangle visibleRect = new Rectangle();
		this.drawPanel.computeVisibleRect(visibleRect);
		
		g2d.setColor(new Color(240,240,255));
		g2d.fillRect(0
				, visibleRect.y
				, (int)(drawPanel.getWidth()*scaleFactor)
				, 26);
		
		for(int i=visibleRules[0]; i < visibleRules[1]+1; i++)
		{
			g2d.setColor(Color.black);
			
			TextLayout textLayout = new TextLayout(i+"", g2d.getFont(), g2d.getFontRenderContext());
			Rectangle2D bounds = textLayout.getBounds();
			g2d.drawString(i+""
					, (int)((scale_Dis*i -(int)bounds.getCenterX() + scaleStart)*scaleFactor)
					, 15);
			g2d.drawLine((int)((scale_Dis*i + scaleStart)*scaleFactor)
					, 20
					, (int)((scale_Dis*i + scaleStart)*scaleFactor)
					, 25);
		}
	}
	
	public void scroll(AdjustmentEvent arg0)
	{
		drawPanel.setPreferredSize(new Dimension(partnerPanel.getWidth(), 30));
		scrollPane.getHorizontalScrollBar().setValue(arg0.getValue());
		drawPanel.revalidate();
		drawPanel.repaint();
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public void setScale_Dis(int scale_Dis) {
		this.scale_Dis = scale_Dis;
	}

	public void setScaleStart(int scaleStart) {
		this.scaleStart = scaleStart;
	}

	public int getScaleStart() {
		return scaleStart;
	}

	public int getTransX() {
		return transX;
	}

	public int getStartX() {
		return startX;
	}

	public void setStartX(int startX) {
		this.startX = startX;
	}

	public double getScaleFactor() {
		return scaleFactor;
	}

	public void setScaleFactor(double scaleFactor) {
		this.scaleFactor = scaleFactor;
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}


}
