package draw;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseEvent;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.TreeSet;

import javax.swing.JPanel;

import listener.CleverMouseListener;
import listener.MouseInteractionObject;
import math.MathUtil;

import fuzzy.FuzzyRule;
import guiCore.ColorGenerator;
import guiCore.RuleVisualizer;
import guiCore.Session;
import guiUtil.NumberFormater;

public class TimeLineView extends RuleVisualizer implements AdjustmentListener{
	
	private int rule_Width = 10;
	private int rule_Height = 10;
	private int rule_Dis = 30;
	private int row_Dis = 20;
	private int disLine_Height = 15;
	
	private double scaleFactor = 1;
	
	private TimeLineSlider timeLineSlider;
	private TimeLine_PopUp tl_PopUp;
	
	private int attr_Width = 18;
	private int attrPadding_North = 0;
	private DrawPanel attrPanel;
	
	private float[] bottomColor = {0,0,0,1};
	private float[] topColor = {1,1,1,1};
	
	private boolean showPearls = true;
	private boolean showLinks = true;
	private boolean showAngle = false;
	private boolean showConclusion = false;
	private boolean showPlaneDis = false;
	private double linkThreshold = 1;
	private double angelSimThreshold = 1;
	private double planeDisThreshold = 0;
	
	private int distanceType = 0;
	private int numHLSamples = 100;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################

	public TimeLineView(Session session_P) 
	{
		super(session_P, new DrawPanel());
		
		tl_PopUp = new TimeLine_PopUp(this);
		
		cm_Listener.addToReleasedList(this);
		cm_Listener.addToPressedList(this);
		
		drawPanel.setBackground(Color.white);
		scrollPane.getHorizontalScrollBar().addAdjustmentListener(this);
		scrollPane.getVerticalScrollBar().addAdjustmentListener(this);
		
		attrPanel = new DrawPanel();
		attrPanel.setPreferredSize(new Dimension(30, drawPanel.getHeight()));
		attrPanel.addDrawObject(new DrawAttribute());
		
		timeLineSlider = new TimeLineSlider(session, drawPanel);
		timeLineSlider.setScale_Dis(rule_Width+rule_Dis);
		timeLineSlider.setScaleStart(0);
		
		JPanel paddingPanel = new JPanel();
		paddingPanel.setPreferredSize(new Dimension(30, 20));
		
		JPanel tlsWrapperPanel = new JPanel();
		tlsWrapperPanel.setLayout(new BorderLayout());
		tlsWrapperPanel.add(paddingPanel, BorderLayout.WEST);
		tlsWrapperPanel.add(timeLineSlider.getComponent(), BorderLayout.CENTER);
		
		mainPanel.add(tlsWrapperPanel, BorderLayout.NORTH);
		mainPanel.add(attrPanel, BorderLayout.WEST);
		
	}

	//###############################################################################################
	// Override Methods RuleVisualizer
	//###############################################################################################
	
	@Override
	protected void updateVisualisation() 
	{		
		timeLineSlider.updateVisualisation();
		int width = (int)((rule_Width+rule_Dis)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_Height+row_Dis)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
		attrPanel.repaint();
//		drawPanel.scrollRectToVisible(this.computeCurrVisibleRect());
	}

	@Override
	protected void invokeDrawMethods(Graphics2D g2D) 
	{
		g2D.scale(scaleFactor, scaleFactor);
		drawTimeLineCutOut(g2D);
		g2D.scale(1, 1/scaleFactor);
		drawRedLine(g2D);
	}
	
	//###############################################################################################
	// Override Methods MouseInteractionObject
	//###############################################################################################

	@Override
	public boolean action(MouseEvent me, int eventType) 
	{
		switch(eventType){
		
		//############ CLICKED #################################################################
			case CleverMouseListener.CLICKED: 	break;
		//############ ENTERED #################################################################
			case CleverMouseListener.ENTERED:	break;
		//############ EXITED  #################################################################
			case CleverMouseListener.EXITED:	break;		
		//############ PRESSED #################################################################
			case CleverMouseListener.PRESSED: 
												
				if(me.isPopupTrigger())
				{	

					tl_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ RELEASED #################################################################		
			case CleverMouseListener.RELEASED: 
				
				if(me.isPopupTrigger())
				{	

					tl_PopUp.show(me.getComponent(), me.getX(), me.getY());
				}
				break;
		//############ DRAGGED #################################################################		
			case CleverMouseListener.DRAGGED: 	break;
		//############ MOVED   #################################################################
			case CleverMouseListener.MOVED: 	break;
		}
		return false;
	}

	@Override
	public boolean hitObject(MouseEvent me) {
		// TODO Auto-generated method stub
		return true;
	}
	
	//###############################################################################################
	// Override Methods AdjustmentListener
	//###############################################################################################
	
	@Override
	public void adjustmentValueChanged(AdjustmentEvent arg0) {
		// TODO Auto-generated method stub
		
		if(arg0.getSource().equals(scrollPane.getHorizontalScrollBar()))
		{
			timeLineSlider.scroll(arg0);
		}
		else
		{
			Rectangle rect = drawPanel.getVisibleRect();	
			attrPadding_North = rect.y;
			attrPanel.repaint();
		}
		drawPanel.repaint();
	}	
	
	//###############################################################################################
	// private draw Methods
	//###############################################################################################
	
	private void drawRedLine(Graphics2D g2d)
	{
		int x = session.getCurrPopulationIndex()*(rule_Width+rule_Dis);
		
		g2d.setColor(Color.red);
		g2d.drawLine(x + timeLineSlider.getTransX()
				, 0
				, x + timeLineSlider.getTransX()
				, drawPanel.getHeight());
	}
		
	private void drawTimeLineCutOut(Graphics2D g2D)
	{
		AffineTransform aTrans;
		
		int[] visibleRules = computeVisibleRules((int)((rule_Width + rule_Dis)*scaleFactor), drawPanel, session);
		
		//draw Ruleblocks
		for(int i=visibleRules[0]; i < visibleRules[1]; i++)
		{
			for(FuzzyRule fr: session.getFuzzyRuleCollectionAt(i).getFrList())
			{
				if(showPearls)
				{
					g2D.setColor(session.getRuleInfoAt(fr.getId()).getColor());
					
					g2D.fillOval((rule_Width+rule_Dis)*i
							, (rule_Height+row_Dis)*fr.getId()
							, rule_Width
						, rule_Height);	
				}
				
				FuzzyRule fr_2 = session.getFuzzyRuleCollectionAt(i+1).getFuzzyRule(fr.getId());
				
				if(fr_2 != null)
				{
					if(showConclusion)
					{
						double angelSim = FuzzyRule.compConsequensSim(fr, fr_2);
						
						if(angelSim < angelSimThreshold)
						{
							g2D.setColor(new Color(1f,(float)angelSim,(float)angelSim,1f));
							g2D.fillRect( (rule_Width+rule_Dis)*i+rule_Width + (rule_Dis/2)
								, (rule_Height+row_Dis)*fr.getId() - (row_Dis)/2
								, (rule_Dis/2)
								, row_Dis + rule_Height);
						}
					}
					if(showPlaneDis)
					{
						double hyperDis = Math.abs(fr.getFc().getCharacteristics()[fr.getFc().getCharacteristics().length-1]
						-fr_2.getFc().getCharacteristics()[fr_2.getFc().getCharacteristics().length-1]);
			
						hyperDis = hyperDis/(session.getOut_Range()[session.getFr_conclusionSize()-1][1]-session.getOut_Range()[session.getFr_conclusionSize()-1][0]);
						
						if(hyperDis > planeDisThreshold)
						{
							g2D.setColor(new Color(1-(float)hyperDis,1-(float)hyperDis,1f,1f));
							g2D.fillRect( (rule_Width+rule_Dis)*i+rule_Width
									, (rule_Height+row_Dis)*fr.getId() - (row_Dis)/2
									, (rule_Dis/2)
									, row_Dis + rule_Height);
						}
					}
					
					int line_Height = 0;
					Color color = null;
					
					if(showLinks)
					{
						double sim = 1;
						
						switch(distanceType)
						{
						case 0: 
							double lastDis = fr.lastDis;
							double curr_Dis = Session.calcDistance(fr, fr_2);
							double family_max_Dis = session.getFamily_max_RuleDis().get(fr.getId());
							double family_min_Dis = session.getFamily_min_RuleDis().get(fr.getId());
						
//							color = ColorGenerator.calculateRuleDisColor(curr_Dis - family_min_Dis
//																				, family_max_Dis - family_min_Dis
//																				, topColor
//																				, bottomColor);
							
							sim = 1;
							if(curr_Dis - family_min_Dis > 0) sim = (1-(curr_Dis - family_min_Dis)/(family_max_Dis - family_min_Dis));
							try{
							color = new Color((float)sim,(float)sim,(float)sim);
							}catch(IllegalArgumentException e)
							{
								System.out.println(sim);
								System.out.println("max "+family_max_Dis);
								System.out.println("min "+family_min_Dis);
								System.out.println("curr "+curr_Dis);
								System.out.println("id "+fr.getId()+ " id2 "+fr_2.getId());
								System.out.println("dis "+lastDis+ " dis2 "+fr_2.lastDis);
							}
							
							line_Height = (int)(disLine_Height*((curr_Dis - family_min_Dis)/(family_max_Dis - family_min_Dis)))+1;
							break;						
							
						case 1:
							sim = (FuzzyRule.compSimilarityErrorFunc(fr
																	, fr_2
																	, null));
							
							double family_max_EFsim = session.getFamily_max_RuleEFsim().get(fr.getId());
							double family_min_EFsim = session.getFamily_min_RuleEFsim().get(fr.getId());
						
							sim = (sim - family_min_EFsim)/(family_max_EFsim - family_min_EFsim);
							
							color = new Color((float)sim,(float)sim,(float)sim);
							
							line_Height = (int)(disLine_Height*(1-sim));
							break;
						case 2:
							sim = (FuzzyRule.compConsequensSim(fr, fr_2));
							
							color = new Color((float)sim,(float)sim,(float)sim);
							
							line_Height = (int)(disLine_Height*(1-sim));
							break;
						}
						
						if(sim < linkThreshold)
						{
							g2D.setColor(color);
							g2D.fillOval((rule_Width+rule_Dis)*i+rule_Width
								, (rule_Height+row_Dis)*fr.getId()+(rule_Height-line_Height)/2
								, rule_Dis
								, line_Height);
						}

					}
					
					//Conclusion  Angle		
					if(showAngle)
					{
						double angle = FuzzyRule.compConsequensAngle(fr, fr_2);
						
						if(angle >Math.PI/2) angle = Math.PI - angle;
						
						aTrans = g2D.getTransform();					
						g2D.translate((rule_Width+rule_Dis)*i+rule_Width + (rule_Dis/2)
										,(rule_Height+row_Dis)*fr.getId()+(rule_Height)/2);
						g2D.rotate(-angle);
						
						g2D.setColor(Color.red);
						g2D.drawLine( - (rule_Dis/2)
							, 0
							, (rule_Dis/2)
							, 0);
						
						g2D.setTransform(aTrans);
					}
				}
			}
		}
		
		for(FuzzyRule fr: session.getFuzzyRuleCollectionAt(visibleRules[1]).getFrList())
		{
			g2D.setColor(session.getRuleInfoAt(fr.getId()).getColor());
			
			g2D.fillOval((rule_Width+rule_Dis)*(visibleRules[1])
					, (rule_Height+row_Dis)*fr.getId()
					, rule_Width
					, rule_Height);
		}
	}
	
	//###############################################################################################
	// public To Latex
	//###############################################################################################
	
	public String toLatex()
	{
		StringBuffer[] sb = new StringBuffer[session.getMaxID()+2];
		
		int picHeight = (rule_Height+row_Dis)*(session.getMaxID()+1);
		int picWidth = session.getNumGenerations() * (rule_Width+rule_Dis);
		
		sb[0] = new StringBuffer();
		sb[session.getMaxID()+1] = new StringBuffer();
		
		sb[0].append("\\documentclass{article}");
		sb[0].append("\n");
		sb[0].append("\\usepackage{amsmath}");
		sb[0].append("\n");
		sb[0].append("\\usepackage{amsfonts}");
		sb[0].append("\n");
		sb[0].append("\\usepackage{pst-all}");
		sb[0].append("\n");
		sb[0].append("\\usepackage{pstricks}");
		sb[0].append("\n");
		sb[0].append("\\usepackage{pstricks-add}");
		sb[0].append("\n");
		sb[0].append("\\usepackage{pst-pdf}");
		sb[0].append("\n");
		
		sb[0].append("\\begin{document}");
		sb[0].append("\n");
		
		sb[0].append("\\psset{xunit=1pt,yunit=1pt}");
		sb[0].append("\n");
		
		sb[0].append("\\begin{pspicture}" +
				"(0,0)" +
				"(" + picWidth + "," + picHeight + ")");
		sb[0].append("\n");
		
		float[] rgb = new float[4];
		
		for(int i=0; i<=session.getMaxID(); i++)
		{		
			session.getRuleInfoAt(i).getColor().getRGBComponents(rgb);
			sb[0].append("\\newrgbcolor{"+ NumberFormater.integerToWord(i) +"}{"+rgb[0]+" "+rgb[1]+" "+rgb[2]+"}");
			sb[0].append("\n");
			
			sb[1+i] = new StringBuffer();
			sb[1+i].append("% Rule "+i+" ##################################################");
			sb[1+i].append("\n");
		}
		
		//draw Ruleblocks
		for(int i=0; i < session.getNewestCollectionIndex(); i++)
		{			
			for(FuzzyRule fr: session.getFuzzyRuleCollectionAt(i).getFrList())
			{			
				if(showPearls)
				{
					sb[fr.getId()].append("\\pscircle[linestyle = none, fillstyle = solid, fillcolor="+  NumberFormater.integerToWord(fr.getId()) +"]" +
							"(" + ((rule_Width+rule_Dis)*i + rule_Width/2) + "," 
									+ (picHeight - ((rule_Height+row_Dis)*fr.getId() + rule_Height/2)) + ")" +
							"{" + rule_Width/2d +"pt}");	
					sb[fr.getId()].append("\n");
				}
				FuzzyRule fr_2 = session.getFuzzyRuleCollectionAt(i+1).getFuzzyRule(fr.getId());
				
				if(fr_2 != null)
				{
					if(showConclusion)
					{
						double angelSim = FuzzyRule.compConsequensSim(fr, fr_2);
						
						if(angelSim < angelSimThreshold)
						{
							Color color = new Color(1f,(float)angelSim,(float)angelSim,1f);
							
							color.getRGBComponents(rgb);
							sb[fr.getId()].append("\\newrgbcolor{"+ NumberFormater.integerToWord(i)+ "z" + NumberFormater.integerToWord(fr.getId()) +"}{"+rgb[0]+" "+rgb[1]+" "+rgb[2]+"}");
							sb[fr.getId()].append("\n");
							
							sb[fr.getId()].append("\\psframe[linestyle=none, fillstyle = solid, fillcolor = "+NumberFormater.integerToWord(i)+ "z" + NumberFormater.integerToWord(fr.getId())+"]" +
									"(" + ((rule_Width+rule_Dis)*i + rule_Width + rule_Dis/2) +"," + (picHeight - ((rule_Height+row_Dis)*fr.getId() - (row_Dis)/2)) +")" +
									"(" + ((rule_Width+rule_Dis)*i + rule_Width + rule_Dis)
										+ "," + (picHeight - ((rule_Height+row_Dis)*fr.getId() + (row_Dis)/2 + rule_Height)) +")");
							sb[fr.getId()].append("\n");
						}
					}
					if(showPlaneDis)
					{
						double hyperDis = Math.abs(fr.getFc().getCharacteristics()[fr.getFc().getCharacteristics().length-1]
						-fr_2.getFc().getCharacteristics()[fr_2.getFc().getCharacteristics().length-1]);
			
						hyperDis = hyperDis/(session.getOut_Range()[session.getFr_conclusionSize()-1][1]-session.getOut_Range()[session.getFr_conclusionSize()-1][0]);
						
						if(hyperDis > planeDisThreshold)
						{
							Color color = new Color(1-(float)hyperDis,1-(float)hyperDis,1f,1f);
							
							color.getRGBComponents(rgb);
							sb[fr.getId()].append("\\newrgbcolor{"+ NumberFormater.integerToWord(i)+ "zz" + NumberFormater.integerToWord(fr.getId()) +"}{"+rgb[0]+" "+rgb[1]+" "+rgb[2]+"}");
							sb[fr.getId()].append("\n");
							
							sb[fr.getId()].append("\\psframe[linestyle=none, fillstyle = solid, fillcolor = "+NumberFormater.integerToWord(i)+ "zz" + NumberFormater.integerToWord(fr.getId())+"]" +
									"(" + ((rule_Width+rule_Dis)*i + rule_Width) +"," + (picHeight - ((rule_Height+row_Dis)*fr.getId() - (row_Dis)/2)) +")" +
									"(" + ((rule_Width+rule_Dis)*i + rule_Width + rule_Dis/2)
										+ "," + (picHeight - ((rule_Height+row_Dis)*fr.getId() + (row_Dis)/2 + rule_Height)) +")");
							sb[fr.getId()].append("\n");
						}
					}
					
					int line_Height = 0;
					Color color = null;
					
					if(this.showLinks)
					{
						double sim = 1;
						
						switch(distanceType)
						{
						case 0: 
							double curr_Dis = Session.calcDistance(fr, fr_2);
							double family_max_Dis = session.getFamily_max_RuleDis().get(fr.getId());
							double family_min_Dis = session.getFamily_min_RuleDis().get(fr.getId());
							
							sim = 1;
							if(curr_Dis - family_min_Dis > 0) sim = (1-(curr_Dis - family_min_Dis)/(family_max_Dis - family_min_Dis));
							
							color = new Color((float)sim,(float)sim,(float)sim);
							
							line_Height = (int)(disLine_Height*((curr_Dis - family_min_Dis)/(family_max_Dis - family_min_Dis)))+1;
							break;						
							
						case 1:
							sim = (FuzzyRule.compSimilarityErrorFunc(fr
																	, fr_2
																	, null));
							
							double family_max_EFsim = session.getFamily_max_RuleEFsim().get(fr.getId());
							double family_min_EFsim = session.getFamily_min_RuleEFsim().get(fr.getId());
						
							sim = (sim - family_min_EFsim)/(family_max_EFsim - family_min_EFsim);
							
							color = new Color((float)sim,(float)sim,(float)sim);
							
							line_Height = (int)(disLine_Height*(1-sim));
							break;
						case 2:
							sim = (FuzzyRule.compConsequensSim(fr, fr_2));
							
							color = new Color((float)sim,(float)sim,(float)sim);
							
							line_Height = (int)(disLine_Height*(1-sim));
							break;
						}
	
						if(sim < linkThreshold)
						{
						color.getRGBComponents(rgb);
						sb[fr.getId()].append("\\newrgbcolor{"+ NumberFormater.integerToWord(i)+ "x" + NumberFormater.integerToWord(fr.getId()) +"}{"+rgb[0]+" "+rgb[1]+" "+rgb[2]+"}");
						sb[fr.getId()].append("\n");
				
						sb[fr.getId()].append("\\psellipse[linestyle = none, fillstyle = solid, fillcolor="+  NumberFormater.integerToWord(i)+ "x" + NumberFormater.integerToWord(fr.getId()) +"]" +
								"(" + ((rule_Width+rule_Dis)*i + rule_Width + rule_Dis/2) + "," 
										+ (picHeight - ((rule_Height+row_Dis)*fr.getId() + rule_Height/2)) + ")" +
								"(" + (rule_Dis/2d + 0.6) +"pt," + line_Height/2d + "pt)");	
						sb[fr.getId()].append("\n");
						}
					}
					//Conclusion  Angle		
					if(showAngle)
					{
						double angel = FuzzyRule.compConsequensAngle(fr, fr_2);
						
						angel = (int)(MathUtil.radToGrad(angel)*1000)/1000;
						
						sb[fr.getId()].append("\\rput{" + angel +"}" +
								"(" + ((rule_Width+rule_Dis)*i + rule_Width + rule_Dis/2) + "," + (picHeight - ((rule_Height+row_Dis)*fr.getId() + rule_Height/2)) + "){");
						sb[fr.getId()].append("\n");
						sb[fr.getId()].append("\\psline[linestyle=solid, linecolor=red]{-}" +
								"(" + (-rule_Dis/2) + "," 
											+ 0 + ")" +
								"(" + rule_Dis/2 + "," 
											+ 0 + ")");
						sb[fr.getId()].append("\n");
						sb[fr.getId()].append("}");
						sb[fr.getId()].append("\n");
					}
				}
			}
		}
		
		int lastIndex = session.getNewestCollectionIndex();
		
		for(FuzzyRule fr: session.getFuzzyRuleCollectionAt(lastIndex).getFrList())
		{
			sb[fr.getId()].append("\\pscircle[linestyle = none, fillstyle = solid, fillcolor="+  NumberFormater.integerToWord(fr.getId()) +"]" +
					"(" + ((rule_Width+rule_Dis)*lastIndex + rule_Width/2) + "," 
							+ (picHeight - ((rule_Height+row_Dis)*fr.getId() + rule_Height/2)) + ")" +
					"{" + rule_Width/2d +"pt}");	
			sb[fr.getId()].append("\n");
		}
		
		sb[session.getMaxID()+1].append("\\end{pspicture}");
		sb[session.getMaxID()+1].append("\n");
		sb[session.getMaxID()+1].append("\\end{document}");
		
		StringBuffer retString = new StringBuffer();
		
		for(int i=0; i<sb.length; i++)
		{
			retString.append(sb[i]);
		}
		
		return retString.toString();
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void zoomIN(){
		
		scaleFactor = scaleFactor*1.1;
		timeLineSlider.setScaleFactor(scaleFactor);
		timeLineSlider.repaintDrawPanel();
		drawPanel.repaint();
		
		int width = (int)((rule_Width+rule_Dis)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_Height+row_Dis)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
		
		attrPanel.repaint();
	}
	
	public void zoomOut(){
		
		scaleFactor = scaleFactor/1.1;
		timeLineSlider.setScaleFactor(scaleFactor);
		timeLineSlider.repaintDrawPanel();
		drawPanel.repaint();
		
		int width = (int)((rule_Width+rule_Dis)*session.getNumGenerations()*scaleFactor)+1;
		int height= (int)((rule_Height+row_Dis)*(session.getMaxID()+1)*scaleFactor)+1;
		drawPanel.setPreferredSize(new Dimension(width, height));
		drawPanel.revalidate();
		
		attrPanel.repaint();
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################

	public Session getSession()
	{
		return session;
	}
	
	//###############################################################################################
	// inner Classes
	//###############################################################################################
	
	private class DrawAttribute extends DrawObject{
		
		//###############################################################################################
		// Override Methods DrawObject
		//###############################################################################################

		@Override
		public void draw(Graphics2D g2d) 
		{
			g2d.setColor(new Color(240,240,255));
			g2d.fillRect(0
					, 0
					, attrPanel.getWidth()
					, attrPanel.getHeight());
			
			g2d.setColor(Color.black);
			for(int i=0; i<=session.getMaxID(); i++)
			{
				g2d.drawString(i+""
					, 10
					, (int)((rule_Height + (rule_Height+row_Dis)*i)*scaleFactor) - attrPadding_North);
			}		
		}	
	}
	
	public int getDistanceType() {
		return distanceType;
	}

	public void setDistanceType(int distanceType) {
		this.distanceType = distanceType;
	}

	public int getNumHLSamples() {
		return numHLSamples;
	}

	public void setNumHLSamples(int numHLSamples) {
		this.numHLSamples = numHLSamples;
	}

	public boolean isShowAngle() {
		return showAngle;
	}

	public void setShowAngle(boolean showAngle) {
		this.showAngle = showAngle;
	}

	public boolean isShowConclusion() {
		return showConclusion;
	}

	public void setShowConclusion(boolean showConclusion) {
		this.showConclusion = showConclusion;
	}

	public double getLinkThreshold() {
		return linkThreshold;
	}

	public void setLinkThreshold(double linkThreshold) {
		this.linkThreshold = linkThreshold;
		this.repaintDrawPanel();
	}

	public double getAngelSimThreshold() {
		return angelSimThreshold;
	}

	public void setAngelSimThreshold(double angelSimThreshold) {
		this.angelSimThreshold = angelSimThreshold;
		this.repaintDrawPanel();
	}

	public double getPlaneDisThreshold() {
		return planeDisThreshold;
	}

	public void setPlaneDisThreshold(double planeDisThreshold) {
		this.planeDisThreshold = planeDisThreshold;
		this.repaintDrawPanel();
	}

	public boolean isShowPearls() {
		return showPearls;
	}

	public void setShowPearls(boolean showPearls) {
		this.showPearls = showPearls;
	}

	public boolean isShowLinks() {
		return showLinks;
	}

	public void setShowLinks(boolean showLinks) {
		this.showLinks = showLinks;
	}

	public boolean isShowPlaneDis() {
		return showPlaneDis;
	}

	public void setShowPlaneDis(boolean showPlaneDis) {
		this.showPlaneDis = showPlaneDis;
	}

	@Override
	protected void setIDList(TreeSet<Integer> ID_List) {
		// TODO Auto-generated method stub
		
	}

}
