package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import fuzzy.FuzzyRule;
import guiCore.Session;

import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import draw.DrawObject;
import draw.DrawPanel;

public class AxisWindowDialog extends JDialog implements ActionListener, ChangeListener{
	
	private Session session;
	
	private JSlider[] minSlider_A;
	private JSlider[] widthSlider_A;
	
	private DrawObject[] axisDraw_A;
	private DrawPanel[] drawPanel_A;
	
//	private JLabel[] attrLabel_A;
	private JRadioButton[] radio_1_A;
	private JRadioButton[] radio_2_A;
	private ButtonGroup[] bGroup_A;
	
	private JPanel[] sliderPanel_A;
	private JPanel[] labelPanel_A;
	private JPanel[] subPanel_A;
	
	private JPanel mainPanel;
	
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public AxisWindowDialog(Session session_P)
	{
		this.session = session_P;
		
		minSlider_A		= new JSlider[session.getFr_premiseSize()];
		widthSlider_A	= new JSlider[session.getFr_premiseSize()];
		drawPanel_A		= new DrawPanel[session.getFr_premiseSize()];
		
//		attrLabel_A		= new JLabel[session.getFr_premiseSize()];
		radio_1_A		= new JRadioButton[session.getFr_premiseSize()];
		radio_2_A		= new JRadioButton[session.getFr_premiseSize()];
		bGroup_A		= new ButtonGroup[session.getFr_premiseSize()];
		
		sliderPanel_A	= new JPanel[session.getFr_premiseSize()];
		labelPanel_A 	= new JPanel[session.getFr_premiseSize()];
		subPanel_A 		= new JPanel[session.getFr_premiseSize()];
		
		mainPanel = new JPanel();
		mainPanel.setLayout(new GridLayout(session.getFr_premiseSize()+1, 1));
		
		for(int i=0; i<session.getFr_premiseSize(); i++)
		{
			int minSlider_Value = (int)((session.getCustomAttr_Range()[i][0] - session.getAttr_Range()[i][0]) * 1000
					/(session.getAttr_Range()[i][1] - session.getAttr_Range()[i][0]));
			
			int widthSlider_Value = (int)((session.getCustomAttr_Range()[i][1] - session.getAttr_Range()[i][0]) * 1000
					/(session.getAttr_Range()[i][1] - session.getAttr_Range()[i][0]));
			
			minSlider_A[i] = new JSlider(JSlider.HORIZONTAL, 0, 1000, minSlider_Value);
			minSlider_A[i].setName("min_"+i);
			minSlider_A[i].addChangeListener(this);
			
			widthSlider_A[i] = new JSlider(JSlider.HORIZONTAL, 0, 1000, widthSlider_Value);
			widthSlider_A[i].setName("width_"+i);
			widthSlider_A[i].addChangeListener(this);
			
			drawPanel_A[i] = new DrawPanel();
			drawPanel_A[i].setBackground(Color.black);
			drawPanel_A[i].addDrawObject(new AxisDraw(i));
			
			radio_1_A[i] = new JRadioButton("default");
			radio_1_A[i].setActionCommand("default_"+i);
			radio_1_A[i].addActionListener(this);
			
			radio_2_A[i] = new JRadioButton("custom");
			radio_2_A[i].setActionCommand("customt_"+i);
			radio_2_A[i].addActionListener(this);
			
			bGroup_A[i] = new ButtonGroup();
			bGroup_A[i].add(radio_1_A[i]);
			bGroup_A[i].add(radio_2_A[i]);
			
			if(session.getCustomAttrFlagAt(i))
			{
				radio_2_A[i].setSelected(true);
			}
			else
			{
				radio_1_A[i].setSelected(true);
			}
			
			sliderPanel_A[i] = new JPanel();
			sliderPanel_A[i].setLayout(new GridLayout(3,1));
			sliderPanel_A[i].add(minSlider_A[i]);
			sliderPanel_A[i].add(widthSlider_A[i]);
			sliderPanel_A[i].add(drawPanel_A[i]);
			
			labelPanel_A[i] = new JPanel();
			labelPanel_A[i].setLayout(new GridLayout(3,1));
			labelPanel_A[i].setBorder(new TitledBorder("Attr "+i));
			labelPanel_A[i].add(radio_1_A[i]);
			labelPanel_A[i].add(radio_2_A[i]);
			
			subPanel_A[i] = new JPanel();
			subPanel_A[i].setLayout(new BorderLayout());
			subPanel_A[i].setBorder(new EtchedBorder());
			subPanel_A[i].add(labelPanel_A[i], BorderLayout.WEST);
			subPanel_A[i].add(sliderPanel_A[i], BorderLayout.CENTER);
			
			mainPanel.add(subPanel_A[i]);
		}
		
		Container cp = this.getContentPane();
		cp.setLayout(new BorderLayout());
		cp.add(new JScrollPane(mainPanel), BorderLayout.CENTER);
		
		this.setModal(true);
		this.setSize(930, 600);
		this.setLocation((int)(this.getToolkit().getScreenSize().getWidth()/2 - this.getWidth()/2)
							, (int)(this.getToolkit().getScreenSize().getHeight()/2 - this.getHeight()/2));
		this.setVisible(true);
	}

	//###############################################################################################
	// Override Methods ActionListener
	//###############################################################################################

	@Override
	public void actionPerformed(ActionEvent arg0) 
	{
		String[] info = arg0.getActionCommand().split("_");
		int index = Integer.parseInt(info[1]);
		
		if(radio_1_A[index].isSelected())
		{
			session.setCustomAttrFlagAt(index, false);
		}
		else
		{
			session.setCustomAttrFlagAt(index, true);
		}
		
		session.repaintRV();
	}
	
	//###############################################################################################
	// Override Methods ChangeListener
	//###############################################################################################

	@Override
	public void stateChanged(ChangeEvent arg0) 
	{
		JSlider slider = (JSlider)arg0.getSource();

		String[] info = slider.getName().split("_");
		int index = Integer.parseInt(info[1]);
		
		if(info[0].equals("min"))
		{			
			if(slider.getValue() > widthSlider_A[index].getValue())
			{
				slider.setValue(widthSlider_A[index].getValue()-1);
			}
					
			session.getCustomAttr_Range()[index][0] = slider.getValue()
														*(session.getAttr_Range()[index][1] - session.getAttr_Range()[index][0])
														/ 1000 
														+ session.getAttr_Range()[index][0];
		}
		else
		{
			if(slider.getValue() < minSlider_A[index].getValue())
			{
				slider.setValue(minSlider_A[index].getValue()+1);
			}
			
			session.getCustomAttr_Range()[index][1] = slider.getValue()
					*(session.getAttr_Range()[index][1] - session.getAttr_Range()[index][0])
					/ 1000 
					+ session.getAttr_Range()[index][0];
		}
		
		drawPanel_A[index].repaint();
		session.repaintRV();
	}
	
	//***********************************************************************************************
	//###############################################################################################
	// inner Class AxisDraw
	//###############################################################################################
	//***********************************************************************************************
	
	private class AxisDraw extends DrawObject{
		
		public int id;
		
		public int padding_x = 5;
		public int padding_y = 10;
		
		public int axis_Width = 800;
		
		//###############################################################################################
		// Constructor
		//###############################################################################################
		
		public AxisDraw(int id_P)
		{
			this.id = id_P;
		}
		
		//###############################################################################################
		// Override Methods DrawObject
		//###############################################################################################
		
		@Override
		public void draw(Graphics2D g2d) 
		{			
			int newX = (int)((session.getCustomAttr_Range()[id][0] - session.getAttr_Range()[id][0]) * axis_Width
						/(session.getAttr_Range()[id][1] - session.getAttr_Range()[id][0]));
			
			int newWidth = (int)((session.getCustomAttr_Range()[id][1] - session.getAttr_Range()[id][0]) * axis_Width
					/(session.getAttr_Range()[id][1] - session.getAttr_Range()[id][0]));
			newWidth = newWidth - newX;
			
			g2d.setColor(Color.red);
			g2d.fillRect(padding_x
					 + newX
					, padding_y - 5
					, newWidth
					, 10);
			
			//draw Axis
			g2d.setColor(Color.white);
			g2d.drawLine(padding_x
					, padding_y
					, padding_x + axis_Width
					, padding_y);
			
			for(FuzzyRule fr: session.getCurrPopulation().getFrList()) 			
			{
				int x = (int)((fr.getCenter()[id] - session.getAttr_Range()[id][0]) * axis_Width
							/(session.getAttr_Range()[id][1] - session.getAttr_Range()[id][0]));
				
				g2d.setColor(session.getRuleInfoAt(fr.getId()).getColor());
				g2d.drawLine(x
						, padding_y - 5
						, x
						, padding_y +5);
			}
			
		}		
	}


}
