package gui;

import guiCore.Session;
import guiUtil.RolloverButton;
import guiUtil.RolloverToggleButton;
import guiUtil.TabToWindowSwitch;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;

import listener.WindowClosingAdapter;

public class SessionFrame extends JFrame implements ActionListener{
	
	private JSplitPane splitpane_1;
	private JSplitPane splitpane_2;
	
	private JPanel panel_South;
	private JPanel panel_West;
	private JPanel panel_East;
	
	private JTabbedPane tabbedPane_South;
	private JTabbedPane tabbedPane_West;
	private JTabbedPane tabbedPane_East;
	
	private JMenuBar menuBar;
	private JMenu editMenu;
	private JMenuItem colorItem;
	private JMenuItem attrZoomItem;
	
	private JPanel rulePanel;
	
	private JCheckBox autoRangeCheckBox;
	private RolloverButton autoRangeButton;
	
	private RolloverToggleButton livingRulesButton;
	
	private JToolBar toolBar;
	
	private Session session;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public SessionFrame(Session session_P)
	{
		this.session = session_P;
		
		tabbedPane_South = new JTabbedPane();
		tabbedPane_West = new JTabbedPane();
		tabbedPane_East = new JTabbedPane();
		
		panel_South = new JPanel();
		panel_South.setLayout(new BorderLayout());
		panel_South.add(tabbedPane_South);
		
		panel_West = new JPanel();
		panel_West.setLayout(new BorderLayout());
		panel_West.add(tabbedPane_West);
		
		panel_East = new JPanel();
		panel_East.setLayout(new BorderLayout());
		panel_East.add(tabbedPane_East);
		
		splitpane_2 = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitpane_2.setLeftComponent(panel_West);
		splitpane_2.setRightComponent(panel_East);
		
		splitpane_1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitpane_1.setTopComponent(splitpane_2);
		splitpane_1.setBottomComponent(panel_South);
		splitpane_1.setDividerLocation(500);
		
		colorItem = new JMenuItem("Color");
		colorItem.setActionCommand("editColor");
		colorItem.addActionListener(this);
		
		attrZoomItem = new JMenuItem("Attribute Zoom");
		attrZoomItem.setActionCommand("zoomAttr");
		attrZoomItem.addActionListener(this);
				
		editMenu = new JMenu("Edit");
		editMenu.add(colorItem);
		editMenu.add(attrZoomItem);
		
		menuBar = new JMenuBar();
		menuBar.add(editMenu);
		
		this.setJMenuBar(menuBar);
		
		autoRangeButton = new RolloverButton("Auto-Range");
		
		autoRangeCheckBox = new JCheckBox();
		autoRangeCheckBox.setActionCommand("autoRange");
		autoRangeCheckBox.addActionListener(this);
		
		livingRulesButton = new RolloverToggleButton("olr");
		livingRulesButton.setActionCommand("olr");
		livingRulesButton.addActionListener(this);
		
		toolBar = new JToolBar();
		toolBar.setLayout(new FlowLayout(FlowLayout.LEFT));
		toolBar.add(autoRangeCheckBox);
		toolBar.add(autoRangeButton);
		toolBar.add(livingRulesButton);
		
		rulePanel = new JPanel();
		rulePanel.setLayout(new BorderLayout());
		
		Container cp = this.getContentPane();
		cp.setLayout(new BorderLayout());
		cp.add(splitpane_1, BorderLayout.CENTER);
		cp.add(new JScrollPane(rulePanel), BorderLayout.WEST);
		cp.add(toolBar, BorderLayout.NORTH);
		
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.addWindowListener(new WindowClosingAdapter());
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void addToRulePanel(Component component)
	{		
		rulePanel.removeAll();
		rulePanel.add(component, BorderLayout.CENTER);
	}
	
	public void addSouth(Component component)
	{
		TabToWindowSwitch tws = new TabToWindowSwitch(tabbedPane_South, component);
	}
	public void addWest(Component component)
	{
		TabToWindowSwitch tws = new TabToWindowSwitch(tabbedPane_West, component);
	}
	public void addEast(Component component)
	{
		TabToWindowSwitch tws = new TabToWindowSwitch(tabbedPane_East, component);
	}
	public void addToolBar(JToolBar toolBar)
	{
		this.add(toolBar, BorderLayout.SOUTH);
	}
	
	//###############################################################################################
	// Override Methods Actionlistener
	//###############################################################################################

	@Override
	public void actionPerformed(ActionEvent e) 
	{
		if(e.getActionCommand().equals("editColor"))
		{
			ColorDialog cd = new ColorDialog(this, session.getColorGen());
		}
		else if(e.getActionCommand().equals("zoomAttr"))
		{
			AxisWindowDialog awd = new AxisWindowDialog(session);
		}
		else if(e.getActionCommand().equals("autoRange"))
		{
			session.setAutoCustomFlag(this.autoRangeCheckBox.isSelected());
		}
		else if(e.getActionCommand().equals("olr"))
		{
			session.setOnlyLiving(livingRulesButton.isSelected());
		}
	}
}
