package guiCore;

import java.util.ArrayList;

import fuzzy.FuzzyRule;

public class EvolvingHistogramm {
	
	private int lastGen = -1;
	
	private double forgetFactor = 1;
	
	private ArrayList<Double> entropy_List = new ArrayList<Double>();
	private double maxEvo = Double.MIN_VALUE;
	
	private ArrayList<Integer> binIndex_To_ID_List = new ArrayList<Integer>();
	private ArrayList<Bin> bin_List = new ArrayList<Bin>();
	
	//###############################################################################################
	// Methods
	//###############################################################################################
	
	public void updateHistogramm(double[][] data, FuzzyRuleCollection frCol)
	{
		lastGen++;
		
		forget();
		
		ArrayList<Bin> old_Bins = this.bin_List;
		ArrayList<Integer> old_index_to_id_List = binIndex_To_ID_List;
		
		bin_List = new ArrayList<Bin>();
		binIndex_To_ID_List = new ArrayList<Integer>();
		
		double[] normMembership_A = frCol.getNormalizedMemberships(data[0]);
		
		for(int r=0; r < frCol.getFrList().size(); r++)
		{
			double value = 0;
			int index = old_index_to_id_List.indexOf(frCol.getFrList().get(r).getId());
			
			if(index >= 0)
			{
				value = old_Bins.get(index).value;
			}
		 
			value = value + normMembership_A[r];
			
			addBin(frCol.getFrList().get(r).getId(), value);
		}	
		
		for(int i=1; i<data.length; i++)
		{	
			normMembership_A = frCol.getNormalizedMemberships(data[i]);
			
			for(int r=0; r < frCol.getFrList().size(); r++)
			{
				fillBin(r, normMembership_A[r]);
			}
		}
	
		normalize();
		addEntropy();
	}
	
	private void fillBin(int index, double value){
		
		double oldValue = bin_List.get(index).value;
		
		bin_List.get(index).value = bin_List.get(index).value + value;	
		if(new Double(bin_List.get(index).value).isNaN() && !new Double(oldValue).isNaN())
		System.err.println("oldValue: "+oldValue+" value: "+value);	
	}
	
	private void addEntropy()
	{
		double entropy = 0;
		
		for(Bin b: bin_List)
		{
			entropy = entropy - (b.value * Math.log(b.value));
		}
		
		entropy = entropy / Math.log(bin_List.size());

		entropy_List.add(entropy);
		if(maxEvo < entropy) maxEvo = entropy;
	}
	
	public void forget()
	{
		for(Bin b: bin_List)
		{
			b.value = b.value * forgetFactor;
		}
	}

	public void addBin(int id, double value)
	{
		bin_List.add(new Bin(value));
		
		binIndex_To_ID_List.add(id);
	}
	
	public void removeBin(int id)
	{
		int index = binIndex_To_ID_List.indexOf(id);
		binIndex_To_ID_List.remove(index);
		bin_List.remove(index);
	}
	
	public void normalize()
	{
		double sum = 0;
		
		for(Bin b: bin_List)
		{

			sum = sum + b.value;
		}

		
		for(Bin b: bin_List)
		{
			b.value = b.value/sum;
		}
	}
	
	public void incGen()
	{
		lastGen++;
	}

	public int getLastGen() {
		return lastGen;
	}
	
	private class Bin{
		
		public double value;
		
		public Bin(double value)
		{
			this.value = value;
		}
		
	}

	public ArrayList<Double> getEntropy_List() {
		return entropy_List;
	}

	public void setEntropy_List(ArrayList<Double> entropy_List) {
		this.entropy_List = entropy_List;
	}

	public double getMaxEvo() {
		return maxEvo;
	}
}
