package guiCore;

import io.RuleCollectionWriter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JFrame;

import draw.AndrewsView;
import draw.AttributeHistoryView;
import draw.DrawPanel;
import draw.FamilyTreeView;
import draw.FisDifMapView;
import draw.GlobalMonitor;
import draw.LocalWalkView;
import draw.MatrixView;
import draw.ParallelAxisView;
import draw.RuleListView;
import draw.TimeLineView;
import exception.AlreadyInitializedException;
import fuzzy.FuzzyConclusion;
import fuzzy.FuzzyRule;
import fuzzy.FuzzySet;
import gui.ControlToolBar;
import gui.SessionFrame;

public class FISVis {
	
	private SessionFrame sessionFrame;
	private ControlToolBar ctb;
	
	private GlobalMonitor gMonitor;
	private ArrayList<RuleVisualizer> rv_List = new ArrayList<RuleVisualizer>();
	
	private Session session;
		
	private int threadSpeed = 10;
	private boolean isStop = false;
	
	private String sessionFile;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public FISVis()
	{		
		session = new Session(this);
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	public boolean startSession(double[][] mean_A
			, double[][] width_A
			, double[][] out_A
			, int[] rule_to_ID
			, int[][] ancestors
			, int max_ID
			, double[][] attr_range
			, double[] out_range
			, double[][] data
			, int numDataAll
			, String url)
	{	
		this.sessionFile = url;
		File file = new File(url);
		
		if(!file.exists()) file.mkdirs();
		
		file = new File(file.toString()+"\\exp_1.efis");
		
		RuleCollectionWriter rcw = new RuleCollectionWriter();		
		
		rcw.write(mean_A
					, width_A
					, out_A
					, ancestors
					, attr_range
					, out_range
					, rule_to_ID
					, max_ID
					, data
					, numDataAll
					, file);
		
		//create FuzzyRule
		ArrayList<FuzzyRule> fr_List = new ArrayList<FuzzyRule>();
		
		FuzzyRule fr = null;
		
		for(int r=0; r < mean_A.length; r++)
		{
			//create FuzzySets antecedent
			ArrayList<FuzzySet> fs_List = new ArrayList<FuzzySet>();
			
			FuzzySet fs = null;
			
			for(int f=0; f<mean_A[r].length; f++)
			{
				fs = FuzzySet.createFuzzySet(FuzzySet.GAUSS, new double[]{mean_A[r][f], width_A[r][f]});
				fs_List.add(fs);
			}
			
			// rule_to_ID[r]-1 because Matlab starts with 1, java with 0
			fr = new FuzzyRule(rule_to_ID[r]
					,0
					,ancestors[r]
					,fs_List
					,new String("A")
					,FuzzyConclusion.createFuzzyConclusion(FuzzyConclusion.LINEAR_FUNCTION, out_A[r]));
			
				fr_List.add(fr);
		}
		
		int premiseSize = fr_List.get(0).numFuzzySets();
		int conclusionSize = fr_List.get(0).getFc().getCharacteristics().length;
		
		session.setSessionURL(url);
		session.setProtoDatapoint(data[0]);
		session.initOutRange(fr_List.get(0));
		
		try {
			session.setFuzzyRuleInfos(premiseSize, conclusionSize);	
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AlreadyInitializedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		session.initCustomAttr_Range(attr_range);
		updateStoredSession(fr_List, attr_range, out_range, max_ID, data, numDataAll);
		buildSessionFrame();
		
		Thread th = new Thread(new SessionThread());
		th.start();
		
		sessionFrame.setVisible(true);
			
			return true;
	}	
	
	public void startStoredSession(ArrayList<FuzzyRule> fr_List_P
									, double[][] attr_range
									, double[] out_range
									, int max_ID_P
									, double[][] data
									, int numDataAll
									, String url) throws NullPointerException
	{		
		if(fr_List_P == null) throw new NullPointerException();
		
		int premiseSize = fr_List_P.get(0).numFuzzySets();
		int conclusionSize = fr_List_P.get(0).getFc().getCharacteristics().length;
		
		session.setSessionURL(url);
		session.setProtoDatapoint(data[0]);
		session.initOutRange(fr_List_P.get(0));
		
		try {
			session.setFuzzyRuleInfos(premiseSize, conclusionSize);	
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AlreadyInitializedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		session.initCustomAttr_Range(attr_range);
		updateStoredSession(fr_List_P, attr_range, out_range, max_ID_P, data, numDataAll);
		buildSessionFrame();
		
		Thread th = new Thread(new SessionThread());
		th.start();
		
		sessionFrame.setVisible(true);
	}
	
	public void updateSession(double[][] mean_A
			, double[][] width_A
			, double[][] out_A
			, int[] rule_to_ID
			, int[][] ancestors
			, int max_ID
			, double[][] attr_range
			, double[] out_range
			, double[][] data
			, int numDataAll)
	{		
		RuleCollectionWriter rcw = new RuleCollectionWriter();
		
		rcw.write(mean_A
				, width_A
				, out_A
				, ancestors
				, attr_range
				, out_range
				, rule_to_ID
				, max_ID
				, data
				, numDataAll
				, new File(sessionFile + "\\exp_1.efis"));
		
		
		//create FuzzyRule
		ArrayList<FuzzyRule> fr_List = new ArrayList<FuzzyRule>();
		
		FuzzyRule fr = null;
		
		for(int r=0; r < mean_A.length; r++)
		{
			//create FuzzySets antecedent
			ArrayList<FuzzySet> fs_List = new ArrayList<FuzzySet>();
			
			FuzzySet fs = null;
			
			for(int f=0; f<mean_A[r].length; f++)
			{
				fs = FuzzySet.createFuzzySet(FuzzySet.GAUSS, new double[]{mean_A[r][f], width_A[r][f]});
				fs_List.add(fs);
			}
		
			fr = new FuzzyRule(rule_to_ID[r]
					,0
					,ancestors[r]
					,fs_List
					,new String("A")
					,FuzzyConclusion.createFuzzyConclusion(FuzzyConclusion.LINEAR_FUNCTION, out_A[r]));
			
			fr_List.add(fr);
		}
		
		//order of methods call is important, do not change!.
		session.setAttr_Range(attr_range);
//		session.setOut_Range(out_range);
		
		session.addFuzzyRulePopulation(new FuzzyRuleCollection(fr_List
				,session.getLastPopIndex()+1), max_ID);
		session.updateOutRange();
		session.setData(data, numDataAll);
		session.updateGlobalMonitor();
	}

	public void updateStoredSession(ArrayList<FuzzyRule> fr_List_P
									, double[][] attr_range
									, double[] out_range
									, int max_ID_P
									, double[][] data
									, int numDataAll)
	{
		//order of methods call is important, do not change!.
		session.setAttr_Range(attr_range);
//		session.setOut_Range(out_range);
		
		session.addFuzzyRulePopulation(new FuzzyRuleCollection(fr_List_P
				,session.getLastPopIndex()+1), max_ID_P);
		session.updateOutRange();
		session.setData(data, numDataAll);
		session.updateGlobalMonitor();
	}
	
	public void stopStreamV()
	{
		ctb.stop();
		this.setStop(true);
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################
	
	private void buildSessionFrame()
	{		
		sessionFrame = new SessionFrame(session);
		
		gMonitor = new GlobalMonitor(session);
		
		MatrixView 				mv = new MatrixView(session);
		ParallelAxisView 		pav = new ParallelAxisView(session);
		LocalWalkView 			lwv = new LocalWalkView(session);
		TimeLineView 			tlv = new TimeLineView(session);
		RuleListView 			rlv = new RuleListView(session);
		AttributeHistoryView 	ahv = new AttributeHistoryView(session);
		FamilyTreeView 			ftv = new FamilyTreeView(session);
		FisDifMapView			fdmv = new FisDifMapView(session);
		AndrewsView				av = new AndrewsView(session);
		
		rlv.addDropTarget(pav);
		rlv.addDropTarget(lwv);
		rlv.addDropTarget(ahv);
		rlv.addDropTarget(av);
		
		ftv.addDropTarget(ahv);
		ftv.addDropTarget(pav);
		ftv.addDropTarget(av);
		
		rv_List.add(mv);
		rv_List.add(pav);
		rv_List.add(lwv);
		rv_List.add(tlv);
		rv_List.add(rlv);
		rv_List.add(ahv);
		rv_List.add(ftv);
//		rv_List.add(fdmv);
		rv_List.add(av);
		
		Component c_0 = gMonitor.getComponent();
		c_0.setName("Monitor");
		Component dp = mv.getComponent();
		dp.setName("Matrix");
		Component dp_2 = pav.getComponent();
		dp_2.setName("Parallel Coordinates");
		Component dp_3 = tlv.getComponent();
		dp_3.setName("Rule Chains");
		Component c_4 = lwv.getComponent();
		c_4.setName("TFP");
		Component c_5 = rlv.getComponent();
		c_5.setName("Rules");
		Component c_6 = ahv.getComponent();
		c_6.setName("VPC Timeline");
		Component c_7 = ftv.getComponent();
		c_7.setName("Family Tree");
//		Component c_8 = fdmv.getComponent();
		Component c_9 = av.getComponent();
		c_9.setName("c-s-C");

		
		ctb = new ControlToolBar(this);
				
		sessionFrame.addEast(c_0);
		sessionFrame.addEast(dp_2);
		sessionFrame.addEast(c_9);
		sessionFrame.addWest(dp);
		sessionFrame.addWest(c_4);
		sessionFrame.addWest(c_7);
//		sessionFrame.addWest(c_8);
		sessionFrame.addSouth(dp_3);
		sessionFrame.addSouth(c_6);
		sessionFrame.addToRulePanel(c_5);
		sessionFrame.addToolBar(ctb);
		sessionFrame.setSize(1200, 900);
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public boolean isStop() {
		return isStop;
	}

	public void setStop(boolean isStop) {
		
		if(!isStop){
			session.setCurrPopulationIndex(-1);
		}
		this.isStop = isStop;
	}
	
	public Session getSession() {
		return session;
	}
	
	public void updateMonitor()
	{
		try
		{
			gMonitor.updateMonitor();
		}
		catch(NullPointerException e)
		{
			//TODO nothing
		}
		
	}

	public void repaintRV()
	{
		gMonitor.updateMonitor();
		
		for(RuleVisualizer rv: rv_List)
		{
			rv.repaintDrawPanel();
		}
	}
	
	//###############################################################################################
	// INNER CLASS ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//###############################################################################################

	private class SessionThread implements Runnable {
		
		private int lastPop = -1;

		@Override
		public void run() {
			
			while(true)
			{
				try 
				{
					Thread.sleep(threadSpeed);
				}
				catch(InterruptedException e)
				{
					e.printStackTrace();
				}
				
				if(!isStop) session.pollChronologicQueue();
				
				if(lastPop != session.getCurrPopulationIndex())
				{
					lastPop = session.getCurrPopulationIndex();
					
					repaintRV();
				}				
			}
		}				
	}

	public SessionFrame getSessionFrame() {
		return sessionFrame;
	}
}
