package guiCore;

import java.util.ArrayList;
import java.util.zip.DataFormatException;

import fuzzy.FuzzyRule;
import fuzzy.PartitionMeasure;

public class GlobalMonitorFunctions {
	
	Session session;
	
	private int lastGen = -1;
	
	private final EvolvingHistogramm evoHis = new EvolvingHistogramm();
	
	//membership Abdeckung
	private ArrayList<Double> entropy2_List = new ArrayList<Double>();
	private ArrayList<Double> globalMem_List = new ArrayList<Double>();
	private ArrayList<Double> fis_fis_Dis_List = new ArrayList<Double>();
	private ArrayList<Double> fis_fis_EFDis_List = new ArrayList<Double>();
	private ArrayList<Double> fis_fis_PMHDis_List = new ArrayList<Double>();
	private ArrayList<Double> fis_fis_WLASSim_List = new ArrayList<Double>();
	private ArrayList<Double> numRules_List = new ArrayList<Double>();
	private ArrayList<Double> deathIndex_List = new ArrayList<Double>();
	private ArrayList<Double> bornIndex_List = new ArrayList<Double>();
	private ArrayList<Double> numBorn_List = new ArrayList<Double>();
	private ArrayList<Double> numDeath_List = new ArrayList<Double>();
	
	private double maxEvo	= 0;
	private double maxMem 	= 0;
	private double maxMD	= 0;
	private double maxEF	= 0;
	private double maxPMH	= 0;
	private double maxWLAS	= 0;
	private int    maxNumR 	= 0;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public GlobalMonitorFunctions(Session session_P)
	{
		session = session_P;
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void updateGlobalMemFunctions(double[][] data, FuzzyRuleCollection frCol)
	{
		lastGen++;
		
		addEntropy(data, frCol);
		addGlobalMem(data, frCol);
		evoHis.updateHistogramm(data, frCol);
		addNumRules(frCol);		
		
		if(frCol.getGeneration() > 0)
		{
			addFisFisDis(frCol, session.getFuzzyRuleCollectionAt(frCol.getGeneration()-1));
			addFisFisHLDis(frCol, session.getFuzzyRuleCollectionAt(frCol.getGeneration()-1));
			addFisFisPMHDis(frCol, session.getFuzzyRuleCollectionAt(frCol.getGeneration()-1));
			addFisFisWLASSim(frCol, session.getFuzzyRuleCollectionAt(frCol.getGeneration()-1));
			addBornDeathIndex(frCol, session.getFuzzyRuleCollectionAt(frCol.getGeneration()-1));
			
		}
		else
		{
			fis_fis_Dis_List.add(0.0);
			fis_fis_EFDis_List.add(0.0);
			fis_fis_PMHDis_List.add(0.0);
			fis_fis_WLASSim_List.add(0.0);
			addBornDeathIndex(frCol,frCol);
		}
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################
	
	private void addEntropy(double[][] data, FuzzyRuleCollection frCol)
	{
		double[] p = new double[frCol.collectionSize()];
		
//		double normQuotient = 0;
//		
		double[][] normMembership_A = new double[data.length][];
//		
//		
		for(int i=0; i<data.length; i++)
		{	
			normMembership_A[i] = frCol.getNormalizedMemberships(data[i]);
		}
		
		double entropy = 0;
		for(int j=0; j<p.length; j++)
		{

			for(int i=0; i<data.length; i++)
			{
				p[j] = p[j] + normMembership_A[i][j];
			}
			
			p[j] = p[j]/data.length;
System.out.println("p "+p[j]);			
			if(p[j]>0) entropy = entropy - (p[j] * Math.log(p[j]));
		}
System.out.println("entropy: "+entropy);		
		entropy = entropy / Math.log(p.length);

		entropy2_List.add(entropy);
		if(maxEvo < entropy) maxEvo = entropy;
	}
	
	// with non normalized memberships
	private void addEntropy3(double[][] data, FuzzyRuleCollection frCol)
	{
		double[] p = new double[frCol.collectionSize()];
		
		double quotient = 0;
		
		for(int i=0; i<p.length; i++)
		{
			for(int j=0; j<data.length; j++)
			{
				quotient = quotient + frCol.getFuzzyRuleAt(i).getMembership(data[j]);
			}
		}
		
		double entropy = 0;

		for(int j=0; j<p.length; j++)
		{
			for(int i=0; i<data.length; i++)
			{
				p[j] = p[j] + frCol.getFuzzyRuleAt(j).getMembership(data[i]);
			}
			
			p[j] = p[j]/quotient;
			
			entropy = entropy - (p[j] * Math.log(p[j]));
		}
		
		entropy = entropy / Math.log(p.length);

		entropy2_List.add(entropy);
		if(maxEvo < entropy) maxEvo = entropy;
	}
	
	private void addGlobalMem(double[][] data, FuzzyRuleCollection frCol)
	{
		double memSum = 0;
		double maxMem_l;
		double currMem = 0;
		
		for(int i=0; i<data.length; i++)
		{		
			maxMem_l = Double.MIN_VALUE;
			
			for(FuzzyRule fr: frCol.getFrList())
			{
//				memSum = memSum + fr.getMembership(data[i]);
				currMem = fr.getMembership(data[i]);
				if(maxMem_l < currMem) maxMem_l = currMem;
			}
			
			memSum = memSum + maxMem_l;
		}	
		memSum = memSum/data.length;
		
		globalMem_List.add(new Double(memSum));
		
		if(maxMem < globalMem_List.get(globalMem_List.size() - 1)) maxMem = globalMem_List.get(globalMem_List.size() - 1);
	}
	
	private void addFisFisDis(FuzzyRuleCollection frCol_1, FuzzyRuleCollection frCol_2)
	{
		fis_fis_Dis_List.add(FuzzyRuleCollection.compWLDistance(frCol_1, frCol_2, 0, session.getAttr_Range()));
		if(maxMD < fis_fis_Dis_List.get(fis_fis_Dis_List.size() - 1)) maxMD = fis_fis_Dis_List.get(fis_fis_Dis_List.size() - 1);
	}
	
	private void addFisFisHLDis(FuzzyRuleCollection frCol_1, FuzzyRuleCollection frCol_2)
	{
		fis_fis_EFDis_List.add(FuzzyRuleCollection.compWLDistance(frCol_1, frCol_2, 2, session.getAttr_Range()));
		if(maxEF < fis_fis_EFDis_List.get(fis_fis_EFDis_List.size() - 1)) maxEF = fis_fis_EFDis_List.get(fis_fis_EFDis_List.size() - 1);
	}
	
	private void addFisFisWLASSim(FuzzyRuleCollection frCol_1, FuzzyRuleCollection frCol_2)
	{
		fis_fis_WLASSim_List.add(FuzzyRuleCollection.compWLAngleSim(frCol_1, frCol_2, 0, session.getAttr_Range()));
		if(maxWLAS < fis_fis_WLASSim_List.get(fis_fis_WLASSim_List.size() - 1)) maxWLAS = fis_fis_WLASSim_List.get(fis_fis_WLASSim_List.size() - 1);
	}
	
	private void addFisFisPMHDis(FuzzyRuleCollection frCol_1, FuzzyRuleCollection frCol_2)
	{
		double[][][] partition_AAA = FuzzyRuleCollection.createPartitions1(frCol_1, frCol_2);
		
		try {
			fis_fis_PMHDis_List.add(PartitionMeasure.compSimHuell(partition_AAA[0], partition_AAA[1]));
			if(maxPMH < fis_fis_PMHDis_List.get(fis_fis_PMHDis_List.size() - 1)) maxPMH = fis_fis_PMHDis_List.get(fis_fis_PMHDis_List.size() - 1);
		} catch (DataFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	private void addNumRules(FuzzyRuleCollection frCol_1)
	{
		numRules_List.add(new Double(frCol_1.collectionSize()));
		if(maxNumR < frCol_1.collectionSize()) maxNumR = frCol_1.collectionSize();
	}
	
	private void addBornDeathIndex(FuzzyRuleCollection frCol_new, FuzzyRuleCollection frCol_old)
	{
		int countDeath = 0;
		int countBorn = 0;
		
		for(FuzzyRule fr: frCol_old.getFrList())
		{
			if(session.getRuleInfoAt(fr.getId()).getDeath() > 0)
			{
				countDeath++;
			}
		}
		for(FuzzyRule fr: frCol_new.getFrList())
		{
			if(session.getRuleInfoAt(fr.getId()).getBirth() == session.getLastPopIndex())
			{
				countBorn++;
			}
		}
		
		numBorn_List.add((double)countBorn);
		numDeath_List.add((double)countDeath);
		
		bornIndex_List.add(countBorn/(double)(countDeath+countBorn));
		deathIndex_List.add(countDeath/(double)(countDeath+countBorn));
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################

	public ArrayList<Double> getEntropy2_List() {
		return entropy2_List;
	}
	
	public ArrayList<Double> getGlobalMem_List() {
		return globalMem_List;
	}

	public ArrayList<Double> getFis_fis_Dis_List() {
		return fis_fis_Dis_List;
	}

	public void setFis_fis_Dis_List(ArrayList<Double> fis_fis_Dis_List) {
		this.fis_fis_Dis_List = fis_fis_Dis_List;
	}

	public ArrayList<Double> getFis_fis_EFDis_List() {
		return fis_fis_EFDis_List;
	}

	public void setFis_fis_EFDis_List(ArrayList<Double> fis_fis_EFDis_List) {
		this.fis_fis_EFDis_List = fis_fis_EFDis_List;
	}

	public ArrayList<Double> getFis_fis_PMHDis_List() {
		return fis_fis_PMHDis_List;
	}

	public void setFis_fis_PMHDis_List(ArrayList<Double> fis_fis_PMHDis_List) {
		this.fis_fis_PMHDis_List = fis_fis_PMHDis_List;
	}

	public ArrayList<Double> getFis_fis_WLASSim_List() {
		return fis_fis_WLASSim_List;
	}

	public void setFis_fis_WLASSim_List(ArrayList<Double> fis_fis_WLASSim_List) {
		this.fis_fis_WLASSim_List = fis_fis_WLASSim_List;
	}
	
	public ArrayList<Double> getEntropy_List()
	{
		return this.evoHis.getEntropy_List();
	}
	
	public ArrayList<Double> getNumRules_List() {
		return numRules_List;
	}

	public void setNumRules_List(ArrayList<Double> numRules_List) {
		this.numRules_List = numRules_List;
	}

	public ArrayList<Double> getDeathIndex_List() {
		return deathIndex_List;
	}

	public void setDeathIndex_List(ArrayList<Double> deathIndex_List) {
		this.deathIndex_List = deathIndex_List;
	}

	public ArrayList<Double> getBornIndex_List() {
		return bornIndex_List;
	}

	public void setBornIndex_List(ArrayList<Double> bornIndex_List) {
		this.bornIndex_List = bornIndex_List;
	}

	public ArrayList<Double> getNumBorn_List() {
		return numBorn_List;
	}

	public void setNumBorn_List(ArrayList<Double> numBorn_List) {
		this.numBorn_List = numBorn_List;
	}

	public ArrayList<Double> getNumDeath_List() {
		return numDeath_List;
	}

	public void setNumDeath_List(ArrayList<Double> numDeath_List) {
		this.numDeath_List = numDeath_List;
	}

	public int getLastGen() {
		return lastGen;
	}

	public void incGen()
	{
		lastGen++;
	}

	public double getMaxMem() {
		return maxMem;
	}

	public double getMaxMD() {
		return maxMD;
	}

	public double getMaxEF() {
		return maxEF;
	}

	public double getMaxPMH() {
		return maxPMH;
	}

	public double getMaxWLAS() {
		return maxWLAS;
	}
	
	public double getMaxEvo() {
		return evoHis.getMaxEvo();
	}
	
	public double getMaxEvo2() {
		return maxEvo;
	}

	public int getMaxNumR() {
		return maxNumR;
	}

	public void setMaxNumR(int maxNumR) {
		this.maxNumR = maxNumR;
	}

}
