package guiCore;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.util.TreeSet;

import javax.swing.JPanel;
import javax.swing.JScrollPane;

import listener.CleverMouseListener;
import listener.MouseInteractionObject;

import draw.DrawObject;
import draw.DrawPanel;
import draw.RuleInfo;

public abstract class RuleVisualizer extends DrawObject implements MouseInteractionObject{
	
	protected Session session;
	protected DrawPanel drawPanel;
	protected JPanel mainPanel;
	protected JScrollPane scrollPane;
	protected FuzzyRuleCollection frCol;
	
	protected CleverMouseListener cm_Listener = new CleverMouseListener();
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public RuleVisualizer(Session session_P, DrawPanel drawPanel_P)
	{
		session = session_P;
		drawPanel = drawPanel_P;
		drawPanel.addDrawObject(this);
		
		this.drawPanel.addMouseListener(cm_Listener);
		this.drawPanel.addMouseMotionListener(cm_Listener);
		
		scrollPane = new JScrollPane(drawPanel);
		
		mainPanel  = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(scrollPane, BorderLayout.CENTER);
	}
		
	//###############################################################################################
	// abstract Methods
	//###############################################################################################
	
	protected abstract void updateVisualisation();
	protected abstract void invokeDrawMethods(Graphics2D g2D);
	protected abstract void setIDList(TreeSet<Integer> ID_List);
	
	//###############################################################################################
	// Override Methods DrawObject
	//###############################################################################################
	
	@Override
	public void draw(Graphics2D g2D) 
	{	
		this.refreshCollection(session.getCurrPopulation());
		updateVisualisation();	
		invokeDrawMethods(g2D);
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void repaintDrawPanel()
	{			
		this.drawPanel.repaint();
	}

	public Component getComponent()
	{
		return this.mainPanel;
	}
	
	public static int[] computeVisibleRules(int ruleIntervall, JPanel panel, Session session)
	{		
		Rectangle visibleRect = new Rectangle();
		panel.computeVisibleRect(visibleRect);
		
		//number of rule-generations fit in the visibleRect
//		int numRule_vr = visibleRect.width/(ruleIntervall-1);
		
		//first rule-generations to be paint
		int firstRule = visibleRect.x/(ruleIntervall+1);	
//		int lastIndex = firstRule + numRule_vr;
		if(ruleIntervall < 2) ruleIntervall = 2;
		int lastIndex = (visibleRect.x + visibleRect.width)/(ruleIntervall-1);
		
//		if(firstRule > 1) firstRule = firstRule;

		if(lastIndex > session.getNewestCollectionIndex())
		{
			lastIndex = session.getNewestCollectionIndex();
		}	
		return new int[]{firstRule, lastIndex};	
	}
	
	public int getCurr_t()
	{
		return session.getCurrPopulationIndex();
	}
	
	public RuleInfo getRuleInfoAt(int index)
	{
		return session.getRuleInfoAt(index);
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################
	
	protected void refreshCollection(FuzzyRuleCollection frCol_P)
	{
		this.frCol = frCol_P;

	}
}
