package guiCore;

import io.ReaderUtil;

import java.awt.Color;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import java.util.TreeMap;
import java.util.TreeSet;

import draw.RuleInfo;

import math.Metric;

import exception.AlreadyInitializedException;
import fuzzy.FuzzyRule;


public class Session {
	
	private FISVis sc;
	
	private String sessionURL;
	
	private ColorGenerator colorGen = new ColorGenerator();
	
	//general fuzzyRule Infos
	private int fr_premiseSize = -1;
	private int fr_conclusionSize = -1;
	
	private double timeGlobal_max_RuleDis = 0;
	
	private int maxID = -1;
	
	private double[] protoDatapoint;
	private double[][] data;
	private ArrayList<Integer> dataCount = new ArrayList<Integer>();
	private ArrayList<Integer> dataAllCount =new ArrayList<Integer>();

	//min and max Distance within a rule history
	private final ArrayList<Double> family_max_RuleDis = new ArrayList<Double>();
	private final ArrayList<Double> family_min_RuleDis = new ArrayList<Double>();
	//min and max EF-similarity within a rule history
	private final ArrayList<Double> family_max_RuleEFsim = new ArrayList<Double>();
	private final ArrayList<Double> family_min_RuleEFsim = new ArrayList<Double>();
	
	private double[][] 	attr_Range;
//	private double[]	out_Range;
	private double[][]  out_Range;
	private double[]	refOut_Range;
	
	private double[][] 	customAttr_Range;
	private double[][] 	autoCustomAttr_Range;

	private boolean[]  	customFlag_A;
	
	private int[] 		autoRangeID;
	private boolean 	autoCustomFlag = false;
	
	private boolean onlyLiving = false;
	
	private final GlobalMonitorFunctions globalMemFunc = new GlobalMonitorFunctions(this);
	private final ArrayList<RuleInfo> ruleInfo_List = new ArrayList<RuleInfo>();
	
	//save fuzzyrule history
	private final ArrayList<FuzzyRuleCollection> frPop_List = new ArrayList<FuzzyRuleCollection>();
	
	
	private LinkedList<Integer> chronologicQueue = new LinkedList<Integer>();
	private int currPopulationIndex = -1;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public Session(FISVis sc_P)
	{
		this.sc = sc_P;
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	public void updateGlobalMonitor()
	{
		if(frPop_List.size()-2 == globalMemFunc.getLastGen())
		{
			globalMemFunc.updateGlobalMemFunctions(data, frPop_List.get(frPop_List.size()-1));
		}
		else
		{
			System.err.println("Wrong Gen to Histogramm - Session 73");
		}
			sc.updateMonitor();		
	}
	
	public void setFuzzyRuleInfos(int premiseSize_P, int conclusionSize_P) throws AlreadyInitializedException
																					, IllegalArgumentException
	{
		if(premiseSize_P <= 0) throw new IllegalArgumentException();
		if(conclusionSize_P <=0 ) throw new IllegalArgumentException();
				
		if(fr_premiseSize < 0)
		{
			this.fr_premiseSize = premiseSize_P;
		}
		else
		{
			throw new AlreadyInitializedException();
		}
		
		if(fr_conclusionSize < 0)
		{
			this.fr_conclusionSize = conclusionSize_P;
		}
		else
		{
			throw new AlreadyInitializedException();
		}			
	}
	
	public void addFuzzyRulePopulation(FuzzyRuleCollection frPop_P, int max_ID_P)
	{
		FuzzyRuleCollection frPop = new FuzzyRuleCollection(frPop_P.getFrList(), frPop_P.getGeneration());
		
		frPop_List.add(frPop);
		
		int curr_max_ID = 0;
		
//		if(max_ID_P < 0)
//		{
			curr_max_ID = findMaxID(frPop.getFrList());
//		}
//		else
//		{
//			curr_max_ID = max_ID_P;
//		}
		
		if(maxID < curr_max_ID)
		{
			for(int i = maxID+1; i <= curr_max_ID; i++)
			{
				ruleInfo_List.add(new RuleInfo(colorGen.calculateRuleColor(i)	
						, frPop_P.getGeneration()
						, frPop_P.getFuzzyRule(i).getAncestors()));
			}
			maxID = curr_max_ID;
		}
		
		this.updateFamily_RuleDis();
		this.checkForDeath();
		
		double curr_Max = Session.calcMaxRuleDis(frPop);
		if(curr_Max > timeGlobal_max_RuleDis)
		{
			timeGlobal_max_RuleDis = curr_Max;
		}
				
		chronologicQueue.add(new Integer(frPop_List.size()-1));
		
		if(this.autoCustomFlag)
		{
			this.autoCustomizeRange(null);
		}
	}
	
	public void pollChronologicQueue()
	{
		if(chronologicQueue.size() > 1) // 1 because Queue should never be empty, generations in queue are updated already!!
		{
			chronologicQueue.poll();
		}		
	}
	
	public void repaintRV()
	{
		sc.repaintRV();
	}
	
	public int getNewestCollectionIndex()
	{
		Integer reValue = null;
		
		try {
			reValue = chronologicQueue.getFirst();
		}
		catch(NoSuchElementException e){
			
			reValue = frPop_List.size()-1;
		}

		if(reValue != null)
		{
			return reValue;
		}
		else
		{
			return frPop_List.size()-1;
		}
	}
	
	public FuzzyRuleCollection getCurrPopulation() 
	{
		if(currPopulationIndex >= 0)
		{
			return frPop_List.get(currPopulationIndex);
		}
		else
		{
			return frPop_List.get(getNewestCollectionIndex());
		}
	}
	
	public int getCurrPopulationIndex() 
	{
		if(currPopulationIndex >= 0)
		{
			return currPopulationIndex;
		}
		else
		{
			return getNewestCollectionIndex();
		}
	}
	
	public int getCurrPopulationSize()
	{
		return getCurrPopulation().collectionSize();
	}
	
	public int getLastPopIndex()
	{
		return frPop_List.size()-1;
	}
	
	public void incrementCurrIndex()
	{
		if(currPopulationIndex >= 0
				&& currPopulationIndex < this.getLastPopIndex())
		{
			this.setCurrPopulationIndex(currPopulationIndex+1);
		}
		else
		{
			if(this.getCurrPopulationIndex() < this.getLastPopIndex())
			{
				this.setCurrPopulationIndex(this.getCurrPopulationIndex()+1);
			}		
		}
	}
	
	public FuzzyRule getFuzzyRule(int id, int generation)
	{
		for(FuzzyRule fr: frPop_List.get(generation).getFrList())
		{
			if(fr.getId() == id) return fr;
		}
		
		return null;
	}
	
	public void decrementCurrIndex()
	{
		if(currPopulationIndex > 0)
		{
			this.setCurrPopulationIndex(currPopulationIndex-1);
		}
		else
		{
			if(this.getCurrPopulationIndex() > 0)
			{
				this.setCurrPopulationIndex(this.getCurrPopulationIndex()-1);
			}		
		}
	}
	
	public FuzzyRuleCollection getFuzzyRuleCollectionAt(int index)
	{
		return frPop_List.get(index);
	}
	
	public void initCustomAttr_Range(double[][] attr_range)
	{
		customAttr_Range = new double[attr_range.length][attr_range[0].length];
		customFlag_A = new boolean[attr_range.length];
		
		for(int i=0; i<customAttr_Range.length; i++)
		{
			customFlag_A[i] = false;
			
			for(int j=0; j<customAttr_Range[i].length; j++)
			{
				customAttr_Range[i][j] = attr_range[i][j];
			}
		}
	}
	
	public void autoCustomizeRange(int[] id_A)
	{
		if(id_A == null)
		{
			id_A = this.autoRangeID;
		}
		else
		{
			this.autoRangeID = id_A;
		}
		
		double[][] range = new double[this.fr_premiseSize][2];
		
		for(int i=0; i<range.length; i++)
		{
				range[i][0] = Double.MAX_VALUE;
				range[i][1] = Double.NEGATIVE_INFINITY;
		}
		
		double currMin;
		double currMax;
		
		for(int i=0; i<id_A.length; i++)
		{
			for(int g=0; g < this.getNumGenerations(); g++)
			{
				for(int p = 0; p< this.fr_premiseSize; p++)
				{
					try{
						currMin = this.frPop_List.get(g).getFuzzyRule(id_A[i]).getCenterFromAttr(p)
									- this.frPop_List.get(g).getFuzzyRule(id_A[i]).getWidthFromAttr(p);
						currMax = this.frPop_List.get(g).getFuzzyRule(id_A[i]).getCenterFromAttr(p)
								+ this.frPop_List.get(g).getFuzzyRule(id_A[i]).getWidthFromAttr(p);
						
						if(currMin < range[p][0]) range[p][0] = currMin;

						if(currMax > range[p][1]) range[p][1] = currMax;
					}
					catch(NullPointerException e){};
				}			
			}
		}
		
		autoCustomAttr_Range = range;
		this.repaintRV();

	}
		
	public static double calcMaxRuleDis(FuzzyRuleCollection frc_P)
	{		
		double max_Dis = 0;
		double curr_Dis = 0;
		for(int i=0; i<frc_P.getFrList().size(); i++)
		{
			for(int j=0; j<frc_P.getFrList().size(); j++)
			{
				curr_Dis = calcDistance(frc_P.getFrList().get(i), frc_P.getFrList().get(j));

				if(curr_Dis > max_Dis)
				{
					max_Dis = curr_Dis;
				}
			}
		}
		return max_Dis;
	}
	
	public static double calcDistance(FuzzyRule fr_1, FuzzyRule fr_2)
	{
		double ret_Value = Metric.euclidean(fr_1.getCenter(), fr_2.getCenter());
			
		return ret_Value;
	}
	
//	public static double calcDistanceMD(FuzzyRule fr_1, FuzzyRule fr_2)
//	{
//		double ret_Value = 0;
//		
//		double c = (fr_1.getMembership(fr_2.getCenter()) + fr_2.getMembership(fr_1.getCenter()))/2;
//		
//		double s = 0;
//		
//		for(int i=0; i < fr_1.getCenter().length; i++)
//		{
//			s = s 
//				+ Math.abs(fr_1.getFs_List().get(i).getValue(x))
//				+ Math.abs();
//		}
//		
//		return ret_Value;
//	}
	
	public static double calcDistance(FuzzyRule fr_1, FuzzyRule fr_2, boolean[] enabledAttr_A)
	{
		double ret_Value = Metric.euclidean(fr_1.getCenter(), fr_2.getCenter(), enabledAttr_A);
			
		return ret_Value;
	}
	
	public void openFindRulesDialog(RuleVisualizer rv)
	{
		new FindRules_Dialog(this, sc.getSessionFrame(), rv);
	}
	
	public TreeSet<Integer> findAllRulesInRange(int[] attributes, double[][] range, int t_Start, int t_End)
	{		
		TreeSet<Integer> ret_List = new TreeSet<Integer>();
		
		boolean isINRange = true;
		
		int lastIndex = this.getNewestCollectionIndex();
		
		if(t_End > lastIndex) t_End = lastIndex;
		for(int i=t_Start; i<=t_End; i++)
		{
			for(FuzzyRule fr: this.getFuzzyRuleCollectionAt(i).getFrList())
			{
				isINRange = true;
				for(int j=0; j<attributes.length; j++)
				{
					if(!(fr.getCenterFromAttr(attributes[j]) >= range[attributes[j]][0]
							&& fr.getCenterFromAttr(attributes[j]) < range[attributes[j]][1]))
					{
						isINRange = false;
					}
				}	
				if(isINRange) ret_List.add(fr.getId());
			}
		}	
		return ret_List;
	}
	
	public void initOutRange(FuzzyRule fr)
	{
		out_Range = new double[fr.getConclustionSize()][2];
		refOut_Range = new double[2];
		
		for(int i=0; i<fr.getConclustionSize(); i++)
		{
			out_Range[i][0] = fr.getConclusionCharAt(i);
			out_Range[i][1] = fr.getConclusionCharAt(i);
		}
		
		refOut_Range[0] = fr.getFc().getValue(fr.getCenter());
		refOut_Range[1] = fr.getFc().getValue(fr.getCenter());
	}
	
	public void updateOutRange()
	{
		for(FuzzyRule fr: this.frPop_List.get(frPop_List.size()-1).getFrList())
		{
			for(int i=0; i<this.fr_conclusionSize; i++)
			{
				if(out_Range[i][0] > fr.getConclusionCharAt(i))
				{
					out_Range[i][0] = fr.getConclusionCharAt(i);
				}
				else if(out_Range[i][1] < fr.getConclusionCharAt(i))
				{
					out_Range[i][1] = fr.getConclusionCharAt(i);
				}				
			}
			
			if(refOut_Range[0] > fr.getFc().getValue(fr.getCenter()))
			{
				refOut_Range[0] = fr.getFc().getValue(fr.getCenter());
			}
			else if(refOut_Range[1] < fr.getFc().getValue(fr.getCenter()))
			{
				refOut_Range[1] = fr.getFc().getValue(fr.getCenter());
			}
		}	
	}
	
	//###############################################################################################
	// private Methods
	//###############################################################################################
	
	private int findMaxID(ArrayList<FuzzyRule> fr_List)
	{
		int maxID = Integer.MIN_VALUE;
		
		for(FuzzyRule fr: fr_List)
		{
			if(maxID < fr.getId())
			{
				maxID = fr.getId();
			}
		}
		
		return maxID;
	}
	
	private void updateFamily_RuleDis()
	{
		if(family_max_RuleDis.size() <= maxID)
		{
			for(int i = family_max_RuleDis.size(); i < maxID+1; i++)
			{
				family_max_RuleDis.add(new Double(0));
				family_min_RuleDis.add(new Double(0));
				
				family_max_RuleEFsim.add(new Double(0));
				family_min_RuleEFsim.add(new Double(1));
			}
		}
		
		double lastDis = 0;
		
		for(int i=0; i<= maxID; i++)
		{
			boolean isInPop_2 = false;
			boolean isInPop_1 = false;
						
			if(frPop_List.size() > 1)
			{
				
				for(int j=0; j<frPop_List.get(frPop_List.size()-1).getFrList().size(); j++)
				{
					
					if(frPop_List
							.get(frPop_List.size()-1)
							.getFuzzyRuleAt(j)
							.getId() == i)
					{
						isInPop_1 = true;
					}
				}
				for(int j=0; j<frPop_List.get(frPop_List.size()-2).getFrList().size(); j++)
				{
					if(frPop_List.get(frPop_List.size()-2).getFuzzyRuleAt(j).getId() == i)
					{
						isInPop_2 = true;
					}
				}
				
				if(isInPop_1 && isInPop_2)
				{
					lastDis = calcDistance(frPop_List.get(frPop_List.size()-2).getFuzzyRule(i)
							,frPop_List.get(frPop_List.size()-1).getFuzzyRule(i));
					
					frPop_List.get(frPop_List.size()-1).getFuzzyRule(i).lastDis = lastDis;
					if(lastDis > family_max_RuleDis.get(i))
					{
						family_max_RuleDis.set(i, lastDis);
					}
					if(lastDis < family_min_RuleDis.get(i))
					{
						family_min_RuleDis.set(i, lastDis);
					}
					
					lastDis = FuzzyRule.compSimilarityErrorFunc(frPop_List.get(frPop_List.size()-2).getFuzzyRule(i)
							,frPop_List.get(frPop_List.size()-1).getFuzzyRule(i), null);

					if(lastDis > family_max_RuleEFsim.get(i))
					{
						family_max_RuleEFsim.set(i, lastDis);
					}
					if(lastDis < family_min_RuleEFsim.get(i))
					{
						family_min_RuleEFsim.set(i, lastDis);
					}
				}
			}
		}
	}
	
	private void checkForDeath()
	{
		boolean isDead = true;
		for(int i=0; i< ruleInfo_List.size(); i++)
		{
			if(ruleInfo_List.get(i).getDeath() < 1)
			{
				isDead = true;
				for(FuzzyRule fr: frPop_List.get(frPop_List.size()-1).getFrList())
				{
					if(fr.getId() == i)
					{
						isDead = false;
						break;
					}
				}
			
				if(isDead) ruleInfo_List.get(i).setDeath(frPop_List.size()-1);
			}		
		}
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public RuleInfo getRuleInfoAt(int index)
	{
		return ruleInfo_List.get(index);
	}

	public void setCurrPopulationIndex(int currPopulationIndex) 
	{
		this.currPopulationIndex = currPopulationIndex;
	
		if(currPopulationIndex >= 0) sc.stopStreamV();		
	}

	public int getFr_premiseSize() {
		return fr_premiseSize;
	}

	public int getFr_conclusionSize() {
		return fr_conclusionSize;
	}

	public double getTimeGlobal_max_RuleDis() {
		return timeGlobal_max_RuleDis;
	}

	public ArrayList<Double> getFamily_max_RuleDis() {
		return family_max_RuleDis;
	}
	
	public ArrayList<Double> getFamily_min_RuleDis() {
		return family_min_RuleDis;
	}
	
	public ArrayList<Double> getFamily_max_RuleEFsim() {
		return family_max_RuleEFsim;
	}
	
	public ArrayList<Double> getFamily_min_RuleEFsim() {
		return family_min_RuleEFsim;
	}
	
	public int getNumGenerations()
	{
		return frPop_List.size();
	}
	public int getMaxID() {
		return maxID;
	}

	public ColorGenerator getColorGen() {
		return colorGen;
	}

	public double[] getAttr_RangeAt(int index) 
	{
		if(customFlag_A[index])
		{
			return customAttr_Range[index];
		}
		else if(autoCustomFlag)
		{
			return autoCustomAttr_Range[index];
		}
		else
		{
			return attr_Range[index];
		}		
	}
	
	public double[][] getAttr_Range() 
	{
		return attr_Range;		
	}

	public void setAttr_Range(double[][] attr_Range) 
	{
		this.attr_Range = attr_Range;
	}

	public double[][] getOut_Range() {
		return out_Range;
	}
//
//	public void setOut_Range(double[] out_Range) {
//		this.out_Range = out_Range;
//	}
	
	public double[] getRefOut_Range() {
		return refOut_Range;
	}

	public void setRefOut_Range(double[] refOut_Range) {
		this.refOut_Range = refOut_Range;
	}

	public double[][] getCustomAttr_Range() {
		return customAttr_Range;
	}

	public void setCustomAttrFlagAt(int index, boolean bool)
	{
		this.customFlag_A[index] = bool;
	}
	
	public boolean getCustomAttrFlagAt(int index)
	{
		return this.customFlag_A[index];
	}

	public boolean isAutoCustomFlag() {
		return autoCustomFlag;
	}

	public void setAutoCustomFlag(boolean autoCustomFlag) 
	{
		this.autoCustomizeRange(null);
		this.autoCustomFlag = autoCustomFlag;
	}

	public boolean isOnlyLiving() {
		return onlyLiving;
	}

	public void setOnlyLiving(boolean onlyLiving) {
		
		this.onlyLiving = onlyLiving;
		this.repaintRV();
	}

	public GlobalMonitorFunctions getGlobalMemFunc() {
		return globalMemFunc;
	}

	public double[] getProtoDatapoint() {
		return protoDatapoint;
	}

	public void setProtoDatapoint(double[] protoDatapoint) {
		this.protoDatapoint = protoDatapoint;
	}
	
	public double[][] getCurrData()
	{
		return ReaderUtil.readData(sessionURL, this.getCurrPopulationIndex());
	}

	public double[][] getData()
	{
		return data;
	}

	public void setData(double[][] data, int numData) 
	{
		this.data = data;
		
		this.dataAllCount.add(numData);
		
		if(dataCount.size() > 0)
		{
			this.dataCount.add(data.length+this.dataCount.get(dataCount.size()-1));			
		}
		else
		{
			this.dataCount.add(data.length);
		}
	}
	
	public int getDataCountAt(int index)
	{
		try
		{
			return dataCount.get(index);
		}
		catch(IndexOutOfBoundsException aob)
		{
			return -1;
		}
	}
	
	public int getDataAllCountAt(int index)
	{
		try
		{
			return dataAllCount.get(index);
		}
		catch(IndexOutOfBoundsException aob)
		{
			return -1;
		}
	}

	public String getSessionURL() {
		return sessionURL;
	}

	public void setSessionURL(String sessionURL) {
		this.sessionURL = sessionURL;
	}
}
