package guiUtil;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JWindow;

public class DragHandler implements MouseListener, MouseMotionListener{
	
	private Object[] data;
	private DragSource dragSource;
	private ArrayList<DropTarget> drop_List = new ArrayList<DropTarget>();
	
	private DropWindow dw;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public DragHandler(DragSource dragSource_P)
	{
		dragSource = dragSource_P;
		dragSource.addMouseListener(this);
		dragSource.addMouseMotionListener(this);
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void addDropTarget(DropTarget dt)
	{
		drop_List.add(dt);
	}
	
	//###############################################################################################
	// Override MouseListener
	//###############################################################################################

	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(MouseEvent me) {
		
		if(me.getButton() == MouseEvent.BUTTON1)
		{
			data = dragSource.getData(me);
			
			if(dw != null) dw.dispose(); 
			 
			if(data != null)
			{
				
				dw = new DropWindow(dragSource, me);
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent me) {
		// TODO Auto-generated method stub
		
		for(DropTarget dt: drop_List)
		{
			if(dt.hit(me))
			{
				dt.drop(data);
			}
		}
		
		if(dw != null)
		{
			dw.dispose();
			dw = null;
		}
	}
	
	//###############################################################################################
	// Override MouseMotionListener
	//###############################################################################################

	@Override
	public void mouseDragged(MouseEvent me) 
	{
		if(dw != null)
		{
			dw.setLocation(me.getXOnScreen()
					, me.getYOnScreen());
		}		
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	
	//###############################################################################################
	// Inner Classes
	//###############################################################################################
	
	private class DropWindow extends JWindow{		
		
		private DropFramePanel dfp;
		
		//###############################################################################################
		// Constructor
		//###############################################################################################
		
		private DropWindow(DragSource ds_P, MouseEvent me)
		{
			dfp = new DropFramePanel(ds_P, me);
			
			Container cp = this.getContentPane();
			cp.setLayout(new BorderLayout());
			cp.add(dfp, BorderLayout.CENTER);
			
			this.setSize(30*data.length, 30);
			this.setLocation(me.getXOnScreen(), me.getYOnScreen());
			this.setBackground(new Color(1f,1f,1f, 0));
			this.setOpacity(0.5f);
			this.setVisible(true);
		}	
	}
	
	private class DropFramePanel extends JPanel
	{
		private DragSource ds;
		private MouseEvent me;
		
		//###############################################################################################
		// Constructor
		//###############################################################################################
		
		public DropFramePanel(DragSource ds_P, MouseEvent me_P)
		{
			this.ds = ds_P;
			this.me = me_P;
			
			this.setBackground(new Color(1,1,1,0));
		}
		
		//###############################################################################################
		// public Methods
		//###############################################################################################
		
		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);
			
			//upcast
			Graphics2D g2d = (Graphics2D)g;
			
			ds.drawDrop(g2d, me, data);
		}
	}

}
