package guiUtil;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.util.ArrayList;

import javax.swing.JWindow;

import listener.CleverMouseListener;

import draw.DrawObject;
import draw.DrawPanel;

public class Interaction extends JWindow implements MouseListener, MouseMotionListener{
	
	private final DrawPanel dp = new DrawPanel();
	
	private final DrawInteraction drawI = new DrawInteraction();
	private InteractionMove iMove = null;
	
	private InteractionField[] if_List;
	private int x;
	private int y;
	private int width;
	private int height;
	private int radius;
	private Object[] data;
	
	private static Interaction instance = new Interaction(null, null, 0, 0, 0, null);
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	private Interaction(InteractionField[] if_List_P, InteractionMove iMove_P, int x_P, int y_P, int radius_P, Object[] data_P)
	{
		if_List = if_List_P;
		iMove	= iMove_P;
		x		= x_P;
		y		= y_P;
		data	= data_P;
		
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		
		Container cp = this.getContentPane();
		
		dp.setBackground(new Color(1f,1f,1f,0.05f));
		dp.addDrawObject(drawI);
	
		cp.setLayout(new BorderLayout());
		cp.add(dp, BorderLayout.CENTER);
		
		this.setBackground(new Color(1f, 1f, 1f, 0f));
	}
	
	public static Interaction getInstance(InteractionField[] if_List_P, InteractionMove iMove_P, int x_P, int y_P, int radius_P, Object[] data_P)
	{
		instance.if_List 	= if_List_P;
		instance.iMove		= iMove_P;
		instance.radius 	= radius_P;
		
		if(instance.if_List.length > 2)
		{
			double alpha = (Math.PI/instance.if_List.length);
			double r = instance.radius * Math.sin(alpha);
			double a = r * Math.sqrt(2)/2;
			
			instance.width 	= (instance.radius + (int)a) * 2;
			instance.height = (instance.radius + (int)a) * 2;
		}
		else
		{
			instance.width 	= instance.radius * 3;
			instance.height = instance.radius * 3;
		}
		
		instance.x		= x_P-instance.width/2;
		instance.y		= y_P-instance.height/2;

		
		instance.data	= data_P;
		
		instance.setSize(instance.width, instance.height);
		instance.setLocation(instance.x, instance.y);
		instance.setVisible(true);
	
		return instance;
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public InteractionField getIfAt(int index)
	{
		if(index > 5) return null;
		
		return if_List[index];
	}
	
	private class DrawInteraction extends DrawObject{
	
		//###############################################################################################
		// Override DrawObject Methods
		//###############################################################################################
		
		@Override
		public void draw(Graphics2D g2d) 
		{
			double a = 0;
			double alpha = Math.PI/if_List.length*2;
			double r = 0;
			
			if(if_List.length > 2)
			{		
				r = instance.radius * Math.sin(alpha/2);
				a = r * Math.sqrt(2);
			}
			else
			{
				a = instance.radius;
			}
			
			AffineTransform identity = g2d.getTransform();
							
			for(int i=0; i<if_List.length; i++)
			{									
				g2d.translate(instance.width/2, instance.height/2);
				g2d.rotate(alpha*i + Math.PI);
				g2d.translate(0, instance.radius);
				g2d.rotate(-alpha*i + Math.PI);
				if_List[i].paintArea(g2d, dp, (int)a);
				
				g2d.setTransform(identity);
			}
			
		}
	}
	
	//###############################################################################################
	// Override MouseListener Methods
	//###############################################################################################

	@Override
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent me) {
		// TODO Auto-generated method stub
		if(iMove != null) iMove.action(me);
		this.dispose();
	}

	@Override
	public void mousePressed(MouseEvent me) 
	{
		double a = instance.height/2 - me.getY();
		double c = Point.distance(instance.width/2
				, instance.height/2
				, me.getX()
				, me.getY());
		
		double alpha = Math.acos((a/c));
		
		if(me.getX() < instance.width/2)
		{
			alpha = Math.PI * 2 - alpha;
		}
		
		alpha = alpha + Math.PI / instance.if_List.length;
		
		if(alpha > Math.PI * 2)
		{
			alpha = alpha - Math.PI * 2;
		}
		
		int index = (int)(alpha/(Math.PI / instance.if_List.length * 2));
		
		try{
			if_List[index].action(me, MouseEvent.MOUSE_PRESSED);
		}
		catch(ArrayIndexOutOfBoundsException e)
		{
			//Do Nothing
		}
		
		instance.dispose();
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	//###############################################################################################
	// Override MouseMotionListener Methods
	//###############################################################################################

	@Override
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseMoved(MouseEvent me) {

		if(iMove != null) iMove.action(me);
		
	}


	
}
