package listener;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

public class CleverMouseListener implements MouseMotionListener, MouseListener{
	
	public static final int CLICKED 	= 0;
	public static final int ENTERED 	= 1;
	public static final int EXITED  	= 2;
	public static final int PRESSED		= 3;
	public static final int RELEASED 	= 4;
	public static final int DRAGGED 	= 5;
	public static final int MOVED 		= 6;
	
	private ArrayList<MouseInteractionObject> clicked_List;
	private ArrayList<MouseInteractionObject> entered_List;
	private ArrayList<MouseInteractionObject> exited_List;
	private ArrayList<MouseInteractionObject> pressed_List;
	private ArrayList<MouseInteractionObject> released_List;
	private ArrayList<MouseInteractionObject> dragged_List;
	private ArrayList<MouseInteractionObject> moved_List;

	//###############################################################################################
	// Override Methods MouseListener
	//###############################################################################################
	
	@Override
	public void mouseClicked(MouseEvent me) {

		if(clicked_List != null)
		for(MouseInteractionObject mio: clicked_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, CLICKED)) break;
			}
		}		
	}

	@Override
	public void mouseEntered(MouseEvent me) {
		
		if(entered_List != null)
		for(MouseInteractionObject mio: entered_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, ENTERED)) break;
			}
		}		
	}

	@Override
	public void mouseExited(MouseEvent me) {

		if(exited_List != null)
		for(MouseInteractionObject mio: exited_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, EXITED)) break;
			}
		}		
	}

	@Override
	public void mousePressed(MouseEvent me) {

		if(pressed_List != null)
		for(MouseInteractionObject mio: pressed_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, PRESSED)) break;
			}
		}		
	}

	@Override
	public void mouseReleased(MouseEvent me) {

		if(released_List != null)
		for(MouseInteractionObject mio: released_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, RELEASED)) break;
			}
		}		
	}
	
	//###############################################################################################
	// Override Methods MouseMotionListener
	//###############################################################################################

	@Override
	public void mouseDragged(MouseEvent me) {
		
		if(dragged_List != null)
		for(MouseInteractionObject mio: dragged_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, DRAGGED)) break;
			}
		}		
	}

	@Override
	public void mouseMoved(MouseEvent me) {

		if(moved_List != null)
		for(MouseInteractionObject mio: moved_List)
		{
			if(mio.hitObject(me))
			{
				if(mio.action(me, MOVED)) break;
			}
		}		
	}
	
	//###############################################################################################
	// public Methods
	//###############################################################################################
	
	public void addToClickedList(MouseInteractionObject mio)
	{
		if(this.clicked_List == null) clicked_List = new ArrayList<MouseInteractionObject>();
		
		clicked_List.add(mio);
	}
	
	public void addToEnteredList(MouseInteractionObject mio)
	{
		if(this.entered_List == null) entered_List = new ArrayList<MouseInteractionObject>();
		
		entered_List.add(mio);
	}
	
	public void addToExitedList(MouseInteractionObject mio)
	{
		if(this.exited_List == null) exited_List = new ArrayList<MouseInteractionObject>();
		
		exited_List.add(mio);
	}
	
	public void addToPressedList(MouseInteractionObject mio)
	{
		if(this.pressed_List == null) pressed_List = new ArrayList<MouseInteractionObject>();
		
		pressed_List.add(mio);
	}
	
	public void addToReleasedList(MouseInteractionObject mio)
	{
		if(this.released_List == null) released_List = new ArrayList<MouseInteractionObject>();
		
		released_List.add(mio);
	}
	
	public void addToDraggedList(MouseInteractionObject mio)
	{
		if(this.dragged_List == null) dragged_List = new ArrayList<MouseInteractionObject>();
		
		dragged_List.add(mio);
	}
	
	public void addToMovedList(MouseInteractionObject mio)
	{
		if(this.moved_List == null) moved_List = new ArrayList<MouseInteractionObject>();
		
		moved_List.add(mio);
	}
	
	public boolean removeFromClickedList(MouseInteractionObject mio)
	{
		if(this.clicked_List == null)
		{
			return false;
		}
		
		return clicked_List.remove(mio);
	}
	
	public boolean removeFromEnteredList(MouseInteractionObject mio)
	{
		if(this.entered_List == null)
		{
			return false;
		}
		
		return entered_List.remove(mio);
	}
	
	public boolean removeFromExitedList(MouseInteractionObject mio)
	{
		if(this.exited_List == null)
		{
			return false;
		}
		
		return exited_List.remove(mio);
	}
	
	public boolean removeFromPressedList(MouseInteractionObject mio)
	{
		if(this.pressed_List == null)
		{
			return false;
		}
		
		return pressed_List.remove(mio);
	}
	
	public boolean removeFromReleasedList(MouseInteractionObject mio)
	{
		if(this.released_List == null)
		{
			return false;
		}
		
		return released_List.remove(mio);
	}
	
	public boolean removeFromDraggedList(MouseInteractionObject mio)
	{
		if(this.dragged_List == null)
		{
			return false;
		}
		
		return dragged_List.remove(mio);
	}
	
	public boolean removeFromMovedList(MouseInteractionObject mio)
	{
		if(this.moved_List == null)
		{
			return false;
		}
		
		return moved_List.remove(mio);
	}
	
	public void removeFromClickedList(int index)
	{
		if(this.clicked_List != null)
		{
			clicked_List.remove(index);
		}
	}
	
	public void removeFromEnteredList(int index)
	{
		if(this.entered_List == null)
		{
			entered_List.remove(index);
		}
	}
	
	public void removeFromExitedList(int index)
	{
		if(this.exited_List == null)
		{
			exited_List.remove(index);
		}
	}
	
	public void removeFromPressedList(int index)
	{
		if(this.pressed_List == null)
		{
			pressed_List.remove(index);
		}
	}
	
	public void removeFromReleasedList(int index)
	{
		if(this.released_List == null)
		{
			released_List.remove(index);
		}
	}
	
	public void removeFromDraggedList(int index)
	{
		if(this.dragged_List == null)
		{
			dragged_List.remove(index);
		}
	}
	
	public void removeFromMovedList(int index)
	{
		if(this.moved_List == null)
		{
			moved_List.remove(index);
		}
	}
	
	public void clearMovedList()
	{
		if(moved_List != null) moved_List.clear();		
	}
}
