package stub;

import fuzzy.FuzzyConclusion;
import fuzzy.FuzzyRule;
import fuzzy.FuzzySet;
import guiCore.FISVis;

import io.IOfis;
import io.RuleCollectionReader;

import java.io.File;
import java.util.ArrayList;

public class RuleStreamReader {
	
	private FISVis sessionControler;
	private RuleCollectionReader rcr;
	private File file;
	
	public RuleStreamReader(File file)
	{
		this.file = file;
		rcr = new RuleCollectionReader(file);
		
		sessionControler = new FISVis();
		startStream();
	}

	
	public void startStream()
	{
		start();
		
		boolean continueStream = true;
		while(continueStream)
		{
			continueStream = update();
			
			try 
			{
				Thread.sleep(2000);
			} 
			catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}
	
	private boolean start()
	{
		IOfis iofis = rcr.readNext();
		
		//create FuzzyRule
		ArrayList<FuzzyRule> fr_List = new ArrayList<FuzzyRule>();
		
		FuzzyRule fr = null;
		
		for(int r=0; r < iofis.mean_A.length; r++)
		{
			//create FuzzySets antecedent
			ArrayList<FuzzySet> fs_List = new ArrayList<FuzzySet>();
			
			FuzzySet fs = null;
			
			for(int f=0; f<iofis.mean_A[r].length; f++)
			{
				fs = FuzzySet.createFuzzySet(FuzzySet.GAUSS, new double[]{iofis.mean_A[r][f], iofis.width_A[r][f]});
				fs_List.add(fs);
			}
			//matlab to java indexing
			if(iofis.ancestors[r][0] > -1)
				iofis.ancestors[r][0] = iofis.ancestors[r][0] - 1;
			if(iofis.ancestors[r][1] > -1)
				iofis.ancestors[r][1] = iofis.ancestors[r][1] - 1;
			
			// rule_to_ID[r]-1 because Matlab starts with 1, java with 0
			fr = new FuzzyRule(iofis.rule_to_ID[r]-1
					,0
					,iofis.ancestors[r]
					,fs_List
					,new String("A")
					,FuzzyConclusion.createFuzzyConclusion(FuzzyConclusion.LINEAR_FUNCTION, iofis.out_A[r]));
			
			fr_List.add(fr);
		}
		
		//inform sessionControler
		sessionControler.startStoredSession(fr_List, iofis.attr_range, iofis.out_range, iofis.max_ID, iofis.data, iofis.numDataAll, file.getParent());
		
		return true;
	}
	
	private boolean update()
	{
		IOfis iofis = rcr.readNext();
		
		if(iofis == null) return false;
		
		//create FuzzyRule
		ArrayList<FuzzyRule> fr_List = new ArrayList<FuzzyRule>();
		
		FuzzyRule fr = null;
		
		for(int r=0; r < iofis.mean_A.length; r++)
		{
			//create FuzzySets antecedent
			ArrayList<FuzzySet> fs_List = new ArrayList<FuzzySet>();
			
			FuzzySet fs = null;
			
			for(int f=0; f<iofis.mean_A[r].length; f++)
			{
				fs = FuzzySet.createFuzzySet(FuzzySet.GAUSS, new double[]{iofis.mean_A[r][f], iofis.width_A[r][f]});
				fs_List.add(fs);
			}
			//matlab to java indexing
			if(iofis.ancestors[r][0] > -1)
				iofis.ancestors[r][0] = iofis.ancestors[r][0] - 1;
			if(iofis.ancestors[r][1] > -1)
				iofis.ancestors[r][1] = iofis.ancestors[r][1] - 1;
			
			// rule_to_ID[r]-1 because Matlab starts with 1, java with 0
			fr = new FuzzyRule(iofis.rule_to_ID[r]-1
					,0
					,iofis.ancestors[r]
					,fs_List
					,new String("A")
					,FuzzyConclusion.createFuzzyConclusion(FuzzyConclusion.LINEAR_FUNCTION, iofis.out_A[r]));
			
			fr_List.add(fr);
		}
		
		//inform sessionControler
		sessionControler.updateStoredSession(fr_List, iofis.attr_range, iofis.out_range, iofis.max_ID, iofis.data, iofis.numDataAll);
		
		return true;
	}
}
