package fuzzy;
//Immutable Class
public class FuzzyConclusionLinear extends FuzzyConclusion{
	
	private final double[] weights;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public FuzzyConclusionLinear(double[] weights)
	{
		super(FuzzyConclusion.LINEAR_FUNCTION);
		
		this.weights = new double[weights.length];
		for(int i=0; i<weights.length; i++)
		{
			this.weights[i] = weights[i];
		}
	}
	
	//###############################################################################################
	// Override Methods
	//###############################################################################################

	@Override
	public double getValue(double[] datapoint) 
	{
		double retValue = 0;
		
		for(int i=0; i < weights.length-1; i++)
		{
			retValue = retValue + weights[i] * datapoint[i]; 
		}
		
		retValue = retValue + weights[weights.length-1];
		
		return retValue;
	}
	
	@Override
	public FuzzyConclusion copy() {
		
		return new FuzzyConclusionLinear(this.weights);
	}

	@Override
	public FuzzyConclusion newVariant(double[] variations) {
		
		return new FuzzyConclusionLinear(variations);
	}

	@Override
	public double[] getCharacteristics() {
		
		double[] newWeights = new double[weights.length];
		
		for(int i=0; i<newWeights.length; i++)
		{
			newWeights[i] = weights[i];
		}

		return newWeights;
	}
}
