package fuzzy;

import org.apache.commons.math3.special.Erf;

public class FuzzyMath {
	
	public static double directedWidth(double[] direction, double[] sigma_A, boolean[] enabledAttr_A)
	{
		double retValue = 0;
		
		if(enabledAttr_A == null)
		{
			for(int i=0; i < sigma_A.length; i++)
			{
				retValue = retValue + (direction[i] * direction[i]) / (sigma_A[i] * sigma_A[i]);
			}		
		}
		else
		{
			for(int i=0; i < sigma_A.length; i++)
			{
				if(enabledAttr_A[i])
				{
					retValue = retValue + (direction[i] * direction[i]) / (sigma_A[i] * sigma_A[i]);
				}				
			}					
		}
		
		retValue = Math.sqrt(1 / retValue);
		
		return retValue;
	}
	
	public static double[] normGaussianIntersection(double center1, double center2, double sigma1, double sigma2)
	{
		double[] ret_A = new double[2];
		
		double a = center1 * sigma2 + center2 * sigma1;
		double b = sigma2 + sigma1;
		
		ret_A[0] = a/b;
		
		a = center1 * sigma2 - center2 * sigma1;
		b = sigma2 - sigma1;
		
		if(b != 0)
		{
			ret_A[1] = a/b;
			
			if(ret_A[0] > ret_A[1])
			{
				double help = ret_A[0];
				ret_A[0] = ret_A[1];
				ret_A[1] = help;
			}
		}
		else
		{
			ret_A[1] = Double.NaN;
		}
		
		return ret_A;
	}
	
	public static double compNormalizedGenerealErrorFunc(double x, double center, double sigma)
	{
		double retValue = sigma * Math.sqrt(2 * Math.PI)/2 * 
								 (1 + Erf.erf((x - center) / (sigma * Math.sqrt(2))));	
		return retValue;
	}

}
