package fuzzy;

import java.util.ArrayList;

import math.Metric;

//Immutable Class
public class FuzzyRule {
	
	private final int id;
	private final int timeStamp;
	private final int[] ancestors;
	
	public double lastDis = 0; //TODO entfernen, nur frs debugging
	

	private final ArrayList<FuzzySet> fs_List = new ArrayList<FuzzySet>();
	private final String rpn_Rule;		//reverse polish notation
	private final FuzzyConclusion fc;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public FuzzyRule(int id, int timeStamp, int[] ancestors, ArrayList<FuzzySet> fs_List, String rpn_Rule, FuzzyConclusion fc)
	{
		this.id = id;
		this.timeStamp = timeStamp;
		
		this.ancestors = new int[ancestors.length];
		for(int i=0; i<ancestors.length; i++)
		{
			this.ancestors[i] = ancestors[i];
		}
					
		this.rpn_Rule = new String(rpn_Rule);
		this.fc = fc.copy();
		
		for(FuzzySet fs: fs_List)
		{
			this.fs_List.add(fs.copy());
		}
	}
	
	//###############################################################################################
	// Methods
	//###############################################################################################
	
	public double getMembership(double[] datapoint)
	{
		double membership = 1;
		
		for(int i=0; i<fs_List.size(); i++)
		{
			membership = membership * fs_List.get(i).getValue(datapoint[i]);
		}
		
		return membership;
	}
	
	public double getConsequensValue(double[] datapoint)
	{		
		return fc.getValue(datapoint);
	}
	
	public int numFuzzySets()
	{
		return fs_List.size();
	}
	
	public int getConclustionSize()
	{
		return fc.getCharacteristics().length;
	}
	
	public double getConclusionCharAt(int index)
	{
		return fc.getCharacteristics()[index];
	}
	
	//###############################################################################################
	// static distance Methods
	//###############################################################################################
	
	public static double compSimilarityMD2(FuzzyRule fr_1, FuzzyRule fr_2, boolean[] enabledAttr_A)
	{			
		double s = Double.MAX_VALUE;
		
		if(enabledAttr_A == null)
		{
			for(int i=0; i < fr_1.getCenter().length; i++)
			{			

				s =	Math.min(s, (1-(Math.abs(fr_1.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)
						+ fr_2.getWidthFromAttr(i))
								- fr_2.getFsAt(i).getValue(fr_2.getCenterFromAttr(i)
						+ fr_1.getWidthFromAttr(i)))))
						* (fr_1.getFsAt(i).getValue(fr_2.getCenterFromAttr(i))
								+ fr_2.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)))/2);
					
				s =	Math.min(s, (1- (Math.abs(fr_1.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)
						+ fr_2.getWidthFromAttr(i))
								- fr_2.getFsAt(i).getValue(fr_2.getCenterFromAttr(i)
						+ fr_1.getWidthFromAttr(i)))))
						* (fr_1.getFsAt(i).getValue(fr_2.getCenterFromAttr(i))
								+fr_2.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)))/2);
				
			}
		}
		else
		{
			for(int i=0; i < fr_1.getCenter().length; i++)
			{			
				if(enabledAttr_A[i])
				{
					s =	Math.min(s, (1-(Math.abs(fr_1.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)
							+ fr_2.getWidthFromAttr(i))
									- fr_2.getFsAt(i).getValue(fr_2.getCenterFromAttr(i)
							+ fr_1.getWidthFromAttr(i)))))
							* (fr_1.getFsAt(i).getValue(fr_2.getCenterFromAttr(i))
									+ fr_2.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)))/2);
					
					s =	Math.min(s, (1- (Math.abs(fr_1.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)
							+ fr_2.getWidthFromAttr(i))
									- fr_2.getFsAt(i).getValue(fr_2.getCenterFromAttr(i)
							+ fr_1.getWidthFromAttr(i)))))
							* (fr_1.getFsAt(i).getValue(fr_2.getCenterFromAttr(i))
									+fr_2.getFsAt(i).getValue(fr_1.getCenterFromAttr(i)))/2);
				}
			}	
		}
		return Math.sqrt(s);
	}
	public static double compSimilarityHL(FuzzyRule fr_1
			, FuzzyRule fr_2
			, boolean[] enabledAttr_A
			, double[][] attrRange_A
			, int numSamples)
{
	double sum_min = 0;
	double sum_A = 0;
	double sum_B = 0;
	
	double msf_degree1;
	double msf_degree2;
	
	double overlap_degree1 = Double.MAX_VALUE;
	double overlap_degree2 = Double.MAX_VALUE;
	
	if(enabledAttr_A == null)
	{
		for(int i = 0; i < attrRange_A.length; i ++)
		{
			sum_min = 0;
			sum_A = 0;
			sum_B = 0;		
			
			double k = attrRange_A[i][0];
			
			for(int j=0; j<numSamples; j++)
			{
				msf_degree1 = fr_1.getFsAt(i).getValue(k);
				msf_degree2 = fr_2.getFsAt(i).getValue(k);
				
				k += (attrRange_A[i][1] - attrRange_A[i][0])/numSamples;
				
				sum_min = sum_min + (msf_degree1 < msf_degree2 ? msf_degree1 : msf_degree2);
				sum_A = sum_A + msf_degree1;
				sum_B = sum_B + msf_degree2;
			}
			
			if(overlap_degree1 > sum_min/sum_A) overlap_degree1 = sum_min/sum_A;
			if(overlap_degree2 > sum_min/sum_B) overlap_degree2 = sum_min/sum_B;
		}
	}
	else
	{
		for(int i = 0; i < attrRange_A.length; i ++)
		{
		if(enabledAttr_A[i])
		{
			sum_min = 0;
			sum_A = 0;
			sum_B = 0;		
			
			double k = attrRange_A[i][0];
			
			for(int j=0; j<numSamples; j++)
			{
				msf_degree1 = fr_1.getFsAt(i).getValue(k);
				msf_degree2 = fr_2.getFsAt(i).getValue(k);
				
				k += (attrRange_A[i][1] - attrRange_A[i][0])/numSamples;
				
				sum_min = sum_min + (msf_degree1 < msf_degree2 ? msf_degree1 : msf_degree2);
				sum_A = sum_A + msf_degree1;
				sum_B = sum_B + msf_degree2;
			}
			
			if(overlap_degree1 > sum_min/sum_A) overlap_degree1 = sum_min/sum_A;
			if(overlap_degree2 > sum_min/sum_B) overlap_degree2 = sum_min/sum_B;
			}
		}
	}
	
	if(overlap_degree1 < overlap_degree2)
	{
		return overlap_degree1;
	}
	
	return overlap_degree2;	
}
	/**
	 * 
	 * @param fr_1
	 * @param fr_2
	 * @param enabledAttr_A
	 * @param attrRange_A
	 * @param numSamples
	 * @return
	 */
	public static double compOverlapHL(FuzzyRule fr_1
											, FuzzyRule fr_2
											, boolean[] enabledAttr_A
											, double[][] attrRange_A
											, int numSamples)
	{
		double sum_min = 0;
		double sum_A = 0;
		double sum_B = 0;
		
		double msf_degree1;
		double msf_degree2;
		
		double overlap_degree1 = Double.MAX_VALUE;
		double overlap_degree2 = Double.MAX_VALUE;
		
		if(enabledAttr_A == null)
		{
			for(int i = 0; i < attrRange_A.length; i ++)
			{
				sum_min = 0;
				sum_A = 0;
				sum_B = 0;		
				
				double k = attrRange_A[i][0];
				
				for(int j=0; j<numSamples; j++)
				{
					msf_degree1 = fr_1.getFsAt(i).getValue(k);
					msf_degree2 = fr_2.getFsAt(i).getValue(k);
					
					k += (attrRange_A[i][1] - attrRange_A[i][0])/numSamples;
					
					sum_min = sum_min + (msf_degree1 < msf_degree2 ? msf_degree1 : msf_degree2);
					sum_A = sum_A + msf_degree1;
					sum_B = sum_B + msf_degree2;
				}
				
				if(overlap_degree1 > sum_min/sum_A) overlap_degree1 = sum_min/sum_A;
				if(overlap_degree2 > sum_min/sum_B) overlap_degree2 = sum_min/sum_B;
			}
		}
		else
		{
			for(int i = 0; i < attrRange_A.length; i ++)
			{
				if(enabledAttr_A[i])
				{
					sum_min = 0;
					sum_A = 0;
					sum_B = 0;		
					
					double k = attrRange_A[i][0];
					
					for(int j=0; j<numSamples; j++)
					{
						msf_degree1 = fr_1.getFsAt(i).getValue(k);
						msf_degree2 = fr_2.getFsAt(i).getValue(k);
						
						k += (attrRange_A[i][1] - attrRange_A[i][0])/numSamples;
						
						sum_min = sum_min + (msf_degree1 < msf_degree2 ? msf_degree1 : msf_degree2);
						sum_A = sum_A + msf_degree1;
						sum_B = sum_B + msf_degree2;
					}
					
					if(overlap_degree1 > sum_min/sum_A) overlap_degree1 = sum_min/sum_A;
					if(overlap_degree2 > sum_min/sum_B) overlap_degree2 = sum_min/sum_B;
				}
			}
		}
		
		if(overlap_degree1 > overlap_degree2)
		{
			return overlap_degree1;
		}
		
		return overlap_degree2;	
	}
	
	public static double compSimilarityErrorFunc(FuzzyRule fr_1, FuzzyRule fr_2, boolean[] enabledAttr_A)
	{
		double retValue = 1;
		
		double[] intersect;
		double area_1;
		double area_2;
		double area_b;
		double help_1;
		
		
		for(int i=0; i<fr_1.numFuzzySets(); i++)
		{
			if(enabledAttr_A == null
					|| enabledAttr_A[i])
			{
				//compute intersection point
				intersect = FuzzyMath.normGaussianIntersection(
													fr_1.getCenterFromAttr(i)
													, fr_2.getCenterFromAttr(i)
													, fr_1.getWidthFromAttr(i)
													, fr_2.getWidthFromAttr(i));
				
				// 0, 0 for x and center because here x == center
				area_1 = fr_1.getWidthFromAttr(i) * Math.sqrt(Math.PI * 2) ;
				area_2 = fr_2.getWidthFromAttr(i) * Math.sqrt(Math.PI * 2) ;
				
				if(intersect[0] == intersect[1])
				{				
					if(area_1 > area_2)
					{
						help_1 = area_2 / area_1;
					}
					else
					{
						help_1 = area_1 / area_2;
					}
					
					if(help_1 < retValue) retValue = help_1;
				}
				else if(new Double(intersect[1]).isNaN()
							&& fr_1.getCenterFromAttr(i) != fr_2.getCenterFromAttr(i))
				{
					if(intersect[0] > fr_1.getCenterFromAttr(i))
					{
						area_b = area_1
									- FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
									, fr_1.getCenterFromAttr(i)
									, fr_1.getWidthFromAttr(i));
					
						area_b = area_b
								+ FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
								, fr_2.getCenterFromAttr(i)
								, fr_2.getWidthFromAttr(i));
					}
					else
					{
						area_b = FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
								, fr_1.getCenterFromAttr(i)
								, fr_1.getWidthFromAttr(i));
				
						area_b = area_b
								+ area_2 
								- FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
								, fr_2.getCenterFromAttr(i)
								, fr_2.getWidthFromAttr(i));
					}
					
					if(area_1 > area_2)
					{
						help_1 = area_b / area_1;
					}
					else
					{
						help_1 = area_b / area_2;
					}
					
					if(help_1 < retValue) retValue = help_1;
	
				}
				else if(!new Double(intersect[1]).isNaN())
				{
					if(fr_1.getWidthFromAttr(i) < fr_2.getWidthFromAttr(i))
					{
						area_b = FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
								, fr_1.getCenterFromAttr(i)
								, fr_1.getWidthFromAttr(i))
								
								+
								
								area_1
									- FuzzyMath.compNormalizedGenerealErrorFunc(intersect[1]
									, fr_1.getCenterFromAttr(i)
									, fr_1.getWidthFromAttr(i));
						
						area_b = area_b
									+ (FuzzyMath.compNormalizedGenerealErrorFunc(intersect[1]
											, fr_2.getCenterFromAttr(i)
											, fr_2.getWidthFromAttr(i))
											
										-
										
										FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
											, fr_2.getCenterFromAttr(i)
											, fr_2.getWidthFromAttr(i)));
					}
					else 
					{
						area_b = FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
								, fr_2.getCenterFromAttr(i)
								, fr_2.getWidthFromAttr(i))
								
								+
								
								area_2
									- FuzzyMath.compNormalizedGenerealErrorFunc(intersect[1]
									, fr_2.getCenterFromAttr(i)
									, fr_2.getWidthFromAttr(i));
						
						area_b = area_b
									+ (FuzzyMath.compNormalizedGenerealErrorFunc(intersect[1]
											, fr_1.getCenterFromAttr(i)
											, fr_1.getWidthFromAttr(i))
											
										-
										
										FuzzyMath.compNormalizedGenerealErrorFunc(intersect[0]
											, fr_1.getCenterFromAttr(i)
											, fr_1.getWidthFromAttr(i)));
					}
					
					if(area_1 > area_2)
					{
						help_1 = area_b / area_1;
					}
					else
					{
						help_1 = area_b / area_2;
					}
					
					if(help_1 < retValue) retValue = help_1;
				}
			}
		}
		
		return retValue;
	}
	
	public static double compConsequensSim(FuzzyRule fr_1, FuzzyRule fr_2)
	{
		double angle = compConsequensAngle(fr_1, fr_2);
		
		double sim = 0;
			
		if(angle <= Math.PI/2)
		{
			sim = 1 - (2 / Math.PI) * angle;
		}
		else
		{
			sim = (2 / Math.PI) * (angle - Math.PI / 2);
		}
		return sim;    		
	}
	
	public static double compConsequensAngle(FuzzyRule fr_1, FuzzyRule fr_2)
	{
		double sum = 0;		
		double[] weights_1 = fr_1.getFc().getCharacteristics();
		double[] weights_2 = fr_2.getFc().getCharacteristics();
		
		weights_1[weights_1.length - 1] = -1;
		weights_2[weights_2.length - 1] = -1;
		
		for(int i=0; i<weights_1.length; i++)
		{
			sum = sum + weights_1[i] * weights_2[i];
		}
		
		sum = sum / (Metric.norm2(weights_1) * Metric.norm2(weights_2));
		
		if(sum > 1) sum = 1;
		
		double angle = Math.acos(sum);
		return angle;
	}
	
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		
		sb.append("FuzzySets\n");
		for(int i=0; i<fs_List.size(); i++)
		{
			sb.append(fs_List.get(i).toString() + " | ");
		}
		
		return sb.toString();
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public double[] getCenter(){
		
		double[] center = new double[fs_List.size()];
	
		for(int i=0; i<center.length; i++)
		{
			center[i] = fs_List.get(i).getCenter();
		}
		
		return center;
	}
	
	public double getCenterFromAttr(int attr_Index)
	{
		return fs_List.get(attr_Index).getCenter();
	}
	
	public double[] getWidth()
	{
		double[] width = new double[fs_List.size()];
		
		for(int i=0; i<width.length; i++)
		{
			width[i] = fs_List.get(i).getWidth();
		}
		
		return width;
	}
	
	public double getWidthFromAttr(int attr_Index)
	{
		return fs_List.get(attr_Index).getWidth();
	}

	public int getId() {
		return id;
	}

	public int getTimeStamp() {
		return timeStamp;
	}

	public int[] getAncestors() {
		return new int[]{ancestors[0], ancestors[1]};
	}
	
	public FuzzySet getFsAt(int index)
	{
		return fs_List.get(index);
	}

	public ArrayList<FuzzySet> getFs_List_Copy() {
		
		ArrayList<FuzzySet> ret_List = new ArrayList<FuzzySet>();
		
		for(FuzzySet fs: this.fs_List)
		{
			ret_List.add(fs.copy());
		}
		return fs_List;
	}

	public String getRpn_Rule() {
		return new String(rpn_Rule);
	}

	public FuzzyConclusion getFc() {
		return fc.copy();
	}
	
	

}
