package fuzzy;

public abstract class FuzzySet {
	
	public final static int PIECEWISE_LINEAR = 0;
	public final static int GAUSS = 1;
	
	private final int type;
	
	//###############################################################################################
	// Methods
	//###############################################################################################
	
	protected FuzzySet(int type)
	{
		this.type = type;
	}
	
	//###############################################################################################
	// Abstract Methods
	//###############################################################################################
	
	public abstract double getCenter();
	public abstract double getWidth();
	public abstract double[][] getCore();
	public abstract double[][] getAlphaCut(double alpha);
	public abstract double getValue(double x);
	public abstract FuzzySet copy();
	public abstract FuzzySet newVariant(double[] variations);
	public abstract double[] getCharacteristics();
	
	//###############################################################################################
	// Methods
	//###############################################################################################
	
	public static FuzzySet createFuzzySet(int type, double[] characteristics)
	{
		FuzzySet ret_Fs = null;
		
		switch(type)
		{
			case PIECEWISE_LINEAR: break;
		
			case GAUSS: ret_Fs = new FuzzySetGauss(characteristics);
					break;
		}
		
		return ret_Fs;
	}

	public int getType(){		
		return type;
	}
}
