package guiCore;

import java.util.ArrayList;

import fuzzy.FuzzyRule;
import fuzzy.PartitionMeasure;

//Immutable
public class FuzzyRuleCollection {
	
	private final ArrayList<FuzzyRule> fr_List = new ArrayList<FuzzyRule>();
	private final int generation;
	
	//###############################################################################################
	// Constructor
	//###############################################################################################
	
	public FuzzyRuleCollection(ArrayList<FuzzyRule> fr_List_P, int generation_P)
	{
		this.generation = generation_P;
		for(FuzzyRule fr: fr_List_P)
		{
			fr_List.add(fr);
		}
	}
	
	//###############################################################################################
	// Methods
	//###############################################################################################
	
	public double getPrediction(double[] datapoint)
	{
		double retValue = 0;
		
		double[] membership_A = getNormalizedMemberships(datapoint);
		
		for(int i=0; i<fr_List.size(); i++)
		{
			retValue = retValue + fr_List.get(i).getConsequensValue(datapoint) * membership_A[i];
		}
		
		return retValue;
	}
	/**
	 * 
	 * @param datapoint
	 * @return
	 */
	public double[] getNormalizedMemberships(double[] datapoint)
	{
		double[] ret_A = new double[fr_List.size()];
		
		double sum = 0;
		
		for(int i=0; i<ret_A.length; i++)
		{
			ret_A[i] = fr_List.get(i).getMembership(datapoint);
			sum = sum + ret_A[i];
		}

		if(sum != 0)
		for(int i=0; i<ret_A.length; i++)
		{
			ret_A[i] = ret_A[i]/sum;
		}
		
		return ret_A;
	}
	
	/**
	 * 
	 * @param frCol_1
	 * @param frCol_2
	 * @param mode
	 * @param attrRange_A
	 * @return
	 */
	public static double compWLDistance(FuzzyRuleCollection frCol_1
											, FuzzyRuleCollection frCol_2
											, int mode
											, double[][] attrRange_A)
	{
		double retValue = 0;
		
		double maxSim 	= 0;
		double[] maxSim_A = new double[frCol_2.getFrList().size()];
		double currSim 	= 0;
		
		for(int j=0; j<frCol_2.getFrList().size(); j++)
		{
			maxSim_A[j] = 0;
		}
		
		
		for(int i=0; i<frCol_1.getFrList().size(); i++)
		{
			for(int j=0; j<frCol_2.getFrList().size(); j++)
			{
				switch(mode)
				{
					case 0:
						currSim = FuzzyRule.compSimilarityMD2(frCol_1.getFuzzyRuleAt(i)
																, frCol_2.getFuzzyRuleAt(j), null);
						break;
					case 1:
						currSim = FuzzyRule.compSimilarityHL(frCol_1.getFuzzyRuleAt(i)
																, frCol_2.getFuzzyRuleAt(j)
																, null
																, attrRange_A
																, 1000);
						break;
					case 2:
						currSim = FuzzyRule.compSimilarityErrorFunc(frCol_1.getFuzzyRuleAt(i)
								, frCol_2.getFuzzyRuleAt(j), null);
						break;
				}				
				if(maxSim < currSim) maxSim = currSim;
				if(maxSim_A[j] < currSim) maxSim_A[j] = currSim;
			}
			
			retValue = retValue + maxSim;
			maxSim = 0;
		}
		
		for(int j=0; j<frCol_2.getFrList().size(); j++)
		{
			retValue = retValue + maxSim_A[j];
		}
		
		retValue = retValue/(frCol_1.getFrList().size() + frCol_2.getFrList().size());
		
		return retValue;
	}
	
	public static double compWLAngleSim(FuzzyRuleCollection frCol_1
											, FuzzyRuleCollection frCol_2
											, int mode
											, double[][] attrRange_A)
	{
		double retValue = 0;
		
		FuzzyRule[][] pairs_AA = compWLPairs(frCol_1
												, frCol_2
												, mode
												, attrRange_A);
		
		for(int i=0; i<pairs_AA.length; i++)
		{
			retValue = retValue + FuzzyRule.compConsequensSim(pairs_AA[i][0], pairs_AA[i][1]);
		}
	
		return retValue / pairs_AA.length;
	}
	
	/**
	 * 
	 * @param frCol_1
	 * @param frCol_2
	 * @param mode
	 * @return
	 */
	public static FuzzyRule[][] compWLPairs(FuzzyRuleCollection frCol_1
											, FuzzyRuleCollection frCol_2
											, int mode
											, double[][] attrRange_A)
	{
		FuzzyRule[][] pairs_AA = new FuzzyRule[frCol_1.collectionSize() + frCol_2.collectionSize()][2];
		
		double maxSim 	= -1;
		double[] maxSim_A = new double[frCol_2.getFrList().size()];
		double currSim 	= 0;
		
		for(int j=0; j<frCol_2.getFrList().size(); j++)
		{
			maxSim_A[j] = -1;
			pairs_AA[frCol_1.collectionSize() + j][0] = frCol_2.getFuzzyRuleAt(j);
		}
		
		
		for(int i=0; i<frCol_1.collectionSize(); i++)
		{
			pairs_AA[i][0] = frCol_1.getFuzzyRuleAt(i);
			
			for(int j=0; j<frCol_2.collectionSize(); j++)
			{
				switch(mode)
				{
					case 0:
						currSim = FuzzyRule.compSimilarityMD2(frCol_1.getFuzzyRuleAt(i)
																, frCol_2.getFuzzyRuleAt(j), null);
						break;
					case 1:
						currSim = FuzzyRule.compSimilarityHL(frCol_1.getFuzzyRuleAt(i)
																, frCol_2.getFuzzyRuleAt(j)
																, null
																, attrRange_A
																, 1000);
						break;
				}

				if(maxSim < currSim)
				{
					maxSim = currSim;
					pairs_AA[i][1] = frCol_2.getFuzzyRuleAt(j);
				}
				if(maxSim_A[j] < currSim)
				{
					maxSim_A[j] = currSim;
					pairs_AA[frCol_1.collectionSize() + j][1] = frCol_1.getFuzzyRuleAt(i);
				}
			}
			
			maxSim = -1;
		}	

		return pairs_AA;
	}
	
	/**
	 * 
	 * @param frCol_1
	 * @param frCol_2
	 * @return
	 */
	public static double[][][] createPartitions1(FuzzyRuleCollection frCol_1, FuzzyRuleCollection frCol_2)
	{
		double[][][] retPart = new double[2][][];
		
		int numFuzzySets = 0;
		try
		{
			numFuzzySets = frCol_1.getFuzzyRuleAt(0).numFuzzySets();
		}
		catch(ArrayIndexOutOfBoundsException e)
		{
			e.printStackTrace();
		}
		
		int numElements = (frCol_1.collectionSize() + frCol_2.collectionSize()) * numFuzzySets * 2;
		retPart[0] = new double[numElements][frCol_1.collectionSize()];
		retPart[1] = new double[numElements][frCol_2.collectionSize()];
		
		double[] 	normMemberships = null;
		double[][]	datapoint		= new double[numElements][numFuzzySets];
		
		//create datapoints
		int eCount = 0;
		for(int f=0; f < numFuzzySets; f++)
		{
			for(int c=0; c<frCol_1.collectionSize(); c++)
			{
				datapoint[eCount] = frCol_1.getFuzzyRuleAt(c).getCenter();
				datapoint[eCount][f] = datapoint[eCount][f] + frCol_1.getFuzzyRuleAt(c).getWidthFromAttr(f);
				
				eCount++;
				
				datapoint[eCount] = frCol_1.getFuzzyRuleAt(c).getCenter();
				datapoint[eCount][f] = datapoint[eCount][f] - frCol_1.getFuzzyRuleAt(c).getWidthFromAttr(f);
				
				eCount++;
			}
			
			for(int c=0; c<frCol_2.collectionSize(); c++)
			{
				datapoint[eCount] = frCol_2.getFuzzyRuleAt(c).getCenter();
				datapoint[eCount][f] = datapoint[eCount][f] + frCol_2.getFuzzyRuleAt(c).getWidthFromAttr(f);
				
				eCount++;
				
				datapoint[eCount] = frCol_2.getFuzzyRuleAt(c).getCenter();
				datapoint[eCount][f] = datapoint[eCount][f] - frCol_2.getFuzzyRuleAt(c).getWidthFromAttr(f);
				
				eCount++;
			}			
		}
		
		for(int e=0; e < numElements; e++)
		{
			normMemberships = frCol_1.getNormalizedMemberships(datapoint[e]);			
			retPart[0][e] =	normMemberships;
			
			normMemberships = frCol_2.getNormalizedMemberships(datapoint[e]);			
			retPart[1][e] =	normMemberships;
			
		}
				
		return retPart;
	}
	
	public static int compWinningRule(double[] datapoint, FuzzyRuleCollection frCol)
	{
		int retID = 0;
		double maxMem = 0;
		double currMem = 0;
		
		for(FuzzyRule fr: frCol.getFrList())
		{
			currMem = fr.getMembership(datapoint);
			
			if(currMem > maxMem)
			{
				maxMem = currMem;
				retID = fr.getId();
			}
		}
		
		return retID;
	}
	
	/**
	 * 
	 * @param id_P
	 * @return
	 */
	public FuzzyRule getFuzzyRule(int id_P)
	{
		FuzzyRule retValue = null;
		
		for(FuzzyRule fr: fr_List)
		{
			if(fr.getId() == id_P)
			{
				retValue = fr;
			}
		}
		
		return retValue;
	}
	
	public FuzzyRule getFuzzyRuleAt(int index)
	{
		return fr_List.get(index);
	}
	
	public int collectionSize()
	{
		return this.fr_List.size();
	}
	
	//###############################################################################################
	// GET and SET
	//###############################################################################################
	
	public ArrayList<FuzzyRule> getFrList()
	{
		ArrayList<FuzzyRule> ret_List = new ArrayList<FuzzyRule>();
		
		for(FuzzyRule fr: fr_List)
		{
			ret_List.add(fr);
		}
		
		return ret_List;
	}

	public int getGeneration() {
		return generation;
	}
}
